//
// Class CPlatforms for dealing with different platforms
//
// Copyright (C) 2000-2004 by Grigori G.Fursin
//
// http://homepages.inf.ed.ac.uk/gfursin
//
// ICSA, School of Informatics,
// University of Edinburgh, UK
//

//Import libraries...
import java.io.*;

public class CPlatforms
{
  //Variables
  int plt_max=0;
  int plt_num=0;

  String f_name="";
  String f_ext="";
  CConfig[] Plt;
 
  //***************************************************************************
  //"Initialization"
  //
  //Input:  cfg1    = configuration object
  //        index1  = index of 'platform' in configuration
  //        f_name1 = beginning of the file name with configurations
  //        f_ext1  = file extension
  //
  //Return: 0  - Ok
  //        >0 - Error
  //

  public int Init(int plt_max1, String f_name1, String f_ext1)
  {
    plt_max=plt_max1;
    f_name=f_name1;
    f_ext=f_ext1;

    Plt=new CConfig[plt_max];
 
    return 0;
  }

  //***************************************************************************
  //"Parse Platform Configuration Files"
  //
  //Return: <0  - Error
  //        >=0 - Identificator 
  //
  public int Parse(int plt_num1)
  {
    int i;

    plt_num=0;

    for (i=0; i<plt_num1; i++)
    {
      Add(-1);

      int misc=Plt[i].Parse();
      if (misc==1) return -3;
      else if (misc==2) return -4;
      else if (misc==3) return -5;
      else if (misc==4) return -6;
      else if (misc==5) return -7;
      else if (misc>0)  return -8;
    }

    return 0;
  }

  //***************************************************************************
  public int GetPlatformsMax()
  {
    return plt_max;
  }

  //***************************************************************************
  public int GetPlatformsNum()
  {
    return plt_num;
  }

  //***************************************************************************
  public CConfig GetConfig(int index)
  {
    return Plt[index];
  }

  //***************************************************************************
  public int Add(int i)
  {
    if (i==-1) i=plt_num;

    if (i<plt_num)
    {
      for (int ii=plt_num+1; ii>i; ii--)
        Plt[ii]=Plt[ii-1];
    }

    //Declaration of class for configuration file parsing
    Plt[i] = new CConfig();

    String FileCFG=f_name+String.valueOf(i)+f_ext;

    if (Plt[i].Init(15, 0, FileCFG, 15)!=0) return -1;
    //               |  |        |   |
    //               |  |        |   |---- Max Number of Array Parameters
    //               |  |        | 
    //               |  |        |-------- Name of File with Parameters
    //               |  |
    //               |  |----------------- Ignore Case
    //               |
    //               |-------------------- Max Number of Parameters

    int misc=0;
  
    misc+=Plt[i].Add("name",      "", 1);  //Param  0
    misc+=Plt[i].Add("processor", "", 1);  //Param  1
    misc+=Plt[i].Add("frequency", "", 1);  //Param  2
    misc+=Plt[i].Add("memory",    "", 1);  //Param  3
    misc+=Plt[i].Add("cache_l0",  "", 1);  //Param  4
    misc+=Plt[i].Add("cache_l1",  "", 1);  //Param  5
    misc+=Plt[i].Add("cache_l2",  "", 1);  //Param  6
    misc+=Plt[i].Add("os",        "", 1);  //Param  7
    misc+=Plt[i].Add("num_opt",   "", 1);  //Param  8
    misc+=Plt[i].Add("time_prec", "", 1);  //Param  9
    misc+=Plt[i].Add("host",      "", 1);  //Param 10
    misc+=Plt[i].Add("path",      "", 1);  //Param 11

    if (misc!=0) return -2;

    plt_num++;

    return 0;
  }

  //***************************************************************************
  public int Remove(int index)
  {
    if (index<0 || index>=plt_num) return -1;

    for (int i=index; i<plt_num-1; i++) Plt[i]=Plt[i+1];

    Plt[plt_num-1]=null;

    plt_num--;

    return 0;
  }
}
