/*
 * MPC Feedback Module 4 for ATOM (analysis)
 *
 * Copyright (C) 2000-2004 by Grigori G. Fursin
 *
 * homepage: http://homepages.inf.ed.ac.uk/gfursin
 *
 * ICSA, School of Informatics,
 * University of Edinburgh, UK
 */

#include <stdio.h>
#include <malloc.h>
#include <assert.h>

static int proc_max=0;
static long pass=0;

static double*        NCalls;
static double*        NAccessFirst;
static double*        NAccessTotal;
static unsigned long* Addr1;
static unsigned long* Addr2;
static char*          BigSpace;
static long*          check_clean;
static double*        ncount;

static int first_run=0;
static long big_space_length=0;

void MPCInit(int proc_max1, int pass1, long big_space_length1)
{
  long x;
  long y;
  long x1;

  proc_max=proc_max1;
  pass=pass1;
  big_space_length=big_space_length1+16;

  x1=proc_max;
  x=(sizeof(double)*x1);
  fprintf(stderr, "ATOM (eos_mpc_fb4): Allocating %u bytes of memory [NCalls]...\n", x);

  NCalls=(double *) calloc(sizeof(double), x1);

  if (NCalls == NULL) 
  {
    fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb4): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<x1; y++)
  {
    NCalls[y]=0;
  }

  x1=proc_max;
  x=(sizeof(double)*x1);
  fprintf(stderr, "ATOM (eos_mpc_fb4): Allocating %u bytes of memory [NAccessFirst]...\n", x);

  NAccessFirst=(double *) calloc(sizeof(double), x1);

  if (NAccessFirst == NULL) 
  {
    fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb4): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<x1; y++)
  {
    NAccessFirst[y]=0;
  }

  x1=proc_max;
  x=(sizeof(double)*x1);
  fprintf(stderr, "ATOM (eos_mpc_fb4): Allocating %u bytes of memory [NAccessTotal]...\n", x);

  NAccessTotal=(double *) calloc(sizeof(double), x1);

  if (NAccessTotal == NULL) 
  {
    fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb4): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<x1; y++)
  {
    NAccessTotal[y]=0;
  }

  x1=proc_max;
  x=(sizeof(long)*x1);
  fprintf(stderr, "ATOM (eos_mpc_fb4): Allocating %u bytes of memory [Addr1]...\n", x);

  Addr1=(unsigned long *) calloc(sizeof(long), x1);

  if (Addr1 == NULL) 
  {
    fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb4): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<x1; y++)
  {
    Addr1[y]=-1;
  }

  x1=proc_max;
  x=(sizeof(long)*x1);
  fprintf(stderr, "ATOM (eos_mpc_fb4): Allocating %u bytes of memory [Addr2]...\n", x);

  Addr2=(unsigned long *) calloc(sizeof(long), x1);

  if (Addr2 == NULL) 
  {
    fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb4): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<x1; y++)
  {
    Addr2[y]=0;
  }

  if (pass>0)
  {
    x1=big_space_length;
    x=(sizeof(char)*x1);
    fprintf(stderr, "ATOM (eos_mpc_fb4): Allocating %u bytes of memory [BigSpace]...\n", x);

    BigSpace=(char *) calloc(sizeof(char), x1);

    if (BigSpace == NULL) 
    {
      fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb4): Can't allocate memory!\n");
      exit(1);
    }

    for (y=0; y<x1; y++)
    {
      BigSpace[y]=0;
    }

    x1=proc_max;
    x=(sizeof(long)*x1);
    fprintf(stderr, "ATOM (eos_mpc_fb4): Allocating %u bytes of memory [Clean Check]...\n", x);

    check_clean=(long *) calloc(sizeof(long), x1);

    if (check_clean == NULL) 
    {
      fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb4): Can't allocate memory!\n");
      exit(1);
    }

    for (y=0; y<x1; y++)
    {
      check_clean[y]=0;
    }
  }

  x1=proc_max;
  x=(sizeof(double)*x1);
  fprintf(stderr, "ATOM (eos_mpc_fb4): Allocating %u bytes of memory [ncount]...\n", x);

  ncount=(double *) calloc(sizeof(double), x1);

  if (ncount == NULL) 
  {
    fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb4): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<x1; y++)
  {
    ncount[y]=0;
  }

  printf("\nATOM (eos_mpc_fb4): Allocated successfully!\n\n",x);
}

void MPCFinish(void)
{
  FILE *f;
  int i=0;
  int j=0;
  int k=0;

  unsigned long addr1=-1;
  unsigned long addr2=0;

  fprintf(stderr, "\nATOM (eos_mpc_fb4): Saving collected data ...\n");

  if (pass==0 && ((f=fopen("_eos_mpc_atom_tmp6a", "wt"))==NULL))
  {
    fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb4): Can't write file!\n");
    exit(1);
  }
  else if (pass>0 && ((f=fopen("_eos_mpc_atom_tmp6c", "wt"))==NULL))
  {
    fprintf(stderr,"\nFatal Error (ATOM, eos_mpc_fb4): Can't write file!\n");
    exit(1);
  }

  for (i=0; i<proc_max; i++)
  {
    if (pass==0)
    {
      fprintf(f, "proc={%3u}, ncalls={%16.0f}, naccessfirst={%16.0f}, naccesstotal={%16.0f}, addr1={%16ld} (0x%08lx), addr2={%16ld} (0x%08lx)\n",
                  i,
                  NCalls[i],
                  NAccessFirst[i],
                  NAccessTotal[i],
                  Addr1[i],
                  Addr1[i],
                  Addr2[i],
                  Addr2[i]);
    }
    else if (pass>0)
    {
      fprintf(f, "proc={%3u}, ncalls={%16.0f}, nsizeaccessfirst={%16.0f}, nsizeaccesstotal={%16.0f}\n",
                  i,
                  NCalls[i],
                  NAccessFirst[i],
                  NAccessTotal[i]);
    }
  }
 
  fclose(f);

  for (i=0; i<proc_max; i++)
  {
    if (Addr1[i]<addr1) addr1=Addr1[i];
    if (Addr2[i]>addr2) addr2=Addr2[i];
  }

  if (pass==0)
  {
    if ((f=fopen("_eos_mpc_atom_tmp6b", "wt"))==NULL)
    {
      printf("\nFatal Error (ATOM, eos_mpc_fb4): Can't write file!\n");
      exit(1);
    }

    fprintf(f, "%16ld\n%16ld\n", addr1, addr2);

    fclose(f);
  }

  printf("ATOM (eos_mpc_fb4): Data saved successfully!\n");

  fprintf(stderr, "ATOM (eos_mpc_fb4): Data saved successfully!\n");
}

void MPCRegAccess(int proc)
{
  NAccessTotal[proc]+=8;
}

void MPCSaveAddr(unsigned long addr, long proc)
{
  if (addr<Addr1[proc]) Addr1[proc]=addr;
  if (addr>Addr2[proc]) Addr2[proc]=addr;
}

void MPCCountProc(int proc)
{
  NCalls[proc]++;

  if (pass>0)
  {
    if (pass>1 && check_clean[proc]==1) ncount[proc]=NAccessTotal[proc]/(NCalls[proc]-1);

    NAccessTotal[proc]+=ncount[proc];
  }

  if (NCalls[proc]==2) NAccessFirst[proc]=NAccessTotal[proc];
}

void MPCBigClean(int proc)
{
  ncount[proc]=0;

  if (check_clean[proc]==0)
  {
    memset(BigSpace, 0, big_space_length);

    if (pass>1 && NCalls[proc]>pass) check_clean[proc]=1;
  }
}

void MPCBigClean1(int proc)
{
  ncount[proc]=0;
}

void MPCBigRegAccess(int proc, unsigned long addr, unsigned long addr1)
{
  if (addr>addr1)
  {
    if (BigSpace[addr-addr1]==0)
    {
      BigSpace[addr-addr1]=1;
      BigSpace[addr+1-addr1]=1;
      BigSpace[addr+2-addr1]=1;
      BigSpace[addr+3-addr1]=1;
      BigSpace[addr+4-addr1]=1;
      BigSpace[addr+5-addr1]=1;
      BigSpace[addr+6-addr1]=1;
      BigSpace[addr+7-addr1]=1;
      ncount[proc]+=8;
    }
  }
}
