/*
 * MPC Feedback Module 4 for FATOM Windows (analysis)
 *
 * Copyright (C) 2000-2004 by Grigori G. Fursin
 *
 * homepage: http://homepages.inf.ed.ac.uk/gfursin
 *
 * ICSA, School of Informatics,
 * University of Edinburgh, UK
 */

#include <stdio.h>
#include <malloc.h>
#include <assert.h>

static int proc_max=0;
static long pass=0;

static double*        NCalls;
static double*        NAccessFirst;
static double*        NAccessTotal;
static unsigned long* Addr1;
static unsigned long* Addr2;
static char*          BigSpace;
static char*          xaddr1;
static char*          xaddr2;
static long*          check_clean;

static int first_run=0;
static long big_space_length=0;

void mpc_init(int proc_max1, int pass1, long big_space_length1)
{
  int y;

  proc_max=proc_max1;
  pass=pass1;
  big_space_length=big_space_length1+16;

  printf("FATOM (eos_mpc_fb4): Allocating %u bytes of memory [NCalls]...\n", proc_max*8);

  NCalls=(double *) calloc(proc_max, 8);

  if (NCalls == NULL) 
  {
    printf("\nFatal Error (FATOM, eos_mpc_fb4): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<proc_max; y++)
  {
    NCalls[y]=0;
  }

  printf("FATOM (eos_mpc_fb4): Allocating %u bytes of memory [NAccessFirst]...\n", proc_max*8);

  NAccessFirst=(double *) calloc(proc_max, 8);

  if (NAccessFirst == NULL) 
  {
    printf("\nFatal Error (FATOM, eos_mpc_fb4): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<proc_max; y++)
  {
    NAccessFirst[y]=0;
  }

  printf("FATOM (eos_mpc_fb4): Allocating %u bytes of memory [NAccessTotal]...\n", proc_max*8);

  NAccessTotal=(double *) calloc(proc_max, 8);

  if (NAccessTotal == NULL) 
  {
    printf("\nFatal Error (FATOM, eos_mpc_fb4): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<proc_max; y++)
  {
    NAccessTotal[y]=0;
  }

  printf("FATOM (eos_mpc_fb4): Allocating %u bytes of memory [Addr1]...\n", proc_max*4);

  Addr1=(long *) calloc(proc_max, 4);

  if (Addr1 == NULL) 
  {
    printf("\nFatal Error (FATOM, eos_mpc_fb4): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<proc_max; y++)
  {
    Addr1[y]=-1;
  }

  printf("FATOM (eos_mpc_fb4): Allocating %u bytes of memory [Addr2]...\n", proc_max*4);

  Addr2=(long *) calloc(proc_max, 4);

  if (Addr2 == NULL) 
  {
    printf("\nFatal Error (FATOM, eos_mpc_fb4): Can't allocate memory!\n");
    exit(1);
  }

  for (y=0; y<proc_max; y++)
  {
    Addr2[y]=0;
  }

  if (pass>0)
  {
    printf("FATOM (eos_mpc_fb4): Allocating %u bytes of memory [BigSpace]...\n", big_space_length);

    BigSpace=(char *) calloc(big_space_length, 1);

    if (BigSpace == NULL) 
    {
      printf("\nFatal Error (FATOM, eos_mpc_fb4): Can't allocate memory!\n");
      exit(1);
    }

    xaddr1=BigSpace;
    xaddr2=xaddr1+big_space_length;

    printf("FATOM (eos_mpc_fb4): Allocating %u bytes of memory [Clean Check]...\n", proc_max*4);

    check_clean=(long *) calloc(proc_max, 4);

    if (check_clean == NULL) 
    {
      printf("\nFatal Error (FATOM, eos_mpc_fb4): Can't allocate memory!\n");
      exit(1);
    }
	
    for (y=0; y<proc_max; y++)
    {
      check_clean[y]=0;
    }
  }

  printf("\nFATOM (eos_mpc_fb4): Allocated successfully!\n\n");
}

int mpc_finish(void)
{
  FILE *f;
  int i=0;
  int j=0;
  int k=0;

  unsigned long addr1=-1;
  unsigned long addr2=0;

  printf("\nFATOM (eos_mpc_fb4): Saving collected data ...\n");

  if (pass==0 && ((f=fopen("_eos_mpc_atom_tmp6a", "wt"))==NULL))
  {
    printf("\nFatal Error (FATOM, eos_mpc_fb4): Can't write file!\n");
    exit(1);
  }
  else if (pass>0 && ((f=fopen("_eos_mpc_atom_tmp6c", "wt"))==NULL))
  {
    printf("\nFatal Error (FATOM, eos_mpc_fb4): Can't write file!\n");
    exit(1);
  }

  for (i=0; i<proc_max; i++)
  {
    if (pass==0)
    {
      fprintf(f, "proc={%3u}, ncalls={%16.0f}, naccessfirst={%16.0f}, naccesstotal={%16.0f}, addr1={%16ld} (0x%08lx), addr2={%16ld} (0x%08lx)\n",
                  i,
                  NCalls[i],
                  NAccessFirst[i],
                  NAccessTotal[i],
                  Addr1[i],
                  Addr1[i],
                  Addr2[i],
                  Addr2[i]);
    }
    else if (pass>0)
    {
      fprintf(f, "proc={%3u}, ncalls={%16.0f}, nsizeaccessfirst={%16.0f}, nsizeaccesstotal={%16.0f}\n",
                  i,
                  NCalls[i],
                  NAccessFirst[i],
                  NAccessTotal[i]);
    }
  }

  fclose(f);

  for (i=0; i<proc_max; i++)
  {
    if (Addr1[i]<addr1) addr1=Addr1[i];
    if (Addr2[i]>addr2) addr2=Addr2[i];
  }

  if (pass==0)
  {
    if ((f=fopen("_eos_mpc_atom_tmp6b", "wt"))==NULL)
    {
      printf("\nFatal Error (FATOM, eos_mpc_fb4): Can't write file!\n");
      exit(1);
    }

    fprintf(f, "addr1={%16ld} (0x%08lx), addr2={%16ld} (0x%08lx)\n",
                addr1, addr1, addr2, addr2);

    fclose(f);
  }

  printf("FATOM (eos_mpc_fb4): Data saved successfully!\n");

  return 0;
}

void eos_save_count(long proc, long data1, long data2)
{
  double na=0;
  double na1=0;
  double na2=0;

  if (first_run==0)
  {
    mpc_init1();
    first_run=1;
  }

  na1=data1;
  na2=data2;
  na=na1*65536*65536+na2;

  if (NCalls[proc]==0)
  {
    NAccessFirst[proc]=na;
    NAccessTotal[proc]=na;
  }
  else
  {
    if (pass>1)
    {
      if (check_clean[proc]==1) na=NAccessTotal[proc]/NCalls[proc];
    }

    NAccessTotal[proc]+=na;
  }

  NCalls[proc]++;
}

void eos_save_addr(long proc, long addr1, long addr2)
{
  if (addr1<Addr1[proc]) Addr1[proc]=addr1;
  if (addr2>Addr2[proc]) Addr2[proc]=addr2;
}

void get_big_addr(char** addr1,
                  char** addr2)
{
  if (first_run==0)
  {
    mpc_init1();
    first_run=1;
  }

  *addr1=xaddr1;
  *addr2=xaddr2;
}

void clean_big_space(long proc)
{
  long y;

  if (check_clean[proc]==0)
  {
    memset(BigSpace, 0, big_space_length);

    if (pass>1 && NCalls[proc]>pass) check_clean[proc]=1;
  }
}
