/*--------------------------------------------------------------------+
 | OCTAVE - Optimising Compiler Tool And Visualisation Environment    |
 |                                                                    |
 | Copyright (C) University of Edinburgh, 1993.                       |
 |                                                                    |
 | OCTAVE is free software; you can redistribute it and/or modify     |
 | it under the terms of the GNU General Public License as published  |
 | by the Free Software Foundation; either version 1, or (at your     |
 | option) any later version.                                         |
 |                                                                    |
 | OCTAVE is distributed in the hope that it will be useful,          |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of     |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      |
 | GNU General Public License for more details.                       |
 |                                                                    |
 | You should have received a copy of the GNU General Public License  |
 | along with OCTAVE; see the file COPYING.  If not, write to         |
 | the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,   |
 | USA.                                                               |
 +--------------------------------------------------------------------*/

#ifndef _defs_h_
#define _defs_h_

// Pointer types are defined here so that they can be
// forward-referenced to create circular type dependencies.
// However, for program clarity, the class name should
// be used in preference to the PTR_xxx form wherever
// possible.

typedef class ParseContext    *PTR_CNTXT ;
typedef class Statement	      *PTR_STMT ;
typedef class StmtList	      *PTR_BLOCK ;
typedef class Expression      *PTR_EXPR ;
typedef class Symbol	      *PTR_SYMB ;
typedef class Datatype        *PTR_TYPE ;
typedef class FtnLabel	      *PTR_FTN_LABEL ;
typedef class CcLabel	      *PTR_CC_LABEL ;
typedef class Comment         *PTR_CMNT ;
typedef class hash_entry      *PTR_HASH ;
typedef class SymbolTable     *PTR_STAB ;
typedef class LabelTable      *PTR_LTAB ;
typedef class ScopeNode       *PTR_SCOPE ;
typedef class TransHist       *PTR_TRANS ;

// Fortran parser constants

#define NO	   0
#define YES	   1
#define EOL 	  -1
#define SAME_GROUP 0
#define NEW_GROUP1 1
#define NEW_GROUP2 2
#define END_GROUP  3
#define FULL       0
#define HALF       1

// label type codes

#define LABUNKNOWN	0
#define LABEXEC		1
#define LABFORMAT	2
#define LABOTHER	3


// parser states

#define OUTSIDE 0
#define INSIDE	1
#define INDCL	2
#define INDATA	3
#define INEXEC	4

// nesting states

#define IN_OUTSIDE 4
#define IN_MODULE 3
#define IN_PROC   2
#define IN_INTERNAL_PROC 1

// These are used in decl_stat field of symbol

#define SOFT 0   // => symbol can be redeclared
#define HARD 1   // => symbol cannot be redeclared

/* These are to be used to indicate whether a symbol's type
 * was assigned implicitly, or whether it was explicitly
 * typed. Implicit variables must have their types set
 * whenever an `implicit <type> ' statement is seen.
 */

#define IMPL  0
#define TYPED 1

// Attributes (used in attr)

#define ATT_CLUSTER 1  /* Ftn only */
#define ATT_GLOBAL  2  /* Ftn only */
#define ATT_AUTO    4
#define ATT_PARAM   8  /* C, C++ only */
#define ATT_ANON   16  /* C, C++ only */

#define SECTION_SUBSCRIPT 1
#define EXTEND_NODE 2

#define MAX_NEST_LEVEL 256

#endif // _defs_h_

