//
// Class CProject for EOS project files
//
// Copyright (C) 2000-2004 by Grigori G.Fursin
//
// http://homepages.inf.ed.ac.uk/gfursin
//
// ICSA, School of Informatics,
// University of Edinburgh, UK
//

//Import libraries...
import java.io.*;

public class CProject
{
  //Variables (public only for copying purposes inside this object)
  public CLanguages lng;
  public String fname;
  public int fmax;
  public int fnum;
  
  public int      prj2len;
  public String[] prj2arr;
  public String   prj3arr;
  public int      prj4len;
  public String[] prj4arr;
  public String   prj5arr;

  public int[]    Lang;
  public String[] Lang1;
  public String[] FileName;
  public String[] FileExt;
  public String[] ExtraOpt;

  private String err="";

  private String qprj1="********** source files **********";
  private String qprj2="********** data files **********";
  private String qprj3="********** output **********";
  private String qprj4="********** output files **********";
  private String qprj5="********** exe **********";

  private String tparam="_param.tmp";

  //****************************************************************************
  //"Initialization"
  //
  //Input:  fname1  = Project filename
  //        fmax1   = Max number of files in the project
  //
  //Return: 0  - Ok
  //        >0 - Error
  //

  public int Init(CLanguages lng1,
                  String fname1,
                  int fmax1)
  {
    int i=0;

    if (fmax1<1) return 1;

    lng=lng1;
    fname=fname1;
    fmax=fmax1;
    fnum=0;

    Lang     =new int [fmax];
    Lang1    =new String [fmax];
    FileName =new String [fmax];
    FileExt  =new String [fmax];
    ExtraOpt =new String [fmax];

    prj3arr="";
    prj5arr="";

    err="";

    return 0;
  }

  public int Init(CProject cp, String fname1)
  {
    lng=cp.lng;
    fname=fname1;
    fmax=cp.fmax;
    fnum=0;

    Lang     =new int [fmax];
    Lang1    =new String [fmax];
    FileName =new String [fmax];
    FileExt  =new String [fmax];
    ExtraOpt =new String [fmax];

    Lang[0]=cp.Lang[0];
    Lang1[0]=cp.Lang1[0];
    FileName[0]="";
    FileExt[0]="";
    ExtraOpt[0]=cp.ExtraOpt[0];
    fnum=1;

    prj2len=cp.prj2len;
    prj2arr=new String[prj2len];
    for (int i=0; i<prj2len; i++)
    {
      prj2arr[i]=cp.prj2arr[i];
    }

    prj4len=cp.prj4len;
    prj4arr=new String[prj4len];
    for (int i=0; i<prj4len; i++)
    {
      prj4arr[i]=cp.prj4arr[i];
    }

    prj3arr=cp.prj3arr;
    prj5arr=cp.prj5arr;

    err="";

    return 0;
  }

  //****************************************************************************
  //"Parse Project File"
  //
  //Return: 0  - Ok
  //        <0 - Error
  //
  public int Parse()
  {
    err="";

    BufferedReader fin=null;

    fnum=0;
    prj2len=0;
    prj4len=0;
    String str1="";

    //********** First pass **********
    try
    {
      fin=new BufferedReader(new FileReader(fname));
    }
    catch (IOException e2)
    {
      err="Error: Can't find project file !";
      return -1;
    }

    try
    {
      str1=fin.readLine();
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file!";
      return -1;
    }

    if (!str1.equals(qprj1))
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (can't find part1)!";
      return -1;
    }

    try
    {
      while (((str1=fin.readLine())!=null) && (!str1.equals(qprj2)))
      {
        if (fnum>fmax) 
        {
          try
          {
            fin.close();
          }
          catch (Exception e2)
          {}
          err="Error: Too many filenames in project file (part1)!";
          return -1;
        }
        fnum++;
      }
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (part1)!";
      return -1;
    }

    if (fnum<2)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: No files in project!";
      return -1;
    }

    if (!str1.equals(qprj2))
    {                
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (can't find part2)!";
      return -1;
    }

    try
    {
      while (((str1=fin.readLine())!=null) && (!str1.equals(qprj3)))
      {
        prj2len++;
      }
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (part2)!";
      return -1;
    }

    if (!str1.equals(qprj3))
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (can't find part3)!";
      return -1;
    }

    try
    {
      str1=fin.readLine();
      if ((str1!=null) && (!str1.equals(qprj4)))
      {
        str1=fin.readLine();
      }
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (part3)!";
      return -1;
    }

    if (!str1.equals(qprj4))
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (can't find part4)!";
      return -1;
    }

    try
    {
      while (((str1=fin.readLine())!=null) && (!str1.equals(qprj5)))
      {
        prj4len++;
      }
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (part4)!";
      return -1;
    }

    if (prj4len<0)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problem with project4 file!";
      return -1;
    }

    if (!str1.equals(qprj5))
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (can't find part5)!";
      return -1;
    }

    try
    {
      str1=fin.readLine();
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (part5)!";
      return -1;
    }

    try
    {
      fin.close();
    }
    catch (Exception e3)
    {}

    // ********** Second pass **********
    String str1a="";
    String str1b="";
    String str1b1="";
    String str1c="";

    fnum=0;

    try
    {
      fin=new BufferedReader(new FileReader(fname));
    }
    catch (IOException e2)
    {
      err="Error: Can't find project file !";
      return -1;
    }

    try
    {
      str1=fin.readLine();
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file!";
      return -1;
    }

    if (!str1.equals(qprj1))
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (can't find part1)!";
      return -1;
    }

    try
    {
      while (((str1=fin.readLine())!=null) && (!str1.equals(qprj2)))
      {
        if (fnum>fmax) 
        {
          try
          {
            fin.close();
          }
          catch (Exception e2)
          {}
          err="Error: Too many filenames in project file (part1)!";
          return -1;
        }

        str1=str1.trim();
        str1a="";
        str1b="";
        str1b1="";
        str1c="";
        
        int il=str1.length();
        int i1=0;
        int i2=0;

        if (fnum==0)
        {
          i1=str1.indexOf(",");
          if (i1==-1) str1a=str1;
          else 
          {
            str1a=str1.substring(0, i1);
            str1c=str1.substring(i1+1, il);
          }
        }
        else
        {
          i1=str1.indexOf(",");
          if (i1==-1)
          {
            try
            {
              fin.close();
            }
            catch (Exception e2)
            {}
            err="Error: Can't understand format of the project file (part1)!";
            return -1;
          }
          else 
          {
            str1a=str1.substring(0, i1);
            i2=str1.indexOf(",", i1+1);
            if (i2==-1)
            {
              str1b=str1.substring(i1+1, il);
            }
            else
            {
              str1b=str1.substring(i1+1, i2);
              str1c=str1.substring(i2+1, il);
            }
          }
        }

        str1a=str1a.trim();
        str1b=str1b.trim();
        str1c=str1c.trim();

        Lang1[fnum]=str1a;
        ExtraOpt[fnum]=str1c;

        if (fnum!=0)
        {
          if (str1b.equals(""))
          {
            try
            {
              fin.close();
            }
            catch (Exception e2)
            {}
            err="Error: Can't understand format of the project file (part1)!";
            return -1;
          }

          int ix=str1b.length();
          int i=str1b.lastIndexOf(".");

          if (i!=-1)
          {
            str1b1=str1b.substring(i, ix);
            str1b=str1b.substring(0,i);
          }
        }

        FileName[fnum]=str1b;
        FileExt[fnum]=str1b1;
 
        fnum++;
      }
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (part1)!";
      return -1;
    }

    if (fnum<2)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: No files in project!";
      return -1;
    }

    if (!str1.equals(qprj2))
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (can't find part2)!";
      return -1;
    }

    prj2arr=new String[prj2len];
    int fnum2=0;

    try
    {
      while (((str1=fin.readLine())!=null) && (!str1.equals(qprj3)))
      {
        if (fnum2>prj2len) 
        {
          try
          {
            fin.close();
          }
          catch (Exception e2)
          {}
          err="Error: Too many filenames in project file (part2)!";
          return -1;
        }

        prj2arr[fnum2]=str1;
 
        fnum2++;
      }
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (part2)!";
      return -1;
    }

    if (!str1.equals(qprj3))
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (can't find part3)!";
      return -1;
    }

    BufferedWriter fout=null;

    try
    {
      fout=new BufferedWriter(new FileWriter(fname+tparam));
    }
    catch (IOException e2)
    {
      err="Error: Problems while opening parameter file for writing!";
      return -1;
    }

    prj3arr="";

    try
    {
      str1=fin.readLine();
      if ((str1!=null) && (!str1.equals(qprj4)))
      {
        prj3arr=str1;
        str1=fin.readLine();

        fout.write(prj3arr);
        fout.newLine();
      }
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
        fout.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (part3)!";
      return -1;
    }

    try
    {
      fout.close();
    }
    catch (Exception e3)
    {}

    if (!str1.equals(qprj4))
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (can't find part4)!";
      return -1;
    }

    prj4arr=new String[prj4len];
    int fnum4=0;

    try
    {
      while (((str1=fin.readLine())!=null) && (!str1.equals(qprj5)))
      {
        if (fnum4>prj4len) 
        {
          try
          {
            fin.close();
          }
          catch (Exception e2)
          {}
          err="Error: Too many filenames in project file (part4)!";
          return -1;
        }

        prj4arr[fnum4]=str1;
 
        fnum4++;
      }
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (part4)!";
      return -1;
    }

    if (!str1.equals(qprj5))
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (can't find part5)!";
      return -1;
    }

    prj5arr="";

    try
    {
      str1=fin.readLine();
      if (str1!=null) prj5arr=str1;
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading project file (part5)!";
      return -1;
    }

    try
    {
      fin.close();
    }
    catch (Exception e3)
    {}

    return 0;
  }

  //****************************************************************************
  //"Parse Project File"
  //
  //Return: 0  - Ok
  //        <0 - Error
  //
  public int ParseLng()
  {
    for (int i=0; i<fnum; i++)
    {
      int x=lng.GetLngIndex(Lang1[i]);
      if (x<0)
      {
        err="Error: Language in project1 file is not supported!";
        return -1;
      }
      Lang[i]=x;
    }

    return 0;
  }

  //****************************************************************************
  public int GetNumFiles()
  {
    return fnum;
  }

  //****************************************************************************
  public String GetLngNameOrig(int index)
  {
    return Lang1[index];
  }

  //****************************************************************************
  public int GetLng(int index)
  {
    return Lang[index];
  }

  //****************************************************************************
  public String GetLngName(int index)
  {
    return lng.GetLanguageName(index);
  }

  //****************************************************************************
  public String GetFileName(int index)
  {
    return FileName[index];
  }

  //****************************************************************************
  public String GetFileExt(int index)
  {
    return FileExt[index];
  }

  //****************************************************************************
  public String GetExtraOpt(int index)
  {
    return ExtraOpt[index];
  }

  //****************************************************************************
  public int AddLine(int ln, String fn, String fe, String eo)
  {
    if (fnum>fmax) return -1;

    Lang[fnum]=ln;
    if (lng!=null) Lang1[fnum]=lng.GetLanguageName(ln);
    else Lang1[fnum]=null;
    FileName[fnum]=fn;
    FileExt[fnum]=fe;
    ExtraOpt[fnum]=eo;

    fnum++;

    return 0;
  }

  //****************************************************************************
  public int AddLine1(String ln, String fn, String fe, String eo)
  {
    if (fnum>fmax) return -1;

//    Lang[fnum]=0;
    Lang1[fnum]=ln;
    FileName[fnum]=fn;
    FileExt[fnum]=fe;
    ExtraOpt[fnum]=eo;

    fnum++;

    return 0;
  }

  //****************************************************************************
  public int Save()
  {
    int i=0;
    BufferedWriter fprjout=null;

    //Saving project file
    try
    {
      fprjout=new BufferedWriter(new FileWriter(fname));
    }
    catch (IOException e1)
    {
      err="Error: Can't open new project file for saving!";
      return -1;
    }

    try
    {
      //******* part1 *******
      fprjout.write(qprj1);
      fprjout.newLine();

      for (i=0; i<fnum; i++)
      {
        fprjout.write(Lang1[i]);

        if (i!=0)
        {
          fprjout.write(", ");
          fprjout.write(FileName[i]);
          fprjout.write(FileExt[i]);
        }

        if (!ExtraOpt[i].equals(""))
        {
          fprjout.write(", ");
          fprjout.write(ExtraOpt[i]);
        }

        fprjout.newLine();
      }

      //******* part2 *******
      fprjout.write(qprj2);
      fprjout.newLine();

      for (i=0; i<prj2len; i++)
      {
        fprjout.write(prj2arr[i]);
        fprjout.newLine();
      }

      //******* part3 *******
      fprjout.write(qprj3);
      fprjout.newLine();

      if (!prj3arr.equals(""))
      {
        fprjout.write(prj3arr);
        fprjout.newLine();
      }

      //******* part4 *******
      fprjout.write(qprj4);
      fprjout.newLine();

      for (i=0; i<prj4len; i++)
      {
        fprjout.write(prj4arr[i]);
        fprjout.newLine();
      }

      //******* part5 *******
      fprjout.write(qprj5);
      fprjout.newLine();

      if (!prj5arr.equals(""))
      {
        fprjout.write(prj5arr);
        fprjout.newLine();
      }
    }
    catch (Exception e2)
    {
      try
      {
        fprjout.close();
      }
      catch (Exception e3)
      {}
      err="Error: Problems while writing new project1 file!";
      return -1;
    }

    try
    {
      fprjout.close();
    }
    catch (Exception e5)
    {
      err="Error: Can't close new project file!";
      return -1;
    }

    return 0;
  }

  //****************************************************************************
  public String GetLastError()
  {
    String err1=err;
    err="";
    return err1;
  }

  //****************************************************************************
  public int GetPrepMax()
  {
    return prj2len;
  }

  //****************************************************************************
  public String GetPrep(int index)
  {
    return prj2arr[index];
  }

  //****************************************************************************
  public String GetParam()
  {
    return prj3arr;
  }

  //****************************************************************************
  public String GetParamFile()
  {
    return fname+tparam;
  }

  //****************************************************************************
  public int GetCorMax()
  {
    return prj4len;
  }

  //****************************************************************************
  public String GetCor(int index)
  {
    return prj4arr[index];
  }

  //****************************************************************************
  public String GetExe()
  {
    return prj5arr;
  }

  //****************************************************************************
  public void SetExe(String str)
  {
    prj5arr=str;
  }


  //****************************************************************************
  public void AddFromProject(String[] fil, int len, CProject cp, String f_add)
  {
    int i=0;
    int j=0;
    int k=1;

    String[] fil1=new String[len];
    String[] fil2=new String[len];

    for (i=0; i<len; i++)
    {
      String x1=fil[i];
      String x2="";

      int ix=fil[i].length();
      for (int ii=ix-1; ii>=0; ii--)
      {
        if (fil[i].charAt(ii)=='.')
        {
          x1=fil[i].substring(0,ii);
          x2=fil[i].substring(ii, ix);
        }
      }

      fil1[i]=x1;
      fil2[i]=x2;
    }

    for (j=1; j<cp.GetNumFiles(); j++)
    {
      for (i=0; i<len; i++)
      {
        if ((cp.GetFileName(j).equals(fil1[i])) && (cp.GetFileExt(j).equals(fil2[i])))
        {
          Lang[k]=cp.GetLng(j);
          Lang1[k]=cp.GetLngNameOrig(j);
          FileName[k]=cp.GetFileName(j)+f_add;
          FileExt[k]=cp.GetFileExt(j);
          ExtraOpt[k]=cp.GetExtraOpt(j);

          k++;
        }
      }
    }

    fnum=k;
  }
}
