/*--------------------------------------------------------------------+
 | OCTAVE - Optimising Compiler Tool And Visualisation Environment    |
 |                                                                    |
 | Copyright (C) University of Edinburgh, 1993.                       |
 |                                                                    |
 | OCTAVE is free software; you can redistribute it and/or modify     |
 | it under the terms of the GNU General Public License as published  |
 | by the Free Software Foundation; either version 1, or (at your     |
 | option) any later version.                                         |
 |                                                                    |
 | OCTAVE is distributed in the hope that it will be useful,          |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of     |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      |
 | GNU General Public License for more details.                       |
 |                                                                    |
 | You should have received a copy of the GNU General Public License  |
 | along with OCTAVE; see the file COPYING.  If not, write to         |
 | the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,   |
 | USA.                                                               |
 +--------------------------------------------------------------------*/

/* Header for files.c - which contains helper functions for dealing
 * with file I/O and file status interrogations.
 */

#if ! defined _files_h_
#define _files_h_

// FUNCTION GROUP: file system predicates

    ////////////////////
    //
    // Returns non-zero (true) value if named file is readable.
    // 
    //  [in] char*  the name of the file
    //
int isFileReadable (char *) ;

    ////////////////////
    //
    // If file already exists, a dialog asks user whether to
    // overwrite, or cancel the request which called this routine
    // A non-zero value is returned if the go-ahead was given to write.
    //
    //  [in] char*  the name of the file
    //
int preventOverWrite (char *) ;

    ////////////////////
    //
    // Returns a non-zero value if the first parameter is a filename
    // which ends in a .pro extension. If it is, then the second
    // parameter is assigned the first parameter and the function.
    // 
    //  [in] char*  a possible project file name
    //  [in] char** pointer to the project filename to be used
    //
int is_a_pro_file (char *, char **) ;

    ////////////////////
    //
    // Checks whether the named file is a source code file.
    // Currently it only checks for a .f extension.
    //
    //  [in] char*  the name of the file
    //
int is_a_code_file (char *) ;

    ////////////////////
    //
    // Checks whether the named file is writable, returning a non-zero
    // value if the named file can be written.
    // Interactive diagnostics are offered in the cases where the
    // file named is a directory, or the user does not have permission
    // to write the file, or if there is a file system error.
    //
    //  [in] char*  the name of the file
    //
int isFileWritable (char *) ;

#endif // _files_h_
