//
// Process Instructions
//
// Copyright (C) 2000-2004 by Grigori G.Fursin
//
// http://homepages.inf.ed.ac.uk/gfursin
//
// ICSA, School of Informatics,
// University of Edinburgh, UK
//

//Languages
#include "param.h"
#include "siparse.h"
#include "global.h"
#include "misc.h"

#include "ast.h"

//Sockets
#ifdef W32
#  include <winsock2.h>
#  include <wsipx.h>
#else
#  include <sys/socket.h>
#  include <sys/types.h>
#  include <memory.h>
#  include <netinet/in.h>
#endif

#include <string.h>

#include <iostream.h>
#include <fstream.h>

#include <stdio.h>
#include <stdlib.h> //for system

int Parse(int, CGlobal*, char*, char*, char*);
int Tree(int, CGlobal*, char*, char*, char*);

#define maxstrlen 1024
#define maxbuf    1024
#define maxbuflen  128


void ProcessInstructions(int sockfd, CGlobal* g, CSIParse* s)
{
  char str[maxstrlen]="";
  char buf[maxstrlen]="";
  char buf1[maxstrlen]="";  
  char xerr[maxstrlen]="";
  int misc;

  //Parameters
  int initiate=0;
  char str_file_in[maxstrlen]="";
  char str_file_out[maxstrlen]="";
  char str_file_out_err[maxstrlen]="";

  //Buffer for array/loop commands
  char cur_proc[maxbuflen]="";
  char cur_param[maxbuflen]="";
  char (*buf_proc)[maxbuflen]  = new char[maxbuf][maxbuflen];
  char (*buf_array)[maxbuflen] = new char[maxbuf][maxbuflen];
  char (*buf_loop)[maxbuflen]  = new char[maxbuf][maxbuflen];
  char (*buf_p1)[maxbuflen]    = new char[maxbuf][maxbuflen];
  char (*buf_p2)[maxbuflen]    = new char[maxbuf][maxbuflen];
  int  (*buf_array_op)         = new int[maxbuf];
  int  (*buf_loop_op)          = new int[maxbuf];
  int  (*buf_corr)             = new int[maxbuf];
  int pnt=0;

  int j=0;

  for (;;)
  {
    buf[0]=0;
    buf1[0]=0;
    int p=0;
    int esc=0;
    int nread=0;

    while (((nread=recv(sockfd, buf1, 1, 0))>0) && (buf1[0]!='\r'))
    {
      //Checking for characters
      if ((buf1[0]>31) && (buf1[0]<127))
      {
        if (esc==0)
        {
          buf[p]=buf1[0];
          p++;

          //send echo  
          buf1[1]=0;
          int misc=send(sockfd, buf1, 1, 0);
  
          if (misc<0)
          {
            cerr << "Error: Can't send data to client!\n";
            return;
          }
        }
        else if (esc==1) esc=2;
        else esc=0;
      }
      else if (buf1[0]==27)
      {
        //Escape
        if (esc==0) esc=1;
        else if (esc==1) esc=0;
      }
      else if (buf1[0]=='\n')
      {
        //Ignore character
        if (esc==1) esc=2;
        else esc=0;
      }
      else if (buf1[0]=='\b')
      {
        //Backspace
        if (esc==0)
        {
          if (p>0) 
          {
            p--;
            buf[p]=0;

            //send echo  
            buf1[0]='\b';
            buf1[1]=' ';
            buf1[2]='\b';
            buf1[3]=0;
            int misc=send(sockfd, buf1, 3, 0);
  
            if (misc<0)
            {
              cerr << "Error: Can't send data to client!\n";
              return;
            }
          }
        }
        else if (esc==1) esc=2;
        else esc=0;
      }
    }

    buf[p]=0;

    if (nread<0)
    {
//      cerr << "Error: Can't receive data from client!\n";
//      Probably connection was closed
      return;
    }

    if (nread==0)
    {
//      cout << "Session complete!\n";
      return;
    }

    if (SendMes(sockfd, "", 0, 1024)<0) 
    {
      cerr << "Error: Can't send data to client!\n";
      return;
    }

    strcat(str, buf);

    //process str ********************************************************
    misc=s->Parse(str);

    if (misc<0)
    {
      switch (misc)
      {
        case -1:
        {
          strcpy(xerr, "Warning: Line is empty!");
          break;
        }
        case -2:
        {
          strcpy(xerr, "Error: Unknown instruction!");
          break;
        }
        case -3:
        {
          strcpy(xerr, "Error: This instruction needs parameter!");
          break;
        }
        case -4:
        {
          strcpy(xerr, "Error: This instruction doesn't need parameters!");
          break;
        }
        case -5:
        {
          strcpy(xerr, "Error: Wrong separator!");
          break;
        }
        default:
        {
          strcpy(xerr, "Error: Can't parse!");
          break;
        }
      }

      if (SendMes(sockfd, xerr, 0, 1024)<0) return;
    }
    else
    {
      //Process instructions
      switch(misc)
      {
        case instrInitiate:
        {
          if (strcmp(s->GetParamStrLwr(instrInitiate), "transformations")==0) 
          {
            initiate=1;
            strcpy(str_file_in, "");
            strcpy(str_file_out, "");
            strcpy(str_file_out_err, "");

            strcpy(cur_proc, "");
            strcpy(cur_param, "");
            for (j=0; j<maxbuf; j++)
            {
              strcpy(buf_proc[j], "");
              strcpy(buf_array[j], "");
              strcpy(buf_loop[j], "");
              strcpy(buf_p1[j], "");
              strcpy(buf_p2[j], "");
              buf_array_op[j]=0;
              buf_loop_op[j]=0;
              buf_corr[j]=0;
            }
            pnt=0;

            if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          }
          else
          {
            if (SendMes(sockfd, "Error: Unknown parameter for 'initiate'!", 0, 1024)<0) return;
          }
          break;
        }
        case instrTerminate:
        {
          SendMes(sockfd, "Ok [Session finished]", 0, 1024);
          return; //Finish
          break;
        }
        case instrFilein:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          strcpy(str_file_in, s->GetParamStr(instrFilein));
          strcpy(cur_proc, "");
          strcpy(cur_param, "");
          for (j=0; j<maxbuf; j++)
          {
            strcpy(buf_proc[j], "");
            strcpy(buf_array[j], "");
            strcpy(buf_loop[j], "");
            strcpy(buf_p1[j], "");
            strcpy(buf_p2[j], "");
            buf_array_op[j]=0;
            buf_loop_op[j]=0;
            buf_corr[j]=0;
          }
          pnt=0;

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrFileout:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }
          strcpy(str_file_out, s->GetParamStr(instrFileout));
          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrFileouterror:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }
          strcpy(str_file_out_err, s->GetParamStr(instrFileouterror));
          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrParse:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          //Parse ...
          if ((strcmp(str_file_in, "")==0) ||
	      (strcmp(str_file_out, "")==0))
	  {
            if (SendMes(sockfd, "Error: One of parameters is empty!", 0, 1024)<0) return; 
            break;
	  }

          //Deleting output files
          if (strcmp(str_file_out, "")!=0) remove(str_file_out);
          if (strcmp(str_file_out_err, "")!=0) remove(str_file_out_err);

          if (Parse(sockfd, g,
                    str_file_in, str_file_out, 
                    str_file_out_err)!=0) return;

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrTree:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          //Tree ...
          if ((strcmp(str_file_in, "")==0) ||
              (strcmp(str_file_out, "")==0))
	  {
            if (SendMes(sockfd, "Error: One of parameters is empty!", 0, 1024)<0) return; 
            break;
	  }

          //Deleting output files
          if (strcmp(str_file_out, "")!=0) remove(str_file_out);
          if (strcmp(str_file_out_err, "")!=0) remove(str_file_out_err);

          if (Tree(sockfd, g,
                   str_file_in, str_file_out, 
                   str_file_out_err)!=0) return;

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrProcedure:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }
          strcpy(cur_proc, s->GetParamStrLwr(instrProcedure));

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrParam:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }
          strcpy(cur_param, s->GetParamStrLwr(instrParam));

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrLoop:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          if (pnt>=maxbuf)
          {
            if (SendMes(sockfd, "Error: Buffer for instuctions is full!", 0, 1024)<0) return; 
            break;
          }
          
          if (strcmp(cur_proc, "")==0)
          {
            if (SendMes(sockfd, "Error: Procedure is needed to be specified!", 0, 1024)<0) return; 
            break;
          }

          strcpy(buf_proc[pnt], cur_proc);
          strcpy(buf_array[pnt], "");
          strcpy(buf_loop[pnt], s->GetParamStr(instrLoop));
          strcpy(buf_p1[pnt], "");
          strcpy(buf_p2[pnt], "");
          strcpy(cur_param, "");
          buf_array_op[pnt]=0;
          buf_loop_op[pnt]=0;
          buf_corr[pnt]=0;

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrArray:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          if (pnt>=maxbuf)
          {
            if (SendMes(sockfd, "Error: Buffer for instuctions is full!", 0, 1024)<0) return; 
            break;
          }
          
          if (strcmp(cur_proc, "")==0)
          {
            if (SendMes(sockfd, "Error: Procedure is needed to be specified!", 0, 1024)<0) return; 
            break;
          }

          strcpy(buf_proc[pnt], cur_proc);
          strcpy(buf_array[pnt], s->GetParamStrLwr(instrArray));
          strcpy(buf_loop[pnt], "");
          strcpy(buf_p1[pnt], "");
          strcpy(buf_p2[pnt], "");
          strcpy(cur_param, "");
          buf_array_op[pnt]=0;
          buf_loop_op[pnt]=0;
          buf_corr[pnt]=0;

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrCorrect:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          if (pnt>=maxbuf)
          {
            if (SendMes(sockfd, "Error: Buffer for instuctions is full!", 0, 1024)<0) return; 
            break;
          }
          
          if (strcmp(cur_proc, "")==0)
          {
            if (SendMes(sockfd, "Error: Procedure is needed to be specified!", 0, 1024)<0) return; 
            break;
          }

          buf_corr[pnt]=1;

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrLooptiling:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          if (pnt>=maxbuf)
          {
            if (SendMes(sockfd, "Error: Buffer for instuctions is full!", 0, 1024)<0) return; 
            break;
          }
          
          if (strcmp(buf_loop[pnt], "")==0)
          {
            if (SendMes(sockfd, "Error: Loop is needed to be specified!", 0, 1024)<0) return; 
            break;
          }

          strcpy(buf_p1[pnt], s->GetParamStr(instrLooptiling));
          strcpy(buf_p2[pnt], cur_param);

          buf_array_op[pnt]=0;
          buf_loop_op[pnt]=instrLooptiling;

          pnt++;

          strcpy(cur_param, "");

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrLoopinterchange:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          if (pnt>=maxbuf)
          {
            if (SendMes(sockfd, "Error: Buffer for instuctions is full!", 0, 1024)<0) return; 
            break;
          }
          
          if (strcmp(buf_loop[pnt], "")==0)
          {
            if (SendMes(sockfd, "Error: Loop is needed to be specified!", 0, 1024)<0) return; 
            break;
          }

          strcpy(buf_p1[pnt], s->GetParamStr(instrLoopinterchange));
          strcpy(buf_p2[pnt], cur_param);

          buf_array_op[pnt]=0;
          buf_loop_op[pnt]=instrLoopinterchange;

          pnt++;

          strcpy(cur_param, "");

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrLoopunrolling:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          if (pnt>=maxbuf)
          {
            if (SendMes(sockfd, "Error: Buffer for instuctions is full!", 0, 1024)<0) return; 
            break;
          }
          
          if (strcmp(buf_loop[pnt], "")==0)
          {
            if (SendMes(sockfd, "Error: Loop is needed to be specified!", 0, 1024)<0) return; 
            break;
          }

          strcpy(buf_p1[pnt], s->GetParamStr(instrLoopunrolling));
          strcpy(buf_p2[pnt], cur_param);

          buf_array_op[pnt]=0;
          buf_loop_op[pnt]=instrLoopunrolling;

          pnt++;

          strcpy(cur_param, "");

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrLoopstripmining:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          if (pnt>=maxbuf)
          {
            if (SendMes(sockfd, "Error: Buffer for instuctions is full!", 0, 1024)<0) return; 
            break;
          }
          
          if (strcmp(buf_loop[pnt], "")==0)
          {
            if (SendMes(sockfd, "Error: Loop is needed to be specified!", 0, 1024)<0) return; 
            break;
          }

          strcpy(buf_p1[pnt], s->GetParamStr(instrLoopstripmining));
          strcpy(buf_p2[pnt], cur_param);

          buf_array_op[pnt]=0;
          buf_loop_op[pnt]=instrLoopstripmining;

          pnt++;

          strcpy(cur_param, "");

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrLoopsplitting:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          if (pnt>=maxbuf)
          {
            if (SendMes(sockfd, "Error: Buffer for instuctions is full!", 0, 1024)<0) return; 
            break;
          }
          
          if (strcmp(buf_loop[pnt], "")==0)
          {
            if (SendMes(sockfd, "Error: Loop is needed to be specified!", 0, 1024)<0) return; 
            break;
          }

          strcpy(buf_p1[pnt], s->GetParamStr(instrLoopsplitting));
          strcpy(buf_p2[pnt], cur_param);

          buf_array_op[pnt]=0;
          buf_loop_op[pnt]=instrLoopsplitting;

          pnt++;

          strcpy(cur_param, "");

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrLoopdistribution:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          if (pnt>=maxbuf)
          {
            if (SendMes(sockfd, "Error: Buffer for instuctions is full!", 0, 1024)<0) return; 
            break;
          }
          
          if (strcmp(buf_loop[pnt], "")==0)
          {
            if (SendMes(sockfd, "Error: Loop is needed to be specified!", 0, 1024)<0) return; 
            break;
          }

          strcpy(buf_p1[pnt], "");
          strcpy(buf_p2[pnt], cur_param);

          buf_array_op[pnt]=0;
          buf_loop_op[pnt]=instrLoopdistribution;

          pnt++;

          strcpy(cur_param, "");

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrArraypad:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          if (pnt>=maxbuf)
          {
            if (SendMes(sockfd, "Error: Buffer for instuctions is full!", 0, 1024)<0) return; 
            break;
          }
          
          if (strcmp(buf_array[pnt], "")==0)
          {
            if (SendMes(sockfd, "Error: Array is needed to be specified!", 0, 1024)<0) return; 
            break;
          }

          strcpy(buf_p1[pnt], s->GetParamStr(instrArraypad));
          strcpy(buf_p2[pnt], cur_param);

          buf_array_op[pnt]=instrArraypad;
          buf_loop_op[pnt]=0;

          pnt++;

          strcpy(cur_param, "");

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrArraygir:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          if (pnt>=maxbuf)
          {
            if (SendMes(sockfd, "Error: Buffer for instuctions is full!", 0, 1024)<0) return; 
            break;
          }
          
          if (strcmp(buf_array[pnt], "")==0)
          {
            if (SendMes(sockfd, "Error: Array is needed to be specified!", 0, 1024)<0) return; 
            break;
          }

          strcpy(buf_p1[pnt], s->GetParamStr(instrArraygir));
          strcpy(buf_p2[pnt], cur_param);

          buf_array_op[pnt]=instrArraygir;
          buf_loop_op[pnt]=0;
          buf_corr[pnt]=0;

          pnt++;

          strcpy(cur_param, "");

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrGo:
        {

          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          if (pnt==0)
          {
            if (SendMes(sockfd, "Error: No instructions in buffer!", 0, 1024)<0) return; 
            break;
          }

          //Parse ...
          if ((strcmp(str_file_in, "")==0) ||
	      (strcmp(str_file_out, "")==0))
	  {
            if (SendMes(sockfd, "Error: One of parameters is empty!", 0, 1024)<0) return; 
            break;
	  }

          //Deleting output files
          if (strcmp(str_file_out, "")!=0) remove(str_file_out);
          if (strcmp(str_file_out_err, "")!=0) remove(str_file_out_err);

          //go
          int parseErrors=0;  

          //check if file exist
          #ifdef W32
            ifstream fin(str_file_in, ios::nocreate);
          #else
            ifstream fin(str_file_in);
          #endif
	  
          if (!fin)
          {
            SendMes(sockfd, "Error: Can't find source file!", 0, 1024); 
            break;
          }

          fin.close();

          parseErrors=MiscParse(str_file_in);

          if (parseErrors!=0)
          {
            if (strcmp(str_file_out_err, "")!=0)
            {
              if (MiscErrors(str_file_out_err)!=0)
              {
                SendMes(sockfd, "Error: Can't open file with errors in source code during parsing!", 0, 1024); 
//                break;
              }
            }
            SendMes(sockfd, "Error: Parsing errors!", 0, 1024); 
            break;
          }

          //Navigating through buffer
          int er=0;

          int loop_corr=0;
          char cur_proc[maxstrlen]="";

          for (j=0; (j<pnt) && (er==0); j++)
          {
            if (strcmp(buf_array[j], "")!=0)
            {
              //array
              switch (buf_array_op[j])
              {
                case instrArraypad:
                {
                  er=MiscArraypad(sockfd, buf_proc[j], buf_array[j], buf_p1[j]);
                  break;
                }
                case instrArraygir:
                {
                  er=MiscArraygir(sockfd, buf_proc[j], buf_array[j], buf_p1[j], buf_p2[j]);
                  break;
                }
                default:
                {
                  break;
                }
              }
            }
            else
            {
              //loop
              if (strcmp(cur_proc, buf_proc[j])!=0)
              {
                loop_corr=0;
                strcpy(cur_proc, buf_proc[j]);
              }

              long loop=atol(buf_loop[j])+loop_corr;
              sprintf(buf_loop[j], "%i", loop);               

              switch (buf_loop_op[j])
              {
                case instrLooptiling:
                {
                  er=MiscLoopTiling(sockfd, buf_proc[j], buf_loop[j], buf_p1[j]);
                  break;
                }
                case instrLoopinterchange:
                {
                  er=MiscLoopInterchange(sockfd, buf_proc[j], buf_loop[j], buf_p1[j]);
                  break;
                }
                case instrLoopunrolling:
                {
                  er=MiscLoopUnrolling(sockfd, buf_proc[j], buf_loop[j], buf_p1[j]);
                  break;
                }
                case instrLoopstripmining:
                {
                  er=MiscLoopStripmining(sockfd, buf_proc[j], buf_loop[j], buf_p1[j]);
                  break;
                }
                case instrLoopsplitting:
                {
                  er=MiscLoopSplitting(sockfd, buf_proc[j], buf_loop[j], buf_p1[j]);
                  break;
                }
                case instrLoopdistribution:
                {
                  er=MiscLoopDistribution(sockfd, buf_proc[j], buf_loop[j]);
                  break;
                }
                default:
                {
                  break;
                }
              }

              if (buf_corr[j]==1) loop_corr+=MiscCorrect();
            }
          }

          //clearing buffer
          strcpy(cur_proc, "");
          strcpy(cur_param, "");
          for (j=0; j<maxbuf; j++)
          {
            strcpy(buf_proc[j], "");
            strcpy(buf_array[j], "");
            strcpy(buf_loop[j], "");
            strcpy(buf_p1[j], "");
            strcpy(buf_p2[j], "");
            buf_array_op[j]=0;
            buf_loop_op[j]=0;
          }
          pnt=0;

          if (er!=0)
          {
            MiscClean();
            break;
          }

          if (MiscUnparse(str_file_out)!=0)
          {
            SendMes(sockfd, "Error: Can't open output file!", 0, 1024); 
            MiscClean();
            break;
          }

          MiscClean();

          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }

        default:
        {
          break;
        }
      }
    }

    strcpy(str,"");
  }
}

int Parse(int sockfd, CGlobal* g,
          char* str_file_in, char* str_file_out, 
          char* str_file_out_err)
{
  int parseErrors=0;  

  //check if file exist
  #ifdef W32
    ifstream fin(str_file_in, ios::nocreate);
  #else
    ifstream fin(str_file_in);
  #endif
	  
  if (!fin)
  {
    SendMes(sockfd, "Error: Can't find source file!", 0, 1024); 
    return -1;
  }

  fin.close();

  parseErrors=MiscParse(str_file_in);

  if (parseErrors!=0)
  {
    if (strcmp(str_file_out_err, "")!=0)
    {
      if (MiscErrors(str_file_out_err)!=0)
      {
        SendMes(sockfd, "Error: Can't open file with errors in source code during parsing!", 0, 1024); 
//        return -2;
      }
    }
    SendMes(sockfd, "Error: Parsing errors!", 0, 1024); 
    return -2;
  }
  else
  {
    if (MiscUnparse(str_file_out)!=0)
    {
      SendMes(sockfd, "Error: Can't open output file!", 0, 1024); 
      return -3;
    }
  }

  MiscClean();

  return 0;
}

int Tree(int sockfd, CGlobal* g,
         char* str_file_in, char* str_file_out, 
         char* str_file_out_err)
{
  int parseErrors=0;  

  //check if file exist
  #ifdef W32
    ifstream fin(str_file_in, ios::nocreate);
  #else
    ifstream fin(str_file_in);
  #endif
	  
  if (!fin)
  {
    SendMes(sockfd, "Error: Can't find source file!", 0, 1024); 
    return -1;
  }

  fin.close();

  parseErrors=MiscParse(str_file_in);

  if (parseErrors!=0)
  {
    if (strcmp(str_file_out_err, "")!=0)
    {
      if (MiscErrors(str_file_out_err)!=0)
      {
        SendMes(sockfd, "Error: Can't open file with errors in source code during parsing!", 0, 1024); 
//        return -2;
      }
    }
    SendMes(sockfd, "Error: Parsing errors!", 0, 1024); 
    return -2;
  }
  else
  {
    if (MiscTree(str_file_out)!=0)
    {
      SendMes(sockfd, "Error: Can't open output file!", 0, 1024); 
      return -3;
    }
  }

  MiscClean();

  return 0;
}
