//
// Process Instructions
//
// Copyright (C) 2000-2004 by Grigori G.Fursin
//
// http://homepages.inf.ed.ac.uk/gfursin
//
// ICSA, School of Informatics,
// University of Edinburgh, UK
//

//Import libraries...
import java.net.*;
import java.io.*;
import java.util.*;

public class ProcessInstructions
{                    
  String f_tmp       = "_eos_mpc_tmp";
  String f_tmp1      = "_eos_mpc_tmp1";
  String f_tmp2      = "_eos_mpc_tmp2";
  String f_tmp_prj   = "_eos_prj_tmp";
  String f_tmp_prj1  = "_eos_prj_1_tmp";
  String f_tmp3      = "_eos_mpc_tmp3";
  String f_mpc2      = "_mpc2";
  String f_mpc2a     = "_mpc2a";
  String f_mpc2b     = "_mpc2b";
  String f_mpc5      = "_mpc5";
  String f_mpc5a     = "_mpc5a";
  String f_mpc5b     = "_mpc5b";
  String f_mpc5b1    = "_mpc5b1";
  String f_mpc5c     = "_mpc5c";
  String f_mpc5d     = "_mpc5d";
  String f_mpc5atom  = "_mpc5b.fatom";
  String f_tmp_atom  = "_eos_mpc_atom_tmp";
  String f_mpc_d     = "eos_mpc_dummy";
  String f_mpc_d1    = "_eos_mpc_atom_tmp";
  String f_mpc_d2    = "eos_mpc_fb1.fatom";
  String f_mpc_d3    = "eos_mpc_fb4.fatom";
  String f_mpc_fb2a  = "_eos_mpc_fb2.dat";
  String f_mpc_fb2b  = "_eos_mpc_fb2.out";

  public ProcessInstructions(Socket sockfd, CGlobal g, CSIParse s)
  {
    String str="";
    String buf="";
    int buf1;
    char buf2;
    String xerr="";
    boolean enter=false;
    int misc;
    
    PrintWriter out;
    BufferedReader in;

    //Parameters
    int initiate=0;
    String str_dir="";
    String str_f_prj="";
    String str_f_diag_out="";
    String str_f_prof="";
    String str_f_tree="";
    String str_optimization="";
    String str_f_info="";
    String str_dep="";
    String str_lp="";
    String str_sr="";
    String str_lc="";
    String str_prm="";
    String str_prm1="";
    String str_apnd="";
    boolean apnd=false;
    
    try 
    {
      out=new PrintWriter(sockfd.getOutputStream(), true);
      in =new BufferedReader(new InputStreamReader(sockfd.getInputStream()));

      for (;;)
      {
        buf="";
        int p=0;
        int esc=0;
        
        while ((buf1=in.read())!='\r')
        {
          if (buf1==-1)
          {
//            System.out.println("Session complete!");
            return;
          }
          
          buf2=(char) buf1;
          
          //Checking for characters
          if ((buf1>31) && (buf1<127))
          {
            if (esc==0)
            {
              p++;
              buf+=buf2;

              //send echo  
              out.print(buf2);
              out.flush();
            }
            else if (esc==1) esc=2;
            else esc=0;
          }
          else if (buf1==27)
          {
            //Escape
            if (esc==0) esc=1;
            else if (esc==1) esc=0;
          }
          else if (buf1=='\n')
          {
            //Ignore character
            if (esc==1) esc=2;
            else esc=0;
          }
          else if (buf1=='\b')
          {
            //Backspace
            if (esc==0)
            {
              if (p>0) 
              {
                p--;
                buf=buf.substring(0,p);

                //send echo  
                out.print("\b \b");
                out.flush();        
              }
            }
            else if (esc==1) esc=2;
            else esc=0;
          }
        }

        out.print('\r');
        out.print('\n');
        out.flush();
        
        //process instruction ********************************************************
        misc=s.Parse(buf);

        if (misc<0)
        {
          switch (misc)
          {
            case -1:
            {
              xerr="Warning: Line is empty!";
              break;
            }
            case -2:
            {
              xerr="Error: Unknown instruction!";
              break;
            }
            case -3:
            {
              xerr="Error: This instruction needs parameter!";
              break;
            }
            case -4:
            {
              xerr="Error: This instruction doesn't need parameters!";
              break;
            } 
            case -5:
            {
              xerr="Error: Wrong separator!";
              break;
            }
            default:
            {
              xerr="Error: Can't parse!";
              break;
            }
          }

          out.println(xerr);
          out.flush();
        }
        else
        {
          //Process instructions
          if (misc==g.instrInitiate)
          {
            if (s.GetParamStrLwr(g.instrInitiate).equals("mpc"))
            {
              initiate=1;

              str_dir="";
              str_f_prj="";
              str_f_diag_out="";
              str_f_prof="";
              str_f_tree="";
              str_optimization="";
              str_f_info="";
              str_dep="";
              str_lp="";
              str_sr="";
              str_lc="";
              str_prm="";
              str_prm1="";
              str_apnd="";
              apnd=false;

              out.println("Ok");
              out.flush();
            }
            else
            {
              out.println("Error: Unknown parameter for 'initiate'!");
              out.flush();
            }
          }
          else if (misc==g.instrTerminate)
          {
            out.println("Ok [Session finished]");
            out.flush();
            return; //Finish
          }
          else if (misc==g.instrDirectory)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_dir=s.GetParamStr(g.instrDirectory);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileProject)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_prj=s.GetParamStr(g.instrFileProject);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileDiagOut)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_diag_out=s.GetParamStr(g.instrFileDiagOut);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileProf)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_prof=s.GetParamStr(g.instrFileProf);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrDOAppend)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_apnd=s.GetParamStr(g.instrDOAppend);
              if (str_apnd.equals("1")) apnd=true; else apnd=false;

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileTree)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_tree=s.GetParamStr(g.instrFileTree);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileInfo)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_info=s.GetParamStr(g.instrFileInfo);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrOptimization)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_optimization=s.GetParamStr(g.instrOptimization);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrDependency)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_dep=s.GetParamStr(g.instrDependency);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrLoopCache)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_lp=s.GetParamStr(g.instrLoopCache);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrStackReg)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_sr=s.GetParamStr(g.instrStackReg);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrLcommCheck)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_lc=s.GetParamStr(g.instrLcommCheck);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrParam)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_prm=s.GetParamStr(g.instrParam);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrParam1)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_prm1=s.GetParamStr(g.instrParam1);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrGo1)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_prof.equals(""))
              {
                out.println("Error: Parameter 'file_prof' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_optimization.equals("")) str_optimization="0";

                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out, apnd));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go1(out, g, fout, str_dir2, str_f_prj,
                                    str_f_prof, str_f_tree,
                                    str_optimization);
                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrGo2)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_prof.equals(""))
              {
                out.println("Error: Parameter 'file_prof' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_optimization.equals("")) str_optimization="0";

                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out, apnd));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go2(out, g, fout, str_dir2, str_f_prj,
                                    str_f_prof, str_f_info, str_lc,
                                    str_optimization, 2);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrGo3)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_prof.equals(""))
              {
                out.println("Error: Parameter 'file_prof' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_optimization.equals("")) str_optimization="0";

                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out, apnd));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go2(out, g, fout, str_dir2, str_f_prj,
                                    str_f_prof, str_f_info, str_lc,
                                    str_optimization, 3);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrGo4)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_prof.equals(""))
              {
                out.println("Error: Parameter 'file_prof' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_optimization.equals("")) str_optimization="0";

                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out, apnd));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go2(out, g, fout, str_dir2, str_f_prj,
                                    str_f_prof, str_f_info, str_lc, 
                                    str_optimization, 4);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrGo5)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_optimization.equals("")) str_optimization="0";

                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out, apnd));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go2(out, g, fout, str_dir2, str_f_prj,
                                    "", str_f_info, str_lc,
                                    str_optimization, 5);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrGo6)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_optimization.equals("")) str_optimization="0";

                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out, apnd));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go2(out, g, fout, str_dir2, str_f_prj,
                                    "", str_f_info, str_lc,
                                    str_optimization, 6);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrGo7)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_optimization.equals("")) str_optimization="0";

                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out, apnd));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go2(out, g, fout, str_dir2, str_f_prj,
                                    "", str_f_info, str_lc,
                                    str_optimization, 7);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrGo8)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_prof.equals(""))
              {
                out.println("Error: Parameter 'file_prof' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_optimization.equals("")) str_optimization="0";

                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out, apnd));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go2(out, g, fout, str_dir2, str_f_prj,
                                    str_f_prof, str_f_info, str_lc,
                                    str_optimization, 8);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrGo9)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_optimization.equals("")) str_optimization="0";

                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out, apnd));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go9(out, g, fout, str_dir2, str_f_prj,
                                    str_f_prof, str_f_info,
                                    str_optimization, "", "", "", 9, "", "");

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrGo10)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_optimization.equals("")) str_optimization="0";

                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out, apnd));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go9(out, g, fout, str_dir2, str_f_prj,
                                    str_f_prof, str_f_info,
                                    str_optimization, 
                                    str_dep, str_lp, str_sr,
                                    10, "", "");

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrGo11)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_prof.equals(""))
              {
                out.println("Error: Parameter 'file_prof' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_optimization.equals("")) str_optimization="0";

                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out, apnd));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go9(out, g, fout, str_dir2, str_f_prj,
                                    str_f_prof, str_f_info,
                                    str_optimization, "", "", "", 11, "", "");

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrGo11x)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_prof.equals(""))
              {
                out.println("Error: Parameter 'file_prof' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_optimization.equals("")) str_optimization="0";

                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out, apnd));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go9(out, g, fout, str_dir2, str_f_prj,
                                    str_f_prof, str_f_info,
                                    str_optimization, "", "", "", 110, "", "");

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrGo12)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_optimization.equals("")) str_optimization="0";

                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                if (!str_prm.equals("0")) apnd=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out, apnd));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go9(out, g, fout, str_dir2, str_f_prj,
                                    str_f_prof, str_f_info,
                                    str_optimization, "", "", "", 12, str_prm, str_prm1);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else
          {
            out.println("Error: Unknown instruction!");
            out.flush();
          }
        }
      }
    }
    catch (IOException e) 
    {
//It may happen when connection is closed
//      System.out.println("Error: Problems during communication!");
      return;
    }
  }      

  //**************************************************************************

  void Go1(PrintWriter out,
           CGlobal g,
           BufferedWriter fout,
           String str_dir,
           String str_fp,
           String str_fpf,
           String str_ft,
           String str_opt)
  {
    int ret=0;

    String str1=str_dir+str_fp;

    //Delete output files if exist
    File fdel=null;
    try
    {
      fdel=new File(str_dir+str_fpf);
      fdel.delete();
    }
    catch (NullPointerException e1)
    {}

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(null, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    //Create CClient for Compilation Server
    CClient cl=new CClient();
    if (cl.Init(g.cfg.GetParamStr(g.cfgCSerHost),
                g.cfg.GetParamInt(g.cfgCSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    //Start output
    try
    {
      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Project filename                : ");
      fout.write(str_fp);
      fout.newLine();
      fout.write("Directory                       : ");
      fout.write(str_dir);
      fout.newLine();
      fout.write("Preliminary diagnostic file is  : ");
      fout.write(f_tmp);
      fout.newLine();
      fout.write("Optimization                    : ");
      fout.write(str_opt);
      fout.newLine();
      fout.newLine();
      fout.write("Compiling                       : ");
      fout.flush();
  
      cl.Add("initiate compilation",    true, true);
      cl.Add("directory="+str_dir,      true, true);
      cl.Add("file_project="+str_fp,    true, true);
      cl.Add("file_diag_out="+f_tmp,    true, true);
      cl.Add("compilation_mode=0",      true, true);
      cl.Add("no_checking=0",           true, true);
      cl.Add("profile=0",               true, true);
      cl.Add("optimizations="+str_opt,  true, true);
      cl.Add("compile",                 true, true);
      cl.Add("terminate",              false, true);
    
      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }
  
      fout.write("Ok");
      fout.newLine();

      fout.write("Running (cycle profiling) 1a    : ");
      fout.flush();

      cl.Add("initiate compilation",          true, true);
      cl.Add("directory="+str_dir,            true, true);
      cl.Add("file_project="+str_fp,          true, true);
      cl.Add("file_out_time="+f_tmp1,         true, true);
      cl.Add("file_out_comp="+f_tmp2,         true, true);
      cl.Add("file_out_prof="+str_fpf+"1a",   true, true);
      cl.Add("prof_type=1",                   true, true);
      cl.Add("runmode=0",                     true, true);
      cl.Add("run",                           true, true);
      cl.Add("terminate",                    false, true);
  
      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();

      if (!str_ft.equals(""))
      {
        fout.write("Creating tree                   : ");
        fout.flush();
  
        cl.Add("initiate compilation",      true, true);
        cl.Add("directory="+str_dir,        true, true);
        cl.Add("file_project="+str_fp,      true, true);
        cl.Add("file_diag_out="+f_tmp,      true, true);
        cl.Add("file_out="+str_ft,          true, true);
        cl.Add("tree",                      true, true);
        cl.Add("terminate",                false, true);
   
        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }
   
        fout.write("Ok");
        fout.newLine();
      }
 
      fout.write("*****************************************************************************");
      fout.newLine();
    }
    catch (Exception e2)
    {
      out.println("Error: Problems while writing diagnostic files!");
      out.flush();
      return;
    }
 
    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void Go2(PrintWriter out,
           CGlobal g,
           BufferedWriter fout,
           String str_dir,
           String str_fp,
           String str_fpf,
           String str_fi,
           String str_lc,
           String str_opt,
           int mode)
  {
    BufferedReader fin=null;

    int asm_id=g.cfg.GetParamInt(g.cfgAsmID);
    int ret=0;
    int i=0;

    String fprj=str_dir+str_fp;

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(null, fprj, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    //Create CClient for Compilation Server
    CClient cl=new CClient();
    if (cl.Init(g.cfg.GetParamStr(g.cfgCSerHost),
                g.cfg.GetParamInt(g.cfgCSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    //Start output
    try
    {
      if (mode==2)
      {
        fout.write("Reading parameters from file    : ");
        fout.flush();
      }

      String str1=str_dir+str_fi;
      String str2="";

      try
      {
        fin=new BufferedReader(new FileReader(str1));
      }
      catch (IOException e2)
      {
        fout.write("Error: Can't find info file!");
        fout.newLine();
        fout.flush();
        return;
      }

      int nf=0;  //number of files
      int nl=0;  //current number of procedures
      int nlt=0; //total number of procedures
      int nlm=0; //max number of procedures

      try
      {
        while ((str2=fin.readLine())!=null)
        {
          if (str2.length()>0)
          {
            if (str2.charAt(0)=='*')
            {
              nf++;
              if (nl>nlm) nlm=nl;
              nl=0;
            }
            else if (str2.charAt(0)=='#')
            {
              nl++;
              nlt++;
            }
          }
        }
        if (nl>nlm) nlm=nl;
      }
      catch (Exception e1)
      {
        try
        {
          fin.close();
        }
        catch (Exception e2)
        {}
        fout.write("Error: Problems reading info file!");
        fout.newLine();
        fout.flush();
        return;
      }
 
      try
      {
        fin.close();
      }
      catch (Exception e3)
      {}

      if (mode==2)
      {
        fout.write("Ok");
        fout.newLine();
      }

      if (mode==2)
      {
        fout.newLine();
        fout.write("Number of files                 : ");
        fout.write(String.valueOf(nf));
        fout.newLine();
        fout.write("Number of subroutines           : ");
        fout.write(String.valueOf(nlt));
        fout.newLine();
        fout.flush();
      }

      if (nlt<1)
      {
        fout.newLine();
        fout.write("Nothing to do!");
        fout.newLine();
        fout.flush();
      }
      else
      {
        String[]    Fil=new String [nf];
        int[]       LM =new int    [nf];
        String[][]  Ind=new String [nf][nlm];
        int[][]     L1 =new int    [nf][nlm];
        int[][]     L2 =new int    [nf][nlm];

        if (mode==6 || mode==7 || mode==8) str_fi+="1";

        str1=str_dir+str_fi;

        try
        {
          fin=new BufferedReader(new FileReader(str1));
        }
        catch (IOException e2)
        {
          fout.write("Error reading file with parameters");
          fout.newLine();
          fout.flush();
          return;
        }

        nf=0;  //number of files
        nl=0;  //current number of loops

        try
        {
          while ((str2=fin.readLine())!=null)
          {
            if (str2.length()>0)
            {
              if (str2.charAt(0)=='*')
              {
                Fil[nf]=str2.substring(1,str2.length());

                if (nf>0) LM[nf-1]=nl;

                nf++;
                nl=0;
              }
              else if (str2.charAt(0)=='#')
              {
                int i1=str2.indexOf(" ");
                int i2=str2.indexOf(" ", i1+1);

                Ind[nf-1][nl]=str2.substring(1,i1); 

                String s1=str2.substring(i1+1, i2);
                String s2=str2.substring(i2+1, str2.length());

                int l1=0;
                try
                {
                  l1=Integer.valueOf(s1).intValue();
                }
                catch (NumberFormatException nfe1)
                {}

                int l2=0;
                try
                {
                  l2=Integer.valueOf(s2).intValue();
                }
                catch (NumberFormatException nfe1)
                {}

                L1[nf-1][nl]=l1;
                L2[nf-1][nl]=l2;
                nl++;
              }
            }
          }
          LM[nf-1]=nl;
        }
        catch (Exception e1)
        {
          try
          {
            fin.close();
          }
          catch (Exception e2)
          {}
          fout.write("Error reading file with parameters");
          fout.newLine();
          fout.flush();
          return;
        }
 
        try
        {
          fin.close();
        }
        catch (Exception e3)
        {}

        if (mode==2)
        {
          //Compiling into asm files
          fout.newLine();
          fout.write("Compiling into asm              : ");
          fout.flush();
  
          String fprj1=str_dir+f_tmp_prj;
         
          CProject cp_new = new CProject();
          cp_new.Init(cp, fprj1);
  
          for (i=0; i<nf; i++)
          {
            String f2a=Fil[i];
            String f2b="";
  
            int i1=Fil[i].lastIndexOf(".");
            if (i1!=-1)
            {
              f2a=Fil[i].substring(0, i1);
              f2b=Fil[i].substring(i1, Fil[i].length());
            }
  
            cp_new.AddLine1(cp.GetLngNameOrig(0), f2a, f2b,
                            cp.GetExtraOpt(1));

          }
  
          if (cp_new.Save()<0)
          {
            fout.write("Error: Can't save new project!");
            fout.newLine();
            fout.flush();
            return;
          }
  
          cl.Add("initiate compilation",    true, true);
          cl.Add("directory="+str_dir,      true, true);
          cl.Add("file_project="+f_tmp_prj, true, true);
          cl.Add("file_diag_out="+f_tmp,    true, true);
          cl.Add("compilation_mode=3",      true, true);
          cl.Add("no_checking=1",           true, true);
          cl.Add("profile=0",               true, true);
          cl.Add("optimizations="+str_opt,  true, true);
          cl.Add("compile",                 true, true);
          cl.Add("terminate",              false, true);
      
          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }
    
          fout.write("Ok");
          fout.newLine();
        }
        else if (mode==3)
        {
          //making _mpc1 with known errors eliminated
          //making _mpc2 with separated loops
          //making _mpc3 with 2 procedures and save/restore procedures

          fout.write("Checking lcomm                  : "+str_lc);
          fout.newLine();
          fout.flush();

          fout.write("Separating loops in asm file    : ");
          fout.flush();

          BufferedWriter fout1=null;
          BufferedWriter fout2=null;
          BufferedWriter fout3=null;

          BufferedWriter ftmp=null;

          int i1=0;
          int i2=0;
          int i3=0;
          int j=0;
          String str3="";
          long lin=0;

          for (i=0; i<nf; i++)
          {
            //work with asm file
            String f2a=Fil[i];
            String f2b="";
  
            i1=Fil[i].lastIndexOf(".");
            if (i1!=-1)
            {
              f2a=Fil[i].substring(0, i1);
              f2b=Fil[i].substring(i1, Fil[i].length());
            }

            //preparing asm files
            str1=str_dir+f2a+g.cfg.GetParamStr(g.cfgAsmExt);
            str2=str_dir+f2a+f_mpc2+g.cfg.GetParamStr(g.cfgAsmExt);
            String xt=str_dir+f_tmp3;

            int[] label=new int[LM[i]];

            for (j=0; j<LM[i]; j++) label[j]=-1;

            String label1="";

            int lcomm=0;
            int lcomm9=0;
            String[] lcomm1=new String[32000];

            //starting tmp procedures
            for (j=0; j<LM[i]; j++)
            {
              try
              {
                ftmp =new BufferedWriter(new FileWriter(xt+"."+String.valueOf(j)));
              }
              catch (IOException e2)
              {
                fout.write("Error opening tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                ftmp.write("        .globl "+Ind[i][j]+"_");
                ftmp.newLine();

                ftmp.write("        .ent   "+Ind[i][j]+"_");
                ftmp.newLine();

                ftmp.write(Ind[i][j]+"_:");
                ftmp.newLine();

                ftmp.newLine();
              }
              catch (Exception e1)
              {
                try
                {
                  ftmp.close();
                }
                catch (Exception e2)
                {}
                fout.write("Error opening tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                ftmp.close();
              }
              catch (Exception e3)
              {}
            }

            //Separating subroutines
            try
            {
                fin=new BufferedReader(new FileReader(str1));
              fout2=new BufferedWriter(new FileWriter(str2));
            }
            catch (IOException e2)
            {
              fout.write("Error opening original asm file!");
              fout.newLine();
              fout.flush();
              return;
            }

            try
            {
              while ((str2=fin.readLine())!=null)
              {
                //check for .lcomm -> remove duplication
                if (str_lc.equals("1"))
                {
                  int lcomm5=0;

                  i1=str2.indexOf(".lcomm");
                  if (i1==-1)
                  {
                    i1=str2.indexOf(".comm");
                    if (i1!=-1) lcomm5=1;
                  }

                  if (i1!=-1)
                  {
                    i2=str2.indexOf(" ", i1+1);
                    if (i2!=-1)
                    {
                      String lcomm2=str2.substring(i1+8, i2);
                      if (lcomm5==1) lcomm2=str2.substring(i1+7, i2);
 
                      boolean found9=false;
  
                      for (int lc=0; lc<lcomm; lc++)
                      {
                        if (lcomm1[lc].equals(lcomm2)) 
                        {
                          found9=true;
                          break;
                        }
                      }

                      if (found9)
                      {
                        str2=str2.substring(0, i2)+
                        "MPC"+String.valueOf(lcomm9++)+
                        str2.substring(i2, str2.length());
                      }
                      else
                      {
                        lcomm1[lcomm]=lcomm2;
                        lcomm++;
                      }
                    }
                  }
                }
 
                lin=getLine(str2, asm_id);
                if (lin!=-1)
                {
                  boolean finish1=false;

                  while (!finish1)
                  {
                    finish1=true;

                    int j2=-1;
                    for (int j1=0; j1<LM[i]; j1++)           
                    {
                      if ((L1[i][j1]<=lin) && (lin<=L2[i][j1])) 
                      {
                        j2=j1;
                        break;
                      }
                    }

                    if (j2!=-1)
                    {
                      boolean finish=false;

                      try
                      {
                        ftmp =new BufferedWriter(new FileWriter(xt+"."+String.valueOf(j2), true));
                      }
                      catch (IOException e2)
                      {
                        fout.write("Error opening tmp file with subroutines!");
                        fout.newLine();
                        fout.flush();
                        return;
                      }

                      try
                      {
                        label[j2]++;
                        label1=String.valueOf(label[j2])+"_";

                        ftmp.write(" #-----------------------------------------------------------");
                        ftmp.newLine();

                        if (label[j2]!=0)
                        {
                          ftmp.write(Ind[i][j2]+"_"+label1+":");
                          ftmp.newLine();
                        }
                        else label1="";

                        ftmp.write(str2);
                        ftmp.newLine();

                        fout2.write("        br "+Ind[i][j2]+"_"+label1+"   #MPC");
                        fout2.newLine();

                        fout2.write(Ind[i][j2]+"_"+label1+"ret_:  #MPC");
                        fout2.newLine();

                      }
                      catch (Exception e1)
                      {
                        try
                        {
                          ftmp.close();
                        }
                        catch (Exception e2)
                        {}
                        fout.write("Error opening tmp file with subroutines!");
                        fout.newLine();
                        fout.flush();
                        return;
                      }

                      while (!finish && ((str2=fin.readLine())!=null))
                      {
                        str3=CMisc.tab2space(str2);
                        str3=str3.trim();

                        if (str3.charAt(0)=='.')
                        {
                          lin=getLine(str2, asm_id);
                          
                          if (lin==-1) 
                          {
                             finish=true;
                             finish1=true;

                             fout2.write(str2);
                             fout2.newLine();
                          }
                          else
                          {
                            if ((L1[i][j2]>lin) || (lin>L2[i][j2])) 
                            {
                              finish=true;
                              finish1=false;
                            }
                          }
                        }

                        if (!finish)
                        {
                          ftmp.write(str2);
                          ftmp.newLine();
                        }
                      }

                      try
                      {
                        ftmp.write("        br "+Ind[i][j2]+"_"+label1+"ret_");
                        ftmp.newLine();
                        ftmp.newLine();
                      }
                      catch (Exception e1)
                      {
                        try
                        {
                          ftmp.close();
                        }
                        catch (Exception e2)
                        {}
                        fout.write("Error opening tmp file with subroutines!");
                        fout.newLine();
                        fout.flush();
                        return;
                      }

                      try
                      {
                        ftmp.close();
                      }
                      catch (Exception e3)
                      {}
                    }
                    else
                    {
                      fout2.write(str2);
                      fout2.newLine();
                    }
                  }
                }
                else
                {
                  fout2.write(str2);
                  fout2.newLine();
                }
              }
            }
            catch (Exception e1)
            {
              try
              {
                fin.close();
                fout2.close();
              }
              catch (Exception e2)
              {}
              fout.write("Error reading original asm file!");
              fout.newLine();
              fout.flush();
              return;
            }

            //finishing tmp procedures
            for (j=0; j<LM[i]; j++)
            {
              try
              {
                ftmp=new BufferedWriter(new FileWriter(xt+"."+String.valueOf(j), true));
              }
              catch (IOException e2)
              {
                fout.write("Error opening tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                ftmp.newLine();

                ftmp.write("        .end "+Ind[i][j]+"_");
                ftmp.newLine();
              }
              catch (Exception e1)
              {
                try
                {
                  ftmp.close();
                }
                catch (Exception e2)
                {}
                fout.write("Error opening tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                ftmp.close();
              }
              catch (Exception e3)
              {}
            }

            //appending tmp procedures
            BufferedReader ftmp1=null;

            for (j=0; j<LM[i]; j++)
            {
              try
              {
                ftmp1=new BufferedReader(new FileReader(xt+"."+String.valueOf(j)));
              }
              catch (IOException e2)
              {
                fout.write("Error opening tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                fout2.write(" #MPC **********************************************************************");
                fout2.newLine();

                while ((str2=ftmp1.readLine())!=null)
                {
                  fout2.write(str2);
                  fout2.newLine();
                }
              }
              catch (Exception e1)
              {
                try
                {
                  ftmp1.close();
                }
                catch (Exception e2)
                {}
                fout.write("Error opening tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                ftmp1.close();
              }
              catch (Exception e3)
              {}
            }

            try
            {
              fin.close();
              fout2.close();
            }
            catch (Exception e3)
            {}
          }

          fout.write("Ok");
          fout.newLine();
        }
        else if (mode==4)
        {
          //Compiling transformed files
          fout.write("Compiling transformed files1    : ");
          fout.flush();
  
          String fprj1=str_dir+f_tmp_prj;
         
          CProject cp_new = new CProject();
          cp_new.Init(cp, fprj1);
  
          for (i=1; i<cp.GetNumFiles(); i++)
          {
            boolean found=false;

            int j=0;

            for (j=0; j<nf; j++)
            {
              if (Fil[j].equals(cp.GetFileName(i)+cp.GetFileExt(i)))
              {
                found=true;
                break;
              }
            }

            if (!found)
            {
              if (cp_new.AddLine1(cp.GetLngNameOrig(i), cp.GetFileName(i), cp.GetFileExt(i), cp.GetExtraOpt(i))<0)
              {
                out.println("Error: Too many files in new project file!");
                out.flush();
                return;
              }
            }
            else
            {
              String f2a=Fil[j];
              String f2b="";
  
              int i1=Fil[j].lastIndexOf(".");
              if (i1!=-1)
              {
                f2a=Fil[j].substring(0, i1);
                f2b=Fil[j].substring(i1, Fil[j].length());
              }

              cp_new.AddLine1(g.cfg.GetParamStr(g.cfgAsmPrjN), 
                              f2a+f_mpc2, 
                              g.cfg.GetParamStr(g.cfgAsmExt),
                              "");
            }
          }

          if (cp_new.Save()<0)
          {
            fout.write("Error: Can't save new project!");
            fout.newLine();
            fout.flush();
            return;
          }
  
          cl.Add("initiate compilation",    true, true);
          cl.Add("directory="+str_dir,      true, true);
          cl.Add("file_project="+f_tmp_prj, true, true);
          cl.Add("file_diag_out="+f_tmp,    true, true);
          cl.Add("compilation_mode=0",      true, true);
          cl.Add("no_checking=0",           true, true);
          cl.Add("profile=0",               true, true);
          cl.Add("optimizations="+str_opt,  true, true);
          cl.Add("compile",                 true, true);
          cl.Add("terminate",              false, true);

          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }
    
          fout.write("Ok");
          fout.newLine();

          fout.write("Running (cycle profiling) 2a    : ");
          fout.flush();
 
          cl.Add("initiate compilation",          true, true);
          cl.Add("directory="+str_dir,            true, true);
          cl.Add("file_project="+f_tmp_prj,       true, true);
          cl.Add("file_out_time="+f_tmp1,         true, true);
          cl.Add("file_out_comp="+f_tmp2,         true, true);
          cl.Add("file_out_prof="+str_fpf+"2a",   true, true);
          cl.Add("prof_type=1",                   true, true);
          cl.Add("runmode=1",                     true, true);
          cl.Add("run",                           true, true);
          cl.Add("terminate",                    false, true);
  
          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }

          fout.write("Ok");
          fout.newLine();

          fout.write("Running (instr. counting) 2b    : ");
          fout.flush();

          str1=str_dir+f_mpc_fb2a;
          BufferedWriter fout1=null;

          try
          {
            fout1=new BufferedWriter(new FileWriter(str1));
          }
          catch (IOException e2)
          {
            fout.write("Can't open _eos_mpc_fb2.dat for writing!");
            fout.newLine();
            fout.flush();
            return;
          }

          try
          {
            for (i=0; i<nf; i++)
            {
              for (int j=0; j<LM[i]; j++)
              {
                fout1.write(Ind[i][j]+"_");
                fout1.newLine();
              }
            }
          }
          catch (Exception e1)
          {
            try
            {
              fout1.close();
            }
            catch (Exception e2)
            {}
            fout.write("Error reading original file!");
            fout.newLine();
            fout.flush();
            return;
          }

          try
          {
            fout1.close();
          }
          catch (Exception e3)
          {}

          /* - for pixie
          cl.Add("initiate compilation",          true, true);
          cl.Add("directory="+str_dir,            true, true);
          cl.Add("file_project="+str_fp,          true, true);
          cl.Add("file_out_time="+f_tmp1,         true, true);
          cl.Add("file_out_comp="+f_tmp2,         true, true);
          cl.Add("file_out_prof="+str_fpf+"2b",   true, true);
          cl.Add("prof_type=2",                   true, true);
          cl.Add("runmode=1",                     true, true);
          cl.Add("run",                           true, true);
          cl.Add("terminate",                    false, true);
          */

          cl.Add("initiate compilation",          true, true);
          cl.Add("directory="+str_dir,            true, true);
          cl.Add("file_project="+str_fp,          true, true);
          cl.Add("tool=2",                        true, true);
          cl.Add("run_tool",                      true, true);
          cl.Add("terminate",                    false, true);
  
          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }
 
          cl.Add("initiate compilation",          true, true);
          cl.Add("directory="+str_dir,            true, true);
          cl.Add("file_project="+str_fp,          true, true);
          cl.Add("file_out_time="+f_tmp1,         true, true);
          cl.Add("runmode=3",                     true, true);
          cl.Add("param1="+str_fi,                true, true);
          cl.Add("tool=1",                        true, true);
          cl.Add("run",                           true, true);
          cl.Add("terminate",                    false, true);
  
          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }

          if (CMisc.CopyTextFile(str_dir+f_mpc_fb2b, str_dir+str_fpf+"2b")!=0)
          {
            fout.write("Error: Can't copy file with instr. counts!");
            fout.newLine();
            fout.flush();
            return;
          }

          fout.write("Ok");
          fout.newLine();

          fout.write("Parsing profiling info    2d    : ");
          fout.flush();

          str1=str_dir+str_fpf+"2b";
          str2=str_dir+str_fpf+"2d";

          int i1=0;
          int i2=0;

          BufferedWriter fout2=null;

          try
          {
              fin=new BufferedReader(new FileReader(str1));
            fout2=new BufferedWriter(new FileWriter(str2));
          }
          catch (IOException e2)
          {
            fout.write("Error opening original prof file!");
            fout.newLine();
            fout.flush();
            return;
          }

          try
          {
            while ((str2=fin.readLine())!=null)
            {
              String prc="";

              i1=str2.indexOf("proc={");
              if (i1!=-1)
              {
                i2=str2.indexOf("}", i1+1);
                if (i2!=-1)
                {
                  prc=str2.substring(i1+6, i2).trim();
                }
              }

              String instr="";

              i1=str2.indexOf("instr={");
              if (i1!=-1)
              {
                i2=str2.indexOf("}", i1+1);
                if (i2!=-1)
                {
                  instr=str2.substring(i1+7, i2).trim();
                }
              }

              long instr1=0;
              try
              {
                instr1=Long.valueOf(instr).longValue();
              }
              catch (NumberFormatException nfe1)
              {}

              boolean found=false;
              String tm="";
              String cyc="";
              BufferedReader fin1=null;
              str1=str_dir+str_fpf+"2a";

              try
              {
                fin1=new BufferedReader(new FileReader(str1));
              }
              catch (IOException e2)
              {
                fout.write("Error opening original prof file!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                while (((str2=fin1.readLine())!=null) && !found)
                {
                  String prc1="";

                  i1=str2.indexOf("'");
                  if (i1!=-1)
                  {
                    i2=str2.indexOf("'", i1+1);
                    if (i2!=-1)
                    {
                      prc1=str2.substring(i1+1, i2).trim()+"_";
                    }
                  }

                  if (prc.equals(prc1))
                  {
                    found=true;
  
                    i1=str2.indexOf("'", i2+1);
                    if (i1!=-1)
                    {
                      i2=str2.indexOf("'", i1+1);
                      if (i2!=-1)
                      {
                        tm=str2.substring(i1+1, i2).trim();;
                      }
                    }

                    i1=str2.indexOf("'", i2+1);
                    if (i1!=-1)
                    {
                      i2=str2.indexOf("'", i1+1);
                      if (i2!=-1)
                      {
                        cyc=str2.substring(i1+1, i2).trim();
                      }
                    }
                  }
                }
              }
              catch (Exception e1)
              {
                try
                {
                  fin1.close();
                }
                catch (Exception e2)
                {}
                fout.write("Error reading prof file!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                fin1.close();
              }
              catch (Exception e3)
              {}

              if (!found)
              {
                fout.write("Failed matching instructions in prof files!");
                fout.newLine();
                fout.flush();
                return;
              }

              long cyc1=0;
              try
              {
                cyc1=Long.valueOf(cyc).longValue();
              }
              catch (NumberFormatException nfe1)
              {}

              long ipc1=instr1*100;
              long ipc2=ipc1/cyc1;
              double ipc3=ipc2;
              double ipc=ipc3/100;

              fout2.write("proc={"+prc+"}, time={"+tm+"}, instr={"+
                          instr+"}, cycles={"+cyc+"}, IPC={"+
                          String.valueOf(ipc)+"}");
              fout2.newLine();
            }
          }
          catch (Exception e1)
          {
            try
            {
              fin.close();
              fout2.close();
            }
            catch (Exception e2)
            {}
            fout.write("Error reading prof file!");
            fout.newLine();
            fout.flush();
            return;
          }

          try
          {
            fin.close();
            fout2.close();
          }
          catch (Exception e3)
          {}

          fout.write("Ok");
          fout.newLine();
        }
        else if (mode==5)
        {
          BufferedWriter fout1=null;
          BufferedWriter fout2=null;
          BufferedWriter fout3=null;
          BufferedWriter fout4=null;
          BufferedWriter fout5=null;

          BufferedWriter ftmp=null;

          fout.write("Adding mpcdummy to source files : ");
          fout.flush();
  
          String str3=str_dir+str_fi+"1";
          String str4=str_dir+str_fi+"2";
          try
          {
            fout3=new BufferedWriter(new FileWriter(str3));
            fout4=new BufferedWriter(new FileWriter(str4));
          }
          catch (IOException e2)
          {
            fout.write("Error opening new info file!");
            fout.newLine();
            fout.flush();
            return;
          }

          for (i=0; i<nf; i++)
          {
            try
            {
              if (i!=0)
              {
                fout3.newLine();
                fout4.newLine();
              }

              fout3.write("*"+Fil[i]);
              fout3.newLine();
              fout3.newLine();

              fout4.write("*"+Fil[i]);
              fout4.newLine();
              fout4.newLine();
            }
            catch (IOException e2)
            {
              fout.write("Error writing new info file!");
              fout.newLine();
              fout.flush();
              return;
            }

            int j1=-1;
            for (int j=1; j<cp.GetNumFiles(); j++)
            {
              if (Fil[i].equals(cp.GetFileName(j)+cp.GetFileExt(j)))
              {
                j1=j;
                break;
              }
            }

            if (j1==-1)
            {
              fout.write("Can't find file from 'info file' in project!");
              fout.newLine();
              fout.flush();
              return;
            }

            String call="";

            if (asm_id==0)
            {
              if      (cp.GetLngNameOrig(j1).equals("f77")) call="      call mpcdummy";
              else if (cp.GetLngNameOrig(j1).equals("f90")) call="      call mpcdummy";
              else if (cp.GetLngNameOrig(j1).equals("c"))   call="mpcdummy_();";
              else if (cp.GetLngNameOrig(j1).equals("c++")) call="mpcdummy_();";
            }
            else
            {
              if      (cp.GetLngNameOrig(j1).equals("f77")) call="      call mpcdummy";
              else if (cp.GetLngNameOrig(j1).equals("f90")) call="      call mpcdummy";
              else if (cp.GetLngNameOrig(j1).equals("c"))   call="mpcdummy();";
              else if (cp.GetLngNameOrig(j1).equals("c++")) call="mpcdummy();";
            }

            String f2a=Fil[i];
            String f2b="";
  
            int i1=Fil[i].lastIndexOf(".");
            if (i1!=-1)
            {
              f2a=Fil[i].substring(0, i1);
              f2b=Fil[i].substring(i1, Fil[i].length());
            }

            str1=str_dir+f2a+f2b;
            str2=str_dir+f2a+f_mpc5+f2b;

            try
            {
                fin=new BufferedReader(new FileReader(str1));
              fout2=new BufferedWriter(new FileWriter(str2));
            }
            catch (IOException e2)
            {
              fout.write("Error opening original file!");
              fout.newLine();
              fout.flush();
              return;
            }

            int l=0;
            int l9=0;

            try
            {
              while ((str2=fin.readLine())!=null)
              {
                l++;
                l9++;

                int j2=-1;
                for (j1=0; j1<LM[i]; j1++)
                {
                  if ((L1[i][j1]<=l) && (l<=L2[i][j1]))
                  {
                    j2=j1;
                    break;
                  }
                }

                if (j2!=-1)
                {
                  try
                  {
                    fout3.write("#"+Ind[i][j2]+" "+String.valueOf(l9)+" ");
                    fout4.write("+"+Ind[i][j2]+" "+String.valueOf(l9)+" ");
                  }
                  catch (IOException e2)
                  {
                    fout.write("Error writing new info file!");
                    fout.newLine();
                    fout.flush();
                    return;
                  }

                  fout2.write(call);
                  fout2.newLine();
                  l9++;

                  try
                  {
                    fout5=new BufferedWriter(new FileWriter(str_dir+f2a+f_mpc5+"_proc."+Ind[i][j2]));
                  }
                  catch (IOException e2)
                  {
                    fout.write("Error opening original file!");
                    fout.newLine();
                    fout.flush();
                    return;
                  }

                  fout2.write(str2);
                  fout2.newLine();

                  fout5.write(str2);
                  fout5.newLine();
                
                  for (; l<L2[i][j2]; l++)
                  {
                    l9++;
                    str2=fin.readLine();

                    fout2.write(str2);
                    fout2.newLine();

                    fout5.write(str2);
                    fout5.newLine();
                  }

                  try
                  {
                    fout5.close();
                  }
                  catch (Exception e2)
                  {}

                  fout2.write(call);
                  fout2.newLine();
                  l9++;

                  try
                  {
                    fout3.write(String.valueOf(l9));
                    fout3.newLine();

                    fout4.write(String.valueOf(l9));
                    fout4.newLine();
                  }
                  catch (IOException e2)
                  {
                    fout.write("Error writing new info file!");
                    fout.newLine();
                    fout.flush();
                    return;
                  }
                }
                else
                {
                  fout2.write(str2);
                  fout2.newLine();
                }
              }
            }
            catch (Exception e1)
            {
              try
              {
                fin.close();
                fout2.close();
              }
              catch (Exception e2)
              {}
              fout.write("Error reading original file!");
              fout.newLine();
              fout.flush();
              return;
            }

            try
            {
              fin.close();
              fout2.close();
            }
            catch (Exception e3)
            {}
          }

          try
          {
            fout3.close();
            fout4.close();
          }
          catch (Exception e3)
          {}

          fout.write("Ok");
          fout.newLine();
        }
        else if (mode==6)
        {
          BufferedWriter fout1=null;
          BufferedWriter fout2=null;
          BufferedWriter fout3=null;

          BufferedWriter ftmp=null;

          //Compiling into asm files
          fout.write("Compiling into asm              : ");
          fout.flush();
  
          String fprj1=str_dir+f_tmp_prj;
         
          CProject cp_new = new CProject();
          cp_new.Init(cp, fprj1);
  
          for (i=0; i<nf; i++)
          {
            String f2a=Fil[i];
            String f2b="";
  
            int i1=Fil[i].lastIndexOf(".");
            if (i1!=-1)
            {
              f2a=Fil[i].substring(0, i1);
              f2b=Fil[i].substring(i1, Fil[i].length());
            }
  
            cp_new.AddLine1(cp.GetLngNameOrig(0), f2a+f_mpc5, f2b,
                            cp.GetExtraOpt(1));

          }
  
          if (cp_new.Save()<0)
          {
            fout.write("Error: Can't save new project!");
            fout.newLine();
            fout.flush();
            return;
          }
  
          cl.Add("initiate compilation",    true, true);
          cl.Add("directory="+str_dir,      true, true);
          cl.Add("file_project="+f_tmp_prj, true, true);
          cl.Add("file_diag_out="+f_tmp,    true, true);
          cl.Add("compilation_mode=3",      true, true);
          cl.Add("no_checking=1",           true, true);
          cl.Add("profile=0",               true, true);
          cl.Add("optimizations="+str_opt,  true, true);
          cl.Add("compile",                 true, true);
          cl.Add("terminate",              false, true);
      
          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }
    
          fout.write("Ok");
          fout.newLine();
        }
        else if (mode==7)
        {
          BufferedWriter fout1=null;
          BufferedWriter fout2=null;
          BufferedWriter fout3=null;

          BufferedWriter ftmp=null;

          fout.write("Checking lcomm                  : "+str_lc);
          fout.newLine();
          fout.flush();

          fout.write("Separating loops in asm file    : ");
          fout.flush();

          int i1=0;
          int i2=0;
          int i3=0;
          int j=0;
          String str3="";
          long lin=0;

          for (i=0; i<nf; i++)
          {
            //work with asm file
            String f2a=Fil[i];
            String f2b="";
  
            i1=Fil[i].lastIndexOf(".");
            if (i1!=-1)
            {
              f2a=Fil[i].substring(0, i1);
              f2b=Fil[i].substring(i1, Fil[i].length());
            }

            //preparing asm files
            str1=str_dir+f2a+f_mpc5+g.cfg.GetParamStr(g.cfgAsmExt);
            str2=str_dir+f2a+f_mpc5a+g.cfg.GetParamStr(g.cfgAsmExt);
            String xt=str_dir+f_tmp3;

            //starting tmp procedures
            for (j=0; j<LM[i]; j++)
            {
              try
              {
                ftmp =new BufferedWriter(new FileWriter(xt+"."+String.valueOf(j)));
              }
              catch (IOException e2)
              {
                fout.write("Error opening tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                if (asm_id==0)
                {
                  ftmp.write("        .globl "+Ind[i][j]+"_");
                  ftmp.newLine();

                  ftmp.write("        .ent   "+Ind[i][j]+"_");
                  ftmp.newLine();

                  ftmp.write(Ind[i][j]+"_:");
                  ftmp.newLine();

                  ftmp.newLine();
                }
                else if (asm_id==1)
                {
                  ftmp.write("_"+Ind[i][j]+" PROC NEAR");
                  ftmp.newLine();

                  ftmp.newLine();
                }
              }
              catch (Exception e1)
              {
                try
                {
                  ftmp.close();
                }
                catch (Exception e2)
                {}
                fout.write("Error writing header of tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                ftmp.close();
              }
              catch (Exception e3)
              {}
            }

            //Separating subroutines
            try
            {
                fin=new BufferedReader(new FileReader(str1));
              fout2=new BufferedWriter(new FileWriter(str2));
            }
            catch (IOException e2)
            {
              fout.write("Error opening original asm file!");
              fout.newLine();
              fout.flush();
              return;
            }

            long lin_last=0;
            int lcomm=0;
            int lcomm9=0;
            String[] lcomm1=new String[32000];

            try
            {
              if (asm_id==0)
              {
                while ((str2=fin.readLine())!=null)
                {
                  //check for .lcomm -> remove duplication
                  if (str_lc.equals("1"))
                  {
                   int lcomm5=0;
             
                    i1=str2.indexOf(".lcomm");
                    if (i1==-1)
                    {
                      i1=str2.indexOf(".comm");
                      if (i1!=-1) lcomm5=1;
                    }

                    if (i1!=-1)
                    {
                      i2=str2.indexOf(" ", i1+1);
                      if (i2!=-1)
                      {
                        String lcomm2=str2.substring(i1+8, i2);
                        if (lcomm5==1) lcomm2=str2.substring(i1+7, i2);
 
                        boolean found9=false;
  
                        for (int lc=0; lc<lcomm; lc++)
                        {
                          if (lcomm1[lc].equals(lcomm2)) 
                          {
                            found9=true;
                            break;
                          }
                        }

                        if (found9)
                        {
                          str2=str2.substring(0, i2)+
                          "MPC"+String.valueOf(lcomm9++)+
                          str2.substring(i2, str2.length());
                        }
                        else
                        {
                          lcomm1[lcomm]=lcomm2;
                          lcomm++;
                        }
                      }
                    }
                  }

                  lin=getLine(str2, asm_id);
                  if (lin!=-1) 
                  {
                    lin_last=lin;

                    fout2.write(str2);
                    fout2.newLine();
                  }
                  else
                  {
                    if ((str2.indexOf("jsr")!=-1) && (str2.indexOf("mpcdummy_")!=-1))
                    {
                      //looking whom it belongs to...

                      int j2=-1;
                      for (int j1=0; j1<LM[i]; j1++)           
                      {
                        if ((L1[i][j1]<=lin_last) && (lin_last<=L2[i][j1])) 
                        {
                          j2=j1;
                          break;
                        }
                      }

                      if (j2!=-1)
                      {
                        try
                        {
                          ftmp =new BufferedWriter(new FileWriter(xt+"."+String.valueOf(j2), true));
                        }
                        catch (IOException e2)
                        {
                          fout.write("Error opening tmp file with subroutines!");
                          fout.newLine();
                          fout.flush();
                          return;
                        }

                        try
                        {
                          ftmp.write(str2);
                          ftmp.newLine();

                          fout2.write("        br "+Ind[i][j2]+"_   #MPC");
                          fout2.newLine();

                          fout2.write(Ind[i][j2]+"_ret_:   #MPC");
                          fout2.newLine();
                        }
                        catch (Exception e1)
                        {
                          try
                          {
                            ftmp.close();
                          }
                          catch (Exception e2)
                          {}
                          fout.write("Error opening tmp file with subroutines!");
                          fout.newLine();
                          fout.flush();
                          return;
                        }

                        boolean finish=false;
                        while (!finish && ((str2=fin.readLine())!=null))
                        {
                          if ((str2.indexOf("ldq")!=-1) && (str2.indexOf("mpcdummy_")!=-1))
                          {
                            try
                            {
                              ftmp.write("        br "+Ind[i][j2]+"_ret_");
                              ftmp.newLine();
                              ftmp.newLine();

                              fout2.write(str2);
                              fout2.newLine();
                            }
                            catch (Exception e1)
                            {
                              try
                              {
                                ftmp.close();
                              }
                              catch (Exception e2)
                              {}
                              fout.write("Error opening tmp file with subroutines!");
                              fout.newLine();
                              fout.flush();
                              return;
                            }

                            try
                            {
                              ftmp.close();
                            }
                            catch (Exception e3)
                            {}

                            //looking for last mpcdummy
                            finish=false;
                            while (!finish && ((str2=fin.readLine())!=null))
                            {
                              fout2.write(str2);
                              fout2.newLine();

                              if ((str2.indexOf("jsr")!=-1) && (str2.indexOf("mpcdummy_")!=-1))
                              {
                                finish=true;
                                break;
                              }
                            }
                 
                            finish=true;
                          }
                          else
                          {
                            ftmp.write(str2);
                            ftmp.newLine();
                          }
                        }
                      }
                    }
                    else
                    {
                      fout2.write(str2);
                      fout2.newLine();
                    }
                  }
                }
              }
              else if (asm_id==1)
              {
                while ((str2=fin.readLine())!=null)
                {
                  if (str2.equals("\tEND"))
                  {
                    fout2.write(";MPC  "+str2);
                    fout2.newLine();
                  }
                  else if (str2.indexOf("_MPCDUMMY")!=-1 || str2.indexOf("_mpcdummy")!=-1)
                  {
                    if (str2.indexOf("EXTRN")==-1)
                    {
                      lin=getLine(str2, asm_id);
                      if (lin!=-1)
                      {
                        //looking whom it belongs to...

                        int j2=-1;
                        for (int j1=0; j1<LM[i]; j1++)           
                        {
                          if ((L1[i][j1]<=lin) && (lin<=L2[i][j1])) 
                          {
                            j2=j1;
                            break;
                          }
                        }

                        if (j2!=-1)
                        {
                          try
                          {
                            ftmp =new BufferedWriter(new FileWriter(xt+"."+String.valueOf(j2), true));
                          }
                          catch (IOException e2)
                          {
                            fout.write("Error opening tmp file with subroutines!");
                            fout.newLine();
                            fout.flush();
                            return;
                          }

                          try
                          {
                            fout2.write(";MPC "+str2);
                            fout2.newLine();
              
                            fout2.write(";MPC ---------------------------------------------------------------");
                            fout2.newLine();

                            fout2.write("  jmp _"+Ind[i][j2]+"  ;MPC");
                            fout2.newLine();

                            fout2.write("_"+Ind[i][j2]+"_ret LABEL NEAR  ;MPC");
                            fout2.newLine();

                            fout2.write(";MPC ---------------------------------------------------------------");
                            fout2.newLine();
                          }
                          catch (Exception e1)
                          {
                            try
                            {
                              ftmp.close();
                            }
                            catch (Exception e2)
                            {}
                            fout.write("Error opening tmp file with subroutines!");
                            fout.newLine();
                            fout.flush();
                            return;
                          }

                          boolean finish=false;
                          while (!finish && ((str2=fin.readLine())!=null))
                          {
                            if (str2.indexOf("_MPCDUMMY")!=-1 || str2.indexOf("_mpcdummy")!=-1)
                            {
                              try
                              {
                                ftmp.write("  jmp _"+Ind[i][j2]+"_ret");
                                ftmp.newLine();
                                ftmp.newLine();
             
                                fout2.write(";MPC  "+str2);
                                fout2.newLine();
                              }
                              catch (Exception e1)
                              {
                                try
                                {
                                  ftmp.close();
                                }
                                catch (Exception e2)
                                {}
                                fout.write("Error opening tmp file with subroutines!");
                                fout.newLine();
                                fout.flush();
                                return;
                              }

                              try
                              {
                                ftmp.close();
                              }
                              catch (Exception e3)
                              {}

                              finish=true;
                            }
                            else
                            {
                              ftmp.write(str2);
                              ftmp.newLine();
                            }
                          }
                        }
                        else
                        {
                          fout2.write(str2);
                          fout2.newLine();
                        }
                      }
                      else
                      {
                        fout2.write(str2);
                        fout2.newLine();
                      }
                    }
                    else
                    {
                      fout2.write(";MPC  "+str2);
                      fout2.newLine();
                    }
                  }
                  else
                  {
                    fout2.write(str2);
                    fout2.newLine();
                  }
                }
              }
            }
            catch (Exception e1)
            {
              try
              {
                fin.close();
                fout2.close();
              }
              catch (Exception e2)
              {}
              fout.write("Error reading original asm file!");
              fout.newLine();
              fout.flush();
              return;
            }

            try
            {
              fin.close();
              fout2.close();
            }
            catch (Exception e3)
            {}

            //Checking tmp procedures for labels
            BufferedReader ftmp1=null;

            String[] lbl_1=new String[32000];
            int[]    lbl_p=new int[32000];

            boolean finish_tr=false;

            String flt1=str_dir+f2a+f_mpc5a+g.cfg.GetParamStr(g.cfgAsmExt);
            int first=0;

            while (!finish_tr)
            {
              int lbl=-1;
              for (j=0; j<LM[i]; j++)
              {
                try
                {
                  ftmp1=new BufferedReader(new FileReader(xt+"."+String.valueOf(j)));
                }
                catch (IOException e2)
                {
                  fout.write("Error opening tmp file with subroutines!");
                  fout.newLine();
                  fout.flush();
                  return;
                }
        
                try
                {
                  while ((str2=ftmp1.readLine())!=null)
                  {
                    String lb2=getLbl2(str2, asm_id);
                    if (!lb2.equals(""))
                    {
                      lbl++;
                      lbl_1[lbl]=lb2;
                      lbl_p[lbl]=j;
                    }
                  }
                }
                catch (Exception e1)
                {
                  try
                  {
                    ftmp1.close();
                  }
                  catch (Exception e2)
                  {}
                  fout.write("Error opening tmp file with subroutines!");
                  fout.newLine();
                  fout.flush();
                  return;
                }

                try
                {
                  ftmp1.close();
                }
                catch (Exception e3)
                {}
              }

              lbl++;
      
              int lbl1=lbl;
              if (lbl!=0)
              {
                for (j=0; j<LM[i]; j++)
                {
                  try
                  {
                    ftmp1=new BufferedReader(new FileReader(xt+"."+String.valueOf(j)));
                  }
                  catch (IOException e2)
                  {
                    fout.write("Error opening tmp file with subroutines!");
                    fout.newLine();
                    fout.flush();
                    return;
                  }

                  try
                  {
                    while ((str2=ftmp1.readLine())!=null)
                    {
                      String lb=getLbl1(str2, asm_id);
                      if (!lb.equals(""))
                      {
                        for (int z=0; z<lbl; z++)
                        {
                          if ((lbl_p[z]!=-1) && lb.equals(lbl_1[z]))
                          {
                            lbl_p[z]=-1;
                            lbl1--;
                          }
                        }
                      }
                    }
                  }
                  catch (Exception e1)
                  {
                    try
                    {
                      ftmp1.close();
                    }
                    catch (Exception e2)
                    {}
                    fout.write("Error opening tmp file with subroutines!");
                    fout.newLine();
                    fout.flush();
                    return;
                  }

                  try
                  {
                    ftmp1.close();
                  }
                  catch (Exception e3)
                  {}
                }
              }
              
              if (lbl1<1) finish_tr=true;

              str2=str_dir+f2a+f_mpc5b1+g.cfg.GetParamStr(g.cfgAsmExt);

              try
              {
                  fin=new BufferedReader(new FileReader(flt1));
               fout2=new BufferedWriter(new FileWriter(str2));
              }
              catch (IOException e2)
              {
                fout.write("Error opening transformed source code!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                while ((str2=fin.readLine())!=null)
                {
                  if (lbl1>0)
                  {
                    String lb=getLbl1(str2, asm_id);
                    if (!lb.equals(""))
                    {
                      boolean found=false;

                      for (int z=0; z<lbl; z++)
                      {
                        if ((lbl_p[z]!=-1) && lb.equals(lbl_1[z]))
                        {
                          found=true;
 
                          try
                          {
                            ftmp =new BufferedWriter(new FileWriter(xt+"."+String.valueOf(lbl_p[z]), true));
                          }
                          catch (IOException e2)
                          {
                            fout.write("Error opening tmp file with subroutines!");
                            fout.newLine();
                            fout.flush();
                            return;
                          }

                          try
                          {
                            if (asm_id==0)
                            {
                              ftmp.write(" #MPC --------------------------------");
                            }
                            else
                            {
                              ftmp.write(";MPC --------------------------------");
                            }
                            ftmp.newLine();

                            ftmp.write(str2);
                            ftmp.newLine();

                            boolean finish=false;
                            while (!finish && ((str2=fin.readLine())!=null))
                            {
                              ftmp.write(str2);
                              ftmp.newLine();

                              if (asm_id==0)
                              {
                                if (str2.indexOf("\tbr")!=-1)
                                {              
                                  finish=true;
                                  break;
                                }
                              }
                              else if (asm_id==1)
                              {
                                if (str2.indexOf(" jmp ")!=-1)
                                {              
                                  finish=true;
                                  break;
                                }
                              }
                            }
                          }
                          catch (Exception e1)
                          {
                            try
                            {
                              ftmp.close();
                            }
                            catch (Exception e2)
                            {}
                            fout.write("Error opening tmp file with subroutines!");
                            fout.newLine();
                            fout.flush();
                            return;
                          }

                          try
                          {
                            ftmp.close();
                          }
                          catch (Exception e3)
                          {}
                        }
                      }
                      
                      if (!found)
                      {
                        fout2.write(str2);
                        fout2.newLine();
                      }
                    }
                    else
                    {
                      fout2.write(str2);
                      fout2.newLine();
                    }
                  }
                  else
                  {
                    fout2.write(str2);
                    fout2.newLine();
                  }
                }
              }
              catch (Exception e1)
              {
                try
                {
                  fin.close();
                  fout2.close();
                }  
                catch (Exception e2)
                {}
                fout.write("Error working with transformed source code!");
                fout.newLine();
                fout.flush();
                return;
              }  
 
              try
              {
                fin.close();
                fout2.close();
              }
              catch (Exception e3)
              {}

              if (first==0)
              {
                flt1=str_dir+f2a+f_mpc5b+g.cfg.GetParamStr(g.cfgAsmExt);
                first=1;
              }

              //copying from mpc5b1 to mpc5b
              str1=str_dir+f2a+f_mpc5b1+g.cfg.GetParamStr(g.cfgAsmExt);
              str2=str_dir+f2a+f_mpc5b+g.cfg.GetParamStr(g.cfgAsmExt);

              try
              {
                  fin=new BufferedReader(new FileReader(str1));
                fout2=new BufferedWriter(new FileWriter(str2));
              }
              catch (IOException e2)
              {
                fout.write("Error opening transformed source code!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                while ((str2=fin.readLine())!=null)
                {
                  fout2.write(str2);
                  fout2.newLine();
                }
              }
              catch (Exception e1)
              {
                try
                {
                  fin.close();
                  fout2.close();
                }  
                catch (Exception e2)
                {}
                fout.write("Error working with transformed source code!");
                fout.newLine();
                fout.flush();
                return;
              }  
 
              try
              {
                fin.close();
                fout2.close();
              }
              catch (Exception e3)
              {}
            }

            //finishing tmp procedures
            for (j=0; j<LM[i]; j++)
            {
              try
              {
                ftmp=new BufferedWriter(new FileWriter(xt+"."+String.valueOf(j), true));
              }
              catch (IOException e2)
              {
                fout.write("Error opening tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                ftmp.newLine();

                if (asm_id==0)
                {
                  ftmp.write("        .end "+Ind[i][j]+"_");
                }
                else if (asm_id==1)
                {
                  ftmp.write("_"+Ind[i][j]+" ENDP");
                }
 
                ftmp.newLine();
              }
              catch (Exception e1)
              {
                try
                {
                  ftmp.close();
                }
                catch (Exception e2)
                {}
                fout.write("Error opening tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                ftmp.close();
              }
              catch (Exception e3)
              {}
            }

            //appending tmp procedures
            str2=str_dir+f2a+f_mpc5b+g.cfg.GetParamStr(g.cfgAsmExt);

            try
            {
              fout2=new BufferedWriter(new FileWriter(str2, true));
            }
            catch (IOException e2)
            {
              fout.write("Error opening transformed source code!");
              fout.newLine();
              fout.flush();
              return;
            }

            if (asm_id==1)
            {
              try
              {
                fout2.write("_TEXT  SEGMENT DWORD PUBLIC USE32 'CODE'");
                fout2.newLine();
              }
              catch (Exception e1)
              {
                try
                {
                  fout2.close();
                }
                catch (Exception e2)
                {}
                fout.write("Error opening tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }
            }

            for (j=0; j<LM[i]; j++)
            {
              try
              {
                ftmp1=new BufferedReader(new FileReader(xt+"."+String.valueOf(j)));
              }
              catch (IOException e2)
              {
                fout.write("Error opening tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                if (asm_id==0)
                {
                  fout2.write(" #MPC **********************************************************************");
                  fout2.newLine();
                }
                else if (asm_id==1)
                {
                  fout2.write(";MPC ***********************************************************************");
                  fout2.newLine();
                }

                while ((str2=ftmp1.readLine())!=null)
                {
                  fout2.write(str2);
                  fout2.newLine();
                }
              }
              catch (Exception e1)
              {
                try
                {
                  ftmp1.close();
                }
                catch (Exception e2)
                {}
                fout.write("Error opening tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                ftmp1.close();
              }
              catch (Exception e3)
              {}
            }

            if (asm_id==1)
            {
              try
              {
                fout2.write("_TEXT  ENDS");
                fout2.newLine();

                fout2.write("END");
                fout2.newLine();
              }
              catch (Exception e1)
              {
                try
                {
                  fout2.close();
                }
                catch (Exception e2)
                {}
                fout.write("Error opening tmp file with subroutines!");
                fout.newLine();
                fout.flush();
                return;
              }
            }

            try
            {
              fin.close();
              fout2.close();
            }
            catch (Exception e3)
            {}
          }

          str2=str_dir+f_mpc_d+g.cfg.GetParamStr(g.cfgAsmExt);

          try
          {
            fout2=new BufferedWriter(new FileWriter(str2));
          }
          catch (IOException e2)
          {
            fout.write("Error opening transformed source code!");
            fout.newLine();
            fout.flush();
            return;
          }

          try
          {
            fout2.write(" #MPC **********************************************************************");
            fout2.newLine();
            fout2.write("	.globl  mpcdummy_");
            fout2.newLine();
            fout2.write("	.ent 	mpcdummy_");
            fout2.newLine();
            fout2.write("mpcdummy_:");
            fout2.newLine();
            fout2.write("	.frame  $sp, 0, $26");
            fout2.newLine();
            fout2.write("	ret	($26)");
            fout2.newLine();
            fout2.write("	.end 	mpcdummy_");
            fout2.newLine();
            fout2.write(" #MPC **********************************************************************");
            fout2.newLine();
          }
          catch (Exception e1)
          {
            try
            {
              fout2.close();
            }
            catch (Exception e2)
            {}
            fout.write("Error writing transformed asm file!");
            fout.newLine();
            fout.flush();
            return;
          }

          try
          {
            fout2.close();
          }
          catch (Exception e3)
          {}

          fout.write("Ok");
          fout.newLine();
        }
        else if (mode==8)
        {
          //Compiling transformed files
          fout.write("Compiling transformed files2    : ");
          fout.flush();
  
          String fprj1=str_dir+f_tmp_prj;
         
          CProject cp_new = new CProject();
          cp_new.Init(cp, fprj1);
  
          for (i=1; i<cp.GetNumFiles(); i++)
          {
            boolean found=false;

            int j=0;

            for (j=0; j<nf; j++)
            {
              if (Fil[j].equals(cp.GetFileName(i)+cp.GetFileExt(i)))
              {
                found=true;
                break;
              }
            }

            if (!found)
            {
              if (cp_new.AddLine1(cp.GetLngNameOrig(i), cp.GetFileName(i), cp.GetFileExt(i), cp.GetExtraOpt(i))<0)
              {
                out.println("Error: Too many files in new project file!");
                out.flush();
                return;
              }
            }
            else
            {
              String f2a=Fil[j];
              String f2b="";
  
              int i1=Fil[j].lastIndexOf(".");
              if (i1!=-1)
              {
                f2a=Fil[j].substring(0, i1);
                f2b=Fil[j].substring(i1, Fil[j].length());
              }

              cp_new.AddLine1(g.cfg.GetParamStr(g.cfgAsmPrjN), 
                              f2a+f_mpc5b, 
                              g.cfg.GetParamStr(g.cfgAsmExt),
                              "");
            }
          }

          if (asm_id==0)
          {
            cp_new.AddLine1(g.cfg.GetParamStr(g.cfgAsmPrjN),
                            f_mpc_d, 
                            g.cfg.GetParamStr(g.cfgAsmExt),
                            "");
          }

          if (cp_new.Save()<0)
          {
            fout.write("Error: Can't save new project!");
            fout.newLine();
            fout.flush();
            return;
          }
  
          cl.Add("initiate compilation",    true, true);
          cl.Add("directory="+str_dir,      true, true);
          cl.Add("file_project="+f_tmp_prj, true, true);
          cl.Add("file_diag_out="+f_tmp,    true, true);
          cl.Add("compilation_mode=0",      true, true);
          cl.Add("no_checking=0",           true, true);
          cl.Add("profile=0",               true, true);
          cl.Add("optimizations="+str_opt,  true, true);
          cl.Add("compile",                 true, true);
          cl.Add("terminate",              false, true);

          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }
    
          fout.write("Ok");
          fout.newLine();

          fout.write("Running (cycle profiling) 3a    : ");
          fout.flush();
 
          if (asm_id==0)
          {
            cl.Add("initiate compilation",          true, true);
            cl.Add("directory="+str_dir,            true, true);
            cl.Add("file_project="+f_tmp_prj,       true, true);
            cl.Add("file_out_time="+f_tmp1,         true, true);
            cl.Add("file_out_comp="+f_tmp2,         true, true);
            cl.Add("file_out_prof="+str_fpf+"3a",   true, true);
            cl.Add("prof_type=1",                   true, true);
            cl.Add("runmode=1",                     true, true);
            cl.Add("run",                           true, true);
            cl.Add("terminate",                    false, true);
          }
          else if (asm_id==1)
          {
            cl.Add("initiate compilation",          true, true);
            cl.Add("directory="+str_dir,            true, true);
            cl.Add("file_project="+f_tmp_prj,       true, true);
            cl.Add("file_out_time="+f_tmp1,         true, true);
            cl.Add("file_out_comp="+f_tmp2,         true, true);
            cl.Add("runmode=1",                     true, true);
            cl.Add("run",                           true, true);
            cl.Add("terminate",                    false, true);
          } 
  
          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }

          fout.write("Ok");
          fout.newLine();

          if (asm_id==0)
          {
            fout.write("Running (cycle profiling) 3b    : ");
            fout.flush();

            str1=str_dir+f_mpc_fb2a;
            BufferedWriter fout1=null;

            try
            {
              fout1=new BufferedWriter(new FileWriter(str1));
            }
            catch (IOException e2)
            {
              fout.write("Can't open _eos_mpc_fb2.dat for writing!");
              fout.newLine();
              fout.flush();
              return;
            }

            try
            {
              for (i=0; i<nf; i++)
              {
                for (int j=0; j<LM[i]; j++)
                {
                  fout1.write(Ind[i][j]+"_");
                  fout1.newLine();
                }
              }
            }
            catch (Exception e1)
            {
              try
              {
                fout1.close();
              }
              catch (Exception e2)
              {}
              fout.write("Error reading original file!");
              fout.newLine();
              fout.flush();
              return;
            }

            try
            {
              fout1.close();
            }
            catch (Exception e3)
            {}

            cl.Add("initiate compilation",          true, true);
            cl.Add("directory="+str_dir,            true, true);
            cl.Add("file_project="+str_fp,          true, true);
            cl.Add("tool=2",                        true, true);
            cl.Add("run_tool",                      true, true);
            cl.Add("terminate",                    false, true);

            ret=cl.Send();
            if (ret==-1)
            {
              out.println("Error: Can't find host with Compilation Server!");
              out.flush();
              return;
            }
            else if (ret==-2)
            {
              out.println("Error: Can't connect to Compilation Server!");
              out.flush();
              return;
            }
            else if (ret==-4)
            {
              out.println("Error: Can't close connection with Compilation Server!");
              out.flush();
              return;
            }  
            else if (ret<0)
            {
              out.println(cl.GetLastError());
              out.flush();
              return;
            }

            cl.Add("initiate compilation",          true, true);
            cl.Add("directory="+str_dir,            true, true);
            cl.Add("file_project="+str_fp,          true, true);
            cl.Add("file_out_time="+f_tmp1,         true, true);
            cl.Add("runmode=3",                     true, true);
            cl.Add("param1="+str_fi,                true, true);
            cl.Add("tool=1",                        true, true);
            cl.Add("run",                           true, true);
            cl.Add("terminate",                    false, true);
  
            ret=cl.Send();
            if (ret==-1)
            {
              out.println("Error: Can't find host with Compilation Server!");
              out.flush();
              return;
            }
            else if (ret==-2)
            {
              out.println("Error: Can't connect to Compilation Server!");
              out.flush();
              return;
            }
            else if (ret==-4)
            {
              out.println("Error: Can't close connection with Compilation Server!");
              out.flush();
              return;
            }
            else if (ret<0)
            {
              out.println(cl.GetLastError());
              out.flush();
              return;
            }

            if (CMisc.CopyTextFile(str_dir+f_mpc_fb2b, str_dir+str_fpf+"3b")!=0)
            {
              fout.write("Error: Can't copy file with instr. counts!");
              fout.newLine();
              fout.flush();
              return;
            }

            fout.write("Ok");
            fout.newLine();
          
            fout.write("Parsing profiling info    3d    : ");
            fout.flush();

            str1=str_dir+str_fpf+"3b";
            str2=str_dir+str_fpf+"3d";

            int i1=0;
            int i2=0;

            BufferedWriter fout2=null;

            try
            {
                fin=new BufferedReader(new FileReader(str1));
              fout2=new BufferedWriter(new FileWriter(str2));
            }
            catch (IOException e2)
            {
              fout.write("Error opening original prof file!");
              fout.newLine();
              fout.flush();
             return;
            } 

            try
            {
              while ((str2=fin.readLine())!=null)
              {
                String prc="";

                i1=str2.indexOf("proc={");
                if (i1!=-1)
                {
                  i2=str2.indexOf("}", i1+1);
                  if (i2!=-1)
                  {
                    prc=str2.substring(i1+6, i2).trim();
                  }
                }

                String instr="";

                i1=str2.indexOf("instr={");
                if (i1!=-1)
                {
                  i2=str2.indexOf("}", i1+1);
                  if (i2!=-1)
                  {
                    instr=str2.substring(i1+7, i2).trim();
                  }
                }

                long instr1=0;
                try
                {
                  instr1=Long.valueOf(instr).longValue();
                }
                catch (NumberFormatException nfe1)
                {}

                boolean found=false;
                String tm="";
                String cyc="";
                BufferedReader fin1=null;
                str1=str_dir+str_fpf+"3a";

                try
                {
                  fin1=new BufferedReader(new FileReader(str1));
                }
                catch (IOException e2)
                {
                  fout.write("Error opening original prof file!");
                  fout.newLine();
                  fout.flush();
                  return;
                }
     
                try
                {
                  while (((str2=fin1.readLine())!=null) && !found)
                  {
                    String prc1="";

                    i1=str2.indexOf("'");
                    if (i1!=-1)
                    {
                      i2=str2.indexOf("'", i1+1);
                      if (i2!=-1)
                      {
                        prc1=str2.substring(i1+1, i2).trim()+"_";
                      }
                    }

                    if (prc.equals(prc1))
                    {
                      found=true;
  
                      i1=str2.indexOf("'", i2+1);
                      if (i1!=-1)
                      {
                        i2=str2.indexOf("'", i1+1);
                        if (i2!=-1)
                        {
                          tm=str2.substring(i1+1, i2).trim();;
                        }
                      }

                      i1=str2.indexOf("'", i2+1);
                      if (i1!=-1)
                      {
                        i2=str2.indexOf("'", i1+1);
                        if (i2!=-1)
                        {
                          cyc=str2.substring(i1+1, i2).trim();
                        }
                      }
                    }
                  }
                }
                catch (Exception e1)
                {
                  try
                  {
                    fin1.close();
                  }
                  catch (Exception e2)
                  {}
                  fout.write("Error reading prof file!");
                  fout.newLine();
                  fout.flush();
                  return;
                }

                try
                {
                  fin1.close();
                }
                catch (Exception e3)
                {}

                if (!found)
                {
                  fout.write("Failed matching instructions in prof files!");
                  fout.newLine();
                  fout.flush();
                  return;
                }

                long cyc1=0;
                try
                {
                  cyc1=Long.valueOf(cyc).longValue();
                }
                catch (NumberFormatException nfe1)
                {}

                long ipc1=instr1*100;
                long ipc2=ipc1/cyc1;
                double ipc3=ipc2;
                double ipc=ipc3/100;

                fout2.write("proc={"+prc+"}, time={"+tm+"}, instr={"+
                            instr+"}, cycles={"+cyc+"}, IPC={"+
                            String.valueOf(ipc)+"}");
                fout2.newLine();
              }
            }
            catch (Exception e1)
            {
              try
              {
                fin.close();
                fout2.close();
              }
              catch (Exception e2)
              {}
              fout.write("Error reading prof file!");
              fout.newLine();
              fout.flush();
              return;
            }

            try
            {
              fin.close();
              fout2.close();
            }
            catch (Exception e3)
            {}

            fout.write("Ok");
            fout.newLine();
          }
        }
      }
 
      fout.write("*****************************************************************************");
      fout.newLine();
    }
    catch (Exception e2)
    {
      out.println("Error: Problems while writing diagnostic files!");
      out.flush();
      return;
    }
 
    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  private static long getLine(String str, int asm_id)
  {
    long lin=0;
    String str1="";

    if (asm_id==0)
    {
      if (str.indexOf(".loc")==-1) return -1;

      int i=str.lastIndexOf(" ");

      str1=str.substring(i+1, str.length());
      lin=0;
      try
      {
        lin=Long.valueOf(str1).longValue();
      }
      catch (NumberFormatException nfe1)
      {}
    }
    else if (asm_id==1)
    {
      int x=str.lastIndexOf(";");
      if (x==-1) return -1;

      int y=str.indexOf(".", x);
      if (y==-1) return -1;

      str1=str.substring(x+1, y);
      lin=0;
      try
      {
        lin=Long.valueOf(str1).longValue();
      }
      catch (NumberFormatException nfe1)
      {}
    }

    return lin;
  }

  //**************************************************************************
  private static String getLbl1(String str, int asm_id)
  {
    String str1="";

    if (str.equals("")) return "";

    int i1=str.indexOf(":");
    if (i1==-1) return "";

    int i2=str.length();

    if (asm_id==0)
    {
      if (((i2>2) && str.substring(0,2).equals("L$")) ||
          ((i2>2) && str.substring(0,2).equals("$$")) ||
          ((i2>4) && str.substring(0,4).equals("lab$")))
      {
        str1=str.substring(0,i1);
      }
    }
    else if (asm_id==1)
    {
      if (((i2>2) && str.substring(0,2).equals("L$")) ||
          ((i2>2) && str.substring(0,2).equals("$B")))
      {
        str1=str.substring(0,i1);
      }
    }

    return str1;
  }

  //**************************************************************************
  private static String getLbl2(String str, int asm_id)
  {
    String str1="";

    if (str.equals("")) return "";

    int i1=0;
    int i2=0;

    if (asm_id==0)
    {
      if ((i1=str.indexOf(" L$"))!=-1)
      {
        i2=str.indexOf("\t", i1+1);
        if (i2==-1) i2=str.length();
 
        str1=str.substring(i1+1, i2);
      }
      else if ((i1=str.indexOf(" lab$"))!=-1)
      {
        i2=str.indexOf("\t", i1+1);
        if (i2==-1) i2=str.length();

        str1=str.substring(i1+1, i2);
      }
      else if ((i1=str.indexOf("$$L"))!=-1)
      {
        i1=str.indexOf(" $$");
        if (i1!=-1)
        {
          i2=str.indexOf("\t", i1+1);
          if (i2==-1) i2=str.length();

          str1=str.substring(i1+1, i2);
        }
      }
    }
    else if (asm_id==1)
    {
      if ((i1=str.indexOf(" L$"))!=-1)
      {
        i2=str.indexOf(" ", i1+1);
      }
      else if ((i1=str.indexOf(" $B"))!=-1)
      {
        i2=str.indexOf(" ", i1+1);
      }

      int i3=str.indexOf(";");
      if (i1>i3) return "";

      if (i2==-1) i2=str.length();
      str1=str.substring(i1+1, i2);
    }

    return str1;
  }

  //**************************************************************************
  void Go9(PrintWriter out,
           CGlobal g,
           BufferedWriter fout,
           String str_dir,
           String str_fp,
           String str_fpf,
           String str_fi,
           String str_opt,
           String str_dep,
           String str_lp,
           String str_sr,
           int mode,
           String str_prm,
           String str_prm1)
  {
    BufferedReader fin=null;
    BufferedReader fin1=null;

    int asm_id=g.cfg.GetParamInt(g.cfgAsmID);

    int ret=0;
    int i=0;
    int j=0;
    int k=0;

    int i1=0;
    int i2=0;

    String fprj=str_dir+str_fp;

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(null, fprj, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    //Create CClient for Compilation Server
    CClient cl=new CClient();
    if (cl.Init(g.cfg.GetParamStr(g.cfgCSerHost),
                g.cfg.GetParamInt(g.cfgCSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    //Start output
    try
    {
      if (mode==9)
      {
        fout.write("Prepare feedback utility        : ");
        fout.flush();

        cl.Add("initiate compilation",          true, true);
        cl.Add("directory="+str_dir,            true, true);
        cl.Add("file_project="+str_fp,          true, true);
        cl.Add("param1="+str_fi,                true, true);
        cl.Add("tool=1",                        true, true);
        cl.Add("run_tool",                      true, true);
        cl.Add("terminate",                    false, true);
  
        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }

        fout.write("Ok");
        fout.newLine();

        //if windows -> compile it from asm
        if (asm_id==1)
        {
          // this is copy of the code which is used for mode=10 and 11
          // just because of lack of time

          String str1=str_dir+str_fi+"2";
          String str2="";

          try
          {
            fin=new BufferedReader(new FileReader(str1));
          }
          catch (IOException e2)
          {
            fout.write("Error: Can't find info2 file!");
            fout.newLine();
            fout.flush();
            return;
          }

          int nf=0;  //number of files
          int nl=0;  //current number of procedures
          int nlt=0; //total number of procedures
          int nlm=0; //max number of procedures
  
          try
          {
            while ((str2=fin.readLine())!=null)
            {
              if (str2.length()>0)
              {
                if (str2.charAt(0)=='*')
                {
                  nf++;
                  if (nl>nlm) nlm=nl;
                  nl=0;
                }
                else if (str2.charAt(0)=='+')
                {
                  nl++;
                  nlt++;
                }
              }
            }
            if (nl>nlm) nlm=nl;
          }
          catch (Exception e1)
          {
            try
            {
              fin.close();
            }
            catch (Exception e2)
            {}
            fout.write("Error: Problems reading info2 file!");
            fout.newLine();
            fout.flush();
            return;
          }
    
          try
          {
            fin.close();
          }
          catch (Exception e3)
          {}
 
          String[]    Fil=new String [nf];
          int[]       LM =new int    [nf];
          String[][]  Ind=new String [nf][nlm];
          int[][]     L1 =new int    [nf][nlm];
          int[][]     L2 =new int    [nf][nlm];

          str1=str_dir+str_fi+"2";

          try
          {
            fin=new BufferedReader(new FileReader(str1));
          }
          catch (IOException e2)
          {
            fout.write("Error reading file with parameters");
            fout.newLine();
            fout.flush();
            return;
          }
    
          nf=0;  //number of files
          nl=0;  //current number of procedures

          try
          {
            while ((str2=fin.readLine())!=null)
            {
              if (str2.length()>0)
              {
                if (str2.charAt(0)=='*')
                {
                  Fil[nf]=str2.substring(1,str2.length());
 
                  if (nf>0) LM[nf-1]=nl;
  
                  nf++;
                  nl=0;
                }
                else if (str2.charAt(0)=='+')
                {
                  i1=str2.indexOf(" ");
                  i2=str2.indexOf(" ", i1+1);
 
                  Ind[nf-1][nl]=str2.substring(1,i1); 
 
                  String s1=str2.substring(i1+1, i2);
                  String s2=str2.substring(i2+1, str2.length());
  
                  int l1=0;
                  try
                  {
                    l1=Integer.valueOf(s1).intValue();
                  }
                  catch (NumberFormatException nfe1)
                  {}
 
                  int l2=0;
                  try
                  {
                    l2=Integer.valueOf(s2).intValue();
                  }
                  catch (NumberFormatException nfe1)
                  {}

                  L1[nf-1][nl]=l1;
                  L2[nf-1][nl]=l2;
                  nl++;
                }
              }
            }
            LM[nf-1]=nl;
          }
          catch (Exception e1)
          {
            try
            {
              fin.close();
            }
            catch (Exception e2)
            {}
            fout.write("Error reading file with parameters");
            fout.newLine();
            fout.flush();
            return;
          }
  
          try
          {
            fin.close();
          }
          catch (Exception e3)
          {}

          //Compiling transformed files
          fout.write("Compiling (FATOM for Windows)   : ");
          fout.flush();
  
          String fprj1=str_dir+f_tmp_prj;
            
          CProject cp_new = new CProject();
          cp_new.Init(cp, fprj1);
   
          for (i=1; i<cp.GetNumFiles(); i++)
          {
            boolean found=false;
      
            j=0;
       
            for (j=0; j<nf; j++)
            {
              if (Fil[j].equals(cp.GetFileName(i)+cp.GetFileExt(i)))
              {
                found=true;
                break;
              }
            }

            if (!found)
            {
              if (cp_new.AddLine1(cp.GetLngNameOrig(i), cp.GetFileName(i), cp.GetFileExt(i), cp.GetExtraOpt(i))<0)
              {
                out.println("Error: Too many files in new project file!");
                out.flush();
                return;
              }
            }
            else
            {
              String f2a=Fil[j];
              String f2b="";

              i1=Fil[j].lastIndexOf(".");
              if (i1!=-1)
              {
                f2a=Fil[j].substring(0, i1);
                f2b=Fil[j].substring(i1, Fil[j].length());
              }

              cp_new.AddLine1(g.cfg.GetParamStr(g.cfgAsmPrjN), 
                              f2a+f_mpc5atom, 
                              g.cfg.GetParamStr(g.cfgAsmExt),
                              "");
            }
          }

          cp_new.AddLine1(g.cfg.GetParamStr(g.cfgCPrjN),
                          f_mpc_d1, 
                          g.cfg.GetParamStr(g.cfgCExt),
                          "");

          cp_new.AddLine1(g.cfg.GetParamStr(g.cfgCPrjN),
                          f_mpc_d2, 
                          g.cfg.GetParamStr(g.cfgCExt),
                          "");

          if (cp_new.Save()<0)
          {
            fout.write("Error: Can't save new project!");
            fout.newLine();
            fout.flush();
            return;
          }
  
          cl.Add("initiate compilation",    true, true);
          cl.Add("directory="+str_dir,      true, true);
          cl.Add("file_project="+f_tmp_prj, true, true);
          cl.Add("file_diag_out="+f_tmp,    true, true);
          cl.Add("compilation_mode=0",      true, true);
          cl.Add("no_checking=0",           true, true);
          cl.Add("profile=0",               true, true);
          cl.Add("optimizations="+str_opt,  true, true);
          cl.Add("compile",                 true, true);
          cl.Add("terminate",              false, true);

          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }  
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }
    
          fout.write("Ok");
          fout.newLine();
        }

        fout.write("Run feedback utility            : ");
        fout.flush();

        cl.Add("initiate compilation",          true, true);
        cl.Add("directory="+str_dir,            true, true);
        cl.Add("file_project="+str_fp,          true, true);
        cl.Add("file_out_time="+f_tmp1,         true, true);
        cl.Add("runmode=3",                     true, true);
        cl.Add("param1="+str_fi,                true, true);
        cl.Add("tool=1",                        true, true);
        cl.Add("run",                           true, true);
        cl.Add("terminate",                    false, true);
  
        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }

        fout.write("Ok");
        fout.newLine();
      }
      else if (mode==12)
      {
        fout.write("Prepare feedback utility        : ");
        fout.flush();

        cl.Add("initiate compilation",          true, true);
        cl.Add("directory="+str_dir,            true, true);
        cl.Add("file_project="+str_fp,          true, true);
        cl.Add("param="+str_prm,                true, true);
//        cl.Add("param1="+str_prm1,              true, true);
        cl.Add("param1="+str_fi,                true, true);
        cl.Add("tool=4",                        true, true);
        cl.Add("run_tool",                      true, true);
        cl.Add("terminate",                    false, true);
  
        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }

        fout.write("Ok");
        fout.newLine();

        //if windows -> compile it from asm
        if (asm_id==1)
        {
          // this is copy of the code which is used for mode=10 and 11
          // just because of lack of time

          String str1=str_dir+str_fi+"2";
          String str2="";

          try
          {
            fin=new BufferedReader(new FileReader(str1));
          }
          catch (IOException e2)
          {
            fout.write("Error: Can't find info2 file!");
            fout.newLine();
            fout.flush();
            return;
          }

          int nf=0;  //number of files
          int nl=0;  //current number of procedures
          int nlt=0; //total number of procedures
          int nlm=0; //max number of procedures
  
          try
          {
            while ((str2=fin.readLine())!=null)
            {
              if (str2.length()>0)
              {
                if (str2.charAt(0)=='*')
                {
                  nf++;
                  if (nl>nlm) nlm=nl;
                  nl=0;
                }
                else if (str2.charAt(0)=='+')
                {
                  nl++;
                  nlt++;
                }
              }
            }
            if (nl>nlm) nlm=nl;
          }
          catch (Exception e1)
          {
            try
            {
              fin.close();
            }
            catch (Exception e2)
            {}
            fout.write("Error: Problems reading info2 file!");
            fout.newLine();
            fout.flush();
            return;
          }
    
          try
          {
            fin.close();
          }
          catch (Exception e3)
          {}
 
          String[]    Fil=new String [nf];
          int[]       LM =new int    [nf];
          String[][]  Ind=new String [nf][nlm];
          int[][]     L1 =new int    [nf][nlm];
          int[][]     L2 =new int    [nf][nlm];

          str1=str_dir+str_fi+"2";

          try
          {
            fin=new BufferedReader(new FileReader(str1));
          }
          catch (IOException e2)
          {
            fout.write("Error reading file with parameters");
            fout.newLine();
            fout.flush();
            return;
          }
    
          nf=0;  //number of files
          nl=0;  //current number of procedures

          try
          {
            while ((str2=fin.readLine())!=null)
            {
              if (str2.length()>0)
              {
                if (str2.charAt(0)=='*')
                {
                  Fil[nf]=str2.substring(1,str2.length());
 
                  if (nf>0) LM[nf-1]=nl;
  
                  nf++;
                  nl=0;
                }
                else if (str2.charAt(0)=='+')
                {
                  i1=str2.indexOf(" ");
                  i2=str2.indexOf(" ", i1+1);
 
                  Ind[nf-1][nl]=str2.substring(1,i1); 
 
                  String s1=str2.substring(i1+1, i2);
                  String s2=str2.substring(i2+1, str2.length());
  
                  int l1=0;
                  try
                  {
                    l1=Integer.valueOf(s1).intValue();
                  }
                  catch (NumberFormatException nfe1)
                  {}
 
                  int l2=0;
                  try
                  {
                    l2=Integer.valueOf(s2).intValue();
                  }
                  catch (NumberFormatException nfe1)
                  {}

                  L1[nf-1][nl]=l1;
                  L2[nf-1][nl]=l2;
                  nl++;
                }
              }
            }
            LM[nf-1]=nl;
          }
          catch (Exception e1)
          {
            try
            {
              fin.close();
            }
            catch (Exception e2)
            {}
            fout.write("Error reading file with parameters");
            fout.newLine();
            fout.flush();
            return;
          }
  
          try
          {
            fin.close();
          }
          catch (Exception e3)
          {}

          //Compiling transformed files
          fout.write("Compiling (FATOM for Windows)   : ");
          fout.flush();
  
          String fprj1=str_dir+f_tmp_prj;
            
          CProject cp_new = new CProject();
          cp_new.Init(cp, fprj1);
   
          for (i=1; i<cp.GetNumFiles(); i++)
          {
            boolean found=false;
      
            j=0;
       
            for (j=0; j<nf; j++)
            {
              if (Fil[j].equals(cp.GetFileName(i)+cp.GetFileExt(i)))
              {
                found=true;
                break;
              }
            }

            if (!found)
            {
              if (cp_new.AddLine1(cp.GetLngNameOrig(i), cp.GetFileName(i), cp.GetFileExt(i), cp.GetExtraOpt(i))<0)
              {
                out.println("Error: Too many files in new project file!");
                out.flush();
                return;
              }
            }
            else
            {
              String f2a=Fil[j];
              String f2b="";

              i1=Fil[j].lastIndexOf(".");
              if (i1!=-1)
              {
                f2a=Fil[j].substring(0, i1);
                f2b=Fil[j].substring(i1, Fil[j].length());
              }

              cp_new.AddLine1(g.cfg.GetParamStr(g.cfgAsmPrjN), 
                              f2a+f_mpc5atom, 
                              g.cfg.GetParamStr(g.cfgAsmExt),
                              "");
            }
          }

          cp_new.AddLine1(g.cfg.GetParamStr(g.cfgCPrjN),
                          f_mpc_d1, 
                          g.cfg.GetParamStr(g.cfgCExt),
                          "");

          cp_new.AddLine1(g.cfg.GetParamStr(g.cfgCPrjN),
                          f_mpc_d3, 
                          g.cfg.GetParamStr(g.cfgCExt),
                          "");

          if (cp_new.Save()<0)
          {
            fout.write("Error: Can't save new project!");
            fout.newLine();
            fout.flush();
            return;
          }
  
          cl.Add("initiate compilation",    true, true);
          cl.Add("directory="+str_dir,      true, true);
          cl.Add("file_project="+f_tmp_prj, true, true);
          cl.Add("file_diag_out="+f_tmp,    true, true);
          cl.Add("compilation_mode=0",      true, true);
          cl.Add("no_checking=0",           true, true);
          cl.Add("profile=0",               true, true);
          cl.Add("optimizations="+str_opt,  true, true);
          cl.Add("compile",                 true, true);
          cl.Add("terminate",              false, true);

          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }  
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }
    
          fout.write("Ok");
          fout.newLine();
        }

        fout.write("Run feedback utility            : ");
        fout.flush();

        cl.Add("initiate compilation",          true, true);
        cl.Add("directory="+str_dir,            true, true);
        cl.Add("file_project="+str_fp,          true, true);
        cl.Add("file_out_time="+f_tmp1,         true, true);
        cl.Add("runmode=3",                     true, true);
        cl.Add("param1="+str_fi,                true, true);
        cl.Add("tool=1",                        true, true);
        cl.Add("run",                           true, true);
        cl.Add("terminate",                    false, true);
  
        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }

        fout.write("Ok");
        fout.newLine();
        fout.newLine();

        if (str_prm.equals("0"))
        {
          String str1=str_dir+f_tmp_atom+"6a";

          if (CMisc.isFileExist(str1))
          {
            try
            {
              fin=new BufferedReader(new FileReader(str1));
            }
            catch (IOException e2)
            {
              fout.write("Error: Can't find file with number of memory accesses!");
              fout.newLine();
              fout.flush();
              return;
            }

            String str2="";
            try
            {
              while ((str2=fin.readLine())!=null)
              {
                fout.write(str2);
                fout.newLine();
              }
            }
            catch (Exception e1)
            {
              try
              {
                fin.close();
              }
              catch (Exception e2)
              {}
              fout.write("Error: Problems reading file with number of memory accesses!");
              fout.newLine();
              fout.flush();
              return;
            }
   
            try
            {
              fin.close();
            }
            catch (Exception e3)
            {}
          }
          else
          {
            fout.write("Error: Can't find file with number of memory accesses!");
            fout.newLine();
          }
        }

        if (!str_prm.equals("0"))
        {
          String str1=str_dir+f_tmp_atom+"6c";

          if (CMisc.isFileExist(str1))
          {
            try
            {
              fin=new BufferedReader(new FileReader(str1));
            }
            catch (IOException e2)
            {
              fout.write("Error: Can't find file with number of memory accesses!");
              fout.newLine();
              fout.flush();
              return;
            }

            String str2="";
            try
            {
              while ((str2=fin.readLine())!=null)
              {
                fout.write(str2);
                fout.newLine();
              }
            }
            catch (Exception e1)
            {
              try
              {
                fin.close();
              }
              catch (Exception e2)
              {}
              fout.write("Error: Problems reading file with number of memory accesses!");
              fout.newLine();
              fout.flush();
              return;
            }
   
            try
            {
              fin.close();
            }
            catch (Exception e3)
            {}
          }
          else
          {
            fout.write("Error: Can't find file with number of memory accesses!");
            fout.newLine();
          }
        }
      }
      else if (mode==10 || mode==11 || mode==110)
      {
        if (mode==10)
        {
          fout.write("Reading parameters from file1   : ");
          fout.flush();
        }

        String str1=str_dir+str_fi+"2";
        String str2="";

        try
        {
          fin=new BufferedReader(new FileReader(str1));
        }
        catch (IOException e2)
        {
          fout.write("Error: Can't find info2 file!");
          fout.newLine();
          fout.flush();
          return;
        }

        int nf=0;  //number of files
        int nl=0;  //current number of procedures
        int nlt=0; //total number of procedures
        int nlm=0; //max number of procedures
 
        try
        {
          while ((str2=fin.readLine())!=null)
          {
            if (str2.length()>0)
            {
              if (str2.charAt(0)=='*')
              {
                nf++;
                if (nl>nlm) nlm=nl;
                nl=0;
              }
              else if (str2.charAt(0)=='+')
              {
                nl++;
                nlt++;
              }
            }
          }
          if (nl>nlm) nlm=nl;
        }
        catch (Exception e1)
        {
          try
          {
            fin.close();
          }
          catch (Exception e2)
          {}
          fout.write("Error: Problems reading info2 file!");
          fout.newLine();
          fout.flush();
          return;
        }
 
        try
        {
          fin.close();
        }
        catch (Exception e3)
        {}
 
        if (mode==10)
        {
          fout.write("Ok");
          fout.newLine();

          fout.newLine();
          fout.write("Number of files                 : ");
          fout.write(String.valueOf(nf));
          fout.newLine();
          fout.write("Number of subroutines           : ");
          fout.write(String.valueOf(nlt));
          fout.newLine();
          fout.flush();
        }

        if (nlt<1)
        {
          fout.newLine();
          fout.write("Nothing to do!");
          fout.newLine();
          fout.flush();
        }
        else
        {
          String[]    Fil=new String [nf];
          int[]       LM =new int    [nf];
          String[][]  Ind=new String [nf][nlm];
          int[][]     L1 =new int    [nf][nlm];
          int[][]     L2 =new int    [nf][nlm];

          str1=str_dir+str_fi+"2";

          try
          {
            fin=new BufferedReader(new FileReader(str1));
          }
          catch (IOException e2)
          {
            fout.write("Error reading file with parameters");
            fout.newLine();
            fout.flush();
            return;
          }
    
          nf=0;  //number of files
          nl=0;  //current number of procedures

          try
          {
            while ((str2=fin.readLine())!=null)
            {
              if (str2.length()>0)
              {
                if (str2.charAt(0)=='*')
                {
                  Fil[nf]=str2.substring(1,str2.length());
 
                  if (nf>0) LM[nf-1]=nl;
  
                  nf++;
                  nl=0;
                }
                else if (str2.charAt(0)=='+')
                {
                  i1=str2.indexOf(" ");
                  i2=str2.indexOf(" ", i1+1);
 
                  Ind[nf-1][nl]=str2.substring(1,i1); 
 
                  String s1=str2.substring(i1+1, i2);
                  String s2=str2.substring(i2+1, str2.length());
  
                  int l1=0;
                  try
                  {
                    l1=Integer.valueOf(s1).intValue();
                  }
                  catch (NumberFormatException nfe1)
                  {}
 
                  int l2=0;
                  try
                  {
                    l2=Integer.valueOf(s2).intValue();
                  }
                  catch (NumberFormatException nfe1)
                  {}

                  L1[nf-1][nl]=l1;
                  L2[nf-1][nl]=l2;
                  nl++;
                }
              }
            }
            LM[nf-1]=nl;
          }
          catch (Exception e1)
          {
            try
            {
              fin.close();
            }
            catch (Exception e2)
            {}
            fout.write("Error reading file with parameters");
            fout.newLine();
            fout.flush();
            return;
          }
  
          try
          {
            fin.close();
          }
          catch (Exception e3)
          {}

          if (mode==10 || mode==110)
          {
            fout.write("Dependency                      : "+str_dep);
            fout.newLine();

            fout.write("Loop Cache                      : "+str_lp);
            fout.newLine();

            fout.write("Stack Register                  : "+str_sr);
            fout.newLine();

            fout.flush();
          }

          if (mode==10)
          {
            fout.newLine();
            fout.write("Reading ATOM files              : ");
            fout.flush();

            int proc_max=0;
            int inst=0;
            int inst_max=0;

            for (i=0; i<nf; i++)
            {
              for (j=0; j<LM[i]; j++)
              {
                String Fil1=str_dir+f_tmp_atom+"5."+String.valueOf(proc_max);

                try
                {
                  fin=new BufferedReader(new FileReader(Fil1));
                }
                catch (IOException e2)
                {  
                  fout.write("Error reading atom file");
                  fout.newLine();
                  fout.flush();
                  return;
                }

                inst=0;
                try
                {
                  while ((str2=fin.readLine())!=null) inst++;
                }
                catch (Exception e1)
                {
                  try
                  {
                    fin.close();
                  }
                  catch (Exception e2)
                  {}
                  fout.write("Error reading atom file");
                  fout.newLine();
                  fout.flush();
                  return;
                }
  
                try
                {
                  fin.close();
                }
                catch (Exception e3)
                {}

                if (inst>inst_max) inst_max=inst;

                proc_max++;
              }
            }
          
            String[] Ind1=new String [proc_max];

            str1=str_dir+f_tmp_atom+"1";

            try
            {
              fin=new BufferedReader(new FileReader(str1));
            }
            catch (IOException e2)
            {
              fout.write("Error reading ATOM1 file");
              fout.newLine();
              fout.flush();
              return;
            }
    
            int zz=0;
            try
            {
              while ((str2=fin.readLine())!=null)
              {
                if (str2.length()>0)
                {
                  i1=str2.indexOf("proc='");
                  i2=str2.indexOf("'", i1+6);

                  if ((i1!=-1) && (i2!=-1)) 
                  {
                    Ind1[zz]=str2.substring(i1+6,i2);
                    zz++;
                  }
                }
              }
            }
            catch (Exception e1)
            {
              try
              {
                fin.close();
              }
              catch (Exception e2)
              {}
              fout.write("Error reading ATOM1 file");
              fout.newLine();
              fout.flush();
              return;
            }
   
            try
            {
              fin.close();
            }
            catch (Exception e3)
            {}

            fout.write("Ok");
            fout.newLine();

            fout.write("Preparing data & offset files   : ");
            fout.flush();
    
            j=0;
  
            int[][]    Offset = new int    [proc_max][inst_max];
            String[][] Addr   = new String [proc_max][inst_max];

            for (i=0; i<proc_max; i++)
            {
              for (j=0; j<inst_max; j++)
              {
                Offset[i][j]=0;
                Addr[i][j]="";
              }
            }
  
            int proc=0;
            BufferedWriter fout2=null;
            BufferedWriter fout3=null;
            BufferedWriter fout4=null;
          
            for (i=0; i<nf; i++)
            {
              for (j=0; j<LM[i]; j++)
              {
                String Fil1=str_dir+f_tmp_atom+"5."+String.valueOf(proc);
                String Fil2=str_dir+f_tmp_atom+"5."+String.valueOf(proc)+"a";
                String Fil3=str_dir+f_tmp_atom+"5."+String.valueOf(proc)+"b";
                String Fil4=str_dir+f_tmp_atom+"5."+String.valueOf(proc)+"c";
        
                try
                {
                  fin=new BufferedReader(new FileReader(Fil1));
                  fout2=new BufferedWriter(new FileWriter(Fil2));
                  fout3=new BufferedWriter(new FileWriter(Fil3));
                  fout4=new BufferedWriter(new FileWriter(Fil4));
                }
                catch (IOException e2)
                {
                  fout.write("Error reading atom file");
                  fout.newLine();
                  fout.flush();
                  return;
                }
            
                inst=0;
        
                int q=0;
    
                int offset1=0;
               
                try
                {
                  if (asm_id==0)
                  {
                    fout2.write(" #MPC Data *********************************************************");
                    fout2.newLine();
                    fout2.newLine();
   
                    fout2.write("$$MPC"+String.valueOf(proc+1)+"A:");
                    fout2.newLine();
                    fout2.newLine();
            
                    fout3.write(" #MPC Data *********************************************************");
                    fout3.newLine();
                    fout3.newLine();
    
                    fout3.write("$$MPC"+String.valueOf(proc+1)+"B:");
                    fout3.newLine();
                    fout3.newLine();
                  }
                  else if (asm_id==1)
                  {
                    fout2.write(";MPC Data *********************************************************");
                    fout2.newLine();
                    fout2.newLine();

//                    fout2.write(" ALIGN 10H");
//                    fout2.newLine();
   
                    fout2.write("$$MPC"+String.valueOf(proc+1)+"A:");
                    fout2.newLine();
            
                    fout3.write(";MPC Data *********************************************************");
                    fout3.newLine();
                    fout3.newLine();
    
                    fout3.write("$$MPC"+String.valueOf(proc+1)+"B:");
                    fout3.newLine();
                  }
            
                  while ((str2=fin.readLine())!=null)
                  {
                    i1=str2.indexOf("change={");
                    i2=str2.indexOf("}", i1+1);
  
                    String change=str2.substring(i1+10,i2); 
                
                    if (!change.equals("0"))
                    {
                      i1=str2.indexOf("addr={");
                      i1=str2.indexOf("(", i1+1);
                      i2=str2.indexOf(")", i1+1);
    
                      String addr=str2.substring(i1+1,i2); 
  
                      i1=str2.indexOf("rw={");
                      i2=str2.indexOf("}", i1+1);
   
                      String rw=str2.substring(i1+4,i2); 
  
                      i1=str2.indexOf("value={");
                      i1=str2.indexOf("(", i1+1);
                      i2=str2.indexOf(")", i1+1);
   
                      String value=str2.substring(i1+1,i2); 

                      if (asm_id==1 && value.equals("0x0"))
                        value="0x00000000000000000000";

                      if (asm_id==1)
                        value="000000000000"+value.substring(2,value.length());
                      boolean found=false;

                      if (str_dep.equals("1"))
                      {
                        for (q=0; q<inst; q++)
                        {
                          if (Addr[proc][q].equals(addr))
                          {
                            found=true; 
                            break;
                          }
                        }
                      }
  
                      Addr[proc][inst]=addr;

                      if (found==false) Offset[proc][inst]=offset1;
                      else              Offset[proc][inst]=Offset[proc][q];

                      if (asm_id==0) offset1+=8;
                      else if (asm_id==1) offset1+=16;
  
                      if (rw.equals("1")) 
                      {
                        if (asm_id==0) value="0x0";
                        else if (asm_id==1) value="00000000000000000000000000000000";
                      }
  
                      if (asm_id==0)
                      {
                        fout2.write("        .quad "+value);
                        fout2.write(" #MPC inst="+
                                    String.valueOf(inst)+", rw="+rw+", offset="+
                                    Offset[proc][inst]);
                      }
                      else if (asm_id==1)
                      {
                        fout2.write("  DB ");
                        for (int z=0; z<16; z++)
                        {
                          if (z!=0) fout2.write(",");
                          fout2.write("0"+value.substring(30-2*z, 30-2*z+2)+"H");
                        }
                        fout2.write(" ;MPC inst="+
                                    String.valueOf(inst)+", rw="+rw+", offset="+
                                    Offset[proc][inst]);
                      }

                      if (found) fout2.write(", [reused]");

                      fout2.newLine();
       
                      if (asm_id==0)
                      {
                        fout3.write("        .quad 0x0");
                        fout3.newLine();
                      }
                      else if (asm_id==1)
                      {
                        fout3.write("  DB 000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H");
                        fout3.newLine();
                      }
                    }
                    else
                    {
                      Addr[proc][inst]="";
                      Offset[proc][inst]=offset1;

                      if (asm_id==0) offset1+=8;
                      else if (asm_id==1) offset1+=16;

                      if (asm_id==0)
                      {
                        fout2.write("        .quad 0x0");
                        fout2.write(" #MPC inst="+
                                    String.valueOf(inst)+", {NOT USED}, offset="+
                                    Offset[proc][inst]);
                        fout2.newLine();
       
                        fout3.write("        .quad 0x0");
                        fout3.newLine();
                      }
                      else if (asm_id==1)
                      {
                        fout2.write("  DB 000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H");
                        fout2.write(" ;MPC inst="+
                                    String.valueOf(inst)+", {NOT USED}, offset="+
                                    Offset[proc][inst]);
                        fout2.newLine();
       
                        fout3.write("  DB 000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H");
                        fout3.newLine();
                      }
                    }

                    inst++;
                  }

                  fout3.write("");
                  fout3.newLine();

                  fout3.write("$$MPC"+String.valueOf(proc+1)+"C:");
                  fout3.newLine();

                  if (asm_id==0)
                  {
                    int reg=0;
                    for (reg=0; reg<=27; reg++)
                    {
                      fout3.write("        .quad 0x0 "+"#MPC $"+String.valueOf(reg));
                      fout3.newLine();
                    }
  
                    for (reg=0; reg<=30; reg++)
                    {
                      fout3.write("        .quad 0x0 "+"#MPC $f"+String.valueOf(reg));
                      fout3.newLine();
                    }
                  }
                  else if (asm_id==1)
                  {
                    fout3.write("  DD 1024 DUP (0)");
                    fout3.newLine();
                  }

                  fout3.write("");
                  fout3.newLine();

                  if (asm_id==0)
                  {
                    fout4.write("        .globl  "+Ind[i][j]+"_restore_");
                    fout4.newLine();
                    fout4.write("        .ent    "+Ind[i][j]+"_restore_");
                    fout4.newLine();
                    fout4.write(Ind[i][j]+"_restore_:");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        lda $sp, -256($sp)");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        .frame  $sp, 256, $26");
                    fout4.newLine();
                    fout4.write("        .prologue 1");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        stq $28, 56($sp)");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        ldq $28, $$MPC"+String.valueOf(proc+1)+"C($gp)!literal!"+String.valueOf(10000+proc*5+3));
                    fout4.newLine();
                    fout4.write("        lda $28, 0($28)!lituse_base!"+String.valueOf(10000+proc*5+3));
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        ldq_u $0, ($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $1, 8($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $2, 16($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $3, 24($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $4, 32($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $5, 40($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $6, 48($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $7, 56($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $8, 64($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $9, 72($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $10, 80($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $11, 88($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $12, 96($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $13, 104($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $14, 112($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $15, 120($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $16, 128($28)");
                    fout4.newLine();                       
                    fout4.write("        ldq_u $17, 136($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $18, 144($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $19, 152($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $20, 160($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $21, 168($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $22, 176($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $23, 184($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $24, 192($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $25, 200($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $26, 208($28)");
                    fout4.newLine();
                    fout4.write("        ldq_u $27, 216($28)");
                    fout4.newLine();
                    fout4.write("        lda $28, 224($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f0, ($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f1, 8($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f2, 16($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f3, 24($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f4, 32($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f5, 40($28)");
                    fout4.newLine();                    
                    fout4.write("        ldt $f6, 48($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f7, 56($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f8, 64($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f9, 72($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f10, 80($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f11, 88($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f12, 96($28)");
                    fout4.newLine();        
                    fout4.write("        ldt $f13, 104($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f14, 112($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f15, 120($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f16, 128($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f17, 136($28)");
                    fout4.newLine();            
                    fout4.write("        ldt $f18, 144($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f19, 152($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f20, 160($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f21, 168($28)");
                    fout4.newLine();        
                    fout4.write("        ldt $f22, 176($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f23, 184($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f24, 192($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f25, 200($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f26, 208($28)");
                    fout4.newLine();                      
                    fout4.write("        ldt $f27, 216($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f28, 224($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f29, 232($28)");
                    fout4.newLine();
                    fout4.write("        ldt $f30, 240($28)");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        ldq $28, 56($sp)");
                    fout4.newLine();            
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        lda $sp, 256($sp)");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        br "+Ind[i][j]+"_restore_ret_");
                    fout4.newLine();
                    fout4.write("        .end "+Ind[i][j]+"_restore_");
                    fout4.newLine();
 
                    fout4.write(" #-------------------------------------------");
                    fout4.newLine();
                    fout4.write("        .globl  "+Ind[i][j]+"_prep_");
                    fout4.newLine();
                    fout4.write("        .ent    "+Ind[i][j]+"_prep_");
                    fout4.newLine();
                    fout4.write(Ind[i][j]+"_prep_:");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        lda $sp, -256($sp)");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        .frame  $sp, 256, $26");
                    fout4.newLine();
                    fout4.write("        .prologue 1");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        stq $1, 8($sp)");
                    fout4.newLine();
                    fout4.write("        stq $2, 16($sp)");
                    fout4.newLine();
                    fout4.write("        stq $3, 24($sp)");
                    fout4.newLine();
                    fout4.write("        stq $4, 32($sp)");
                    fout4.newLine();
                    fout4.write("        stq $5, 40($sp)");
                    fout4.newLine();
                    fout4.write("        stq $6, 48($sp)");
                    fout4.newLine();
                    fout4.write("        stq $28, 56($sp)");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        clr $6");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write(Ind[i][j]+"_prep_2_:");
                    fout4.newLine();
                    fout4.write("        ldq $1, $$MPC"+String.valueOf(proc+1)+"A($gp)!literal!"+String.valueOf(10000+proc*5));
                    fout4.newLine();
                    fout4.write("        lda $1, 0($1)!lituse_base!"+String.valueOf(10000+proc*5));
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        ldq $2, $$MPC"+String.valueOf(proc+1)+"B($gp)!literal!"+String.valueOf(10000+proc*5+1));
                    fout4.newLine();
                    fout4.write("        lda $2, ($2)!lituse_base!"+String.valueOf(10000+proc*5+1));
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        clr $4");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write(Ind[i][j]+"_prep_1_:");
                    fout4.newLine();
                    fout4.write("        ldq_u $3, ($1)");
                    fout4.newLine();
                    fout4.write("        stq_u $3, ($2)");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        lda $1, 8($1)");
                    fout4.newLine();
                    fout4.write("        lda $2, 8($2)");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        addl $4, 1, $4");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        lda $5, -"+String.valueOf(inst)+"($4) #MPC array length");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        blt $5, "+Ind[i][j]+"_prep_1_");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        addl $6, 1, $6");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    if (str_lp.equals("")) str_lp="1";
                    fout4.write("        lda $5, -"+String.valueOf(str_lp)+"($6) #loop for cache");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        blt $5, "+Ind[i][j]+"_prep_2_");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        ldq $28, $$MPC"+String.valueOf(proc+1)+"C($gp)!literal!"+String.valueOf(10000+proc*5+2));
                    fout4.newLine();
                    fout4.write("        lda $28, 0($28)!lituse_base!"+String.valueOf(10000+proc*5+2));
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        ldq $6, 48($sp)");
                    fout4.newLine();
                    fout4.write("        ldq $5, 40($sp)");
                    fout4.newLine();
                    fout4.write("        ldq $4, 32($sp)");
                    fout4.newLine();
                    fout4.write("        ldq $3, 24($sp)");
                    fout4.newLine();
                    fout4.write("        ldq $2, 16($sp)");
                    fout4.newLine();
                    fout4.write("        ldq $1, 8($sp)");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        stq_u $0, ($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $1, 8($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $2, 16($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $3, 24($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $4, 32($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $5, 40($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $6, 48($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $7, 56($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $8, 64($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $9, 72($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $10, 80($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $11, 88($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $12, 96($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $13, 104($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $14, 112($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $15, 120($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $16, 128($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $17, 136($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $18, 144($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $19, 152($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $20, 160($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $21, 168($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $22, 176($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $23, 184($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $24, 192($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $25, 200($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $26, 208($28)");
                    fout4.newLine();
                    fout4.write("        stq_u $27, 216($28)");
                    fout4.newLine();
                    fout4.write("        lda $28, 224($28)");
                    fout4.newLine();
                    fout4.write("        stt $f0, ($28)");
                    fout4.newLine();
                    fout4.write("        stt $f1, 8($28)");
                    fout4.newLine();
                    fout4.write("        stt $f2, 16($28)");
                    fout4.newLine();
                    fout4.write("        stt $f3, 24($28)");
                    fout4.newLine();
                    fout4.write("        stt $f4, 32($28)");
                    fout4.newLine();
                    fout4.write("        stt $f5, 40($28)");
                    fout4.newLine();
                    fout4.write("        stt $f6, 48($28)");
                    fout4.newLine();
                    fout4.write("        stt $f7, 56($28)");
                    fout4.newLine();
                    fout4.write("        stt $f8, 64($28)");
                    fout4.newLine();
                    fout4.write("        stt $f9, 72($28)");
                    fout4.newLine();
                    fout4.write("        stt $f10, 80($28)");
                    fout4.newLine();
                    fout4.write("        stt $f11, 88($28)");
                    fout4.newLine();
                    fout4.write("        stt $f12, 96($28)");
                    fout4.newLine();        
                    fout4.write("        stt $f13, 104($28)");
                    fout4.newLine();
                    fout4.write("        stt $f14, 112($28)");
                    fout4.newLine();
                    fout4.write("        stt $f15, 120($28)");
                    fout4.newLine();
                    fout4.write("        stt $f16, 128($28)");
                    fout4.newLine();
                    fout4.write("        stt $f17, 136($28)");
                    fout4.newLine();            
                    fout4.write("        stt $f18, 144($28)");
                    fout4.newLine();
                    fout4.write("        stt $f19, 152($28)");
                    fout4.newLine();
                    fout4.write("        stt $f20, 160($28)");
                    fout4.newLine();
                    fout4.write("        stt $f21, 168($28)");
                    fout4.newLine();        
                    fout4.write("        stt $f22, 176($28)");
                    fout4.newLine();
                    fout4.write("        stt $f23, 184($28)");
                    fout4.newLine();
                    fout4.write("        stt $f24, 192($28)");
                    fout4.newLine();
                    fout4.write("        stt $f25, 200($28)");
                    fout4.newLine();
                    fout4.write("        stt $f26, 208($28)");
                    fout4.newLine();
                    fout4.write("        stt $f27, 216($28)");
                    fout4.newLine();
                    fout4.write("        stt $f28, 224($28)");
                    fout4.newLine();
                    fout4.write("        stt $f29, 232($28)");
                    fout4.newLine();
                    fout4.write("        stt $f30, 240($28)");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
//                    fout4.write("        ldq $28, 56($sp)");
//                    fout4.newLine();
                    fout4.write("        ldq $28, $$MPC"+String.valueOf(proc+1)+"B($gp)!literal!"+String.valueOf(10000+proc*5+4));
                    fout4.newLine();
                    fout4.write("        lda $28, 0($28)!lituse_base!"+String.valueOf(10000+proc*5+4));
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("        lda $sp, 256($sp)");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                  }
                  else if (asm_id==1)
                  {
                    fout4.write("_"+Ind[i][j]+"_restore PROC NEAR");
                    fout4.newLine();
                    fout4.write("  push eax");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C");
                    fout4.newLine();
                    fout4.write("  frstor [eax]");
                    fout4.newLine();
                    fout4.write("  pop eax");
                    fout4.newLine();
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C+516");
                    fout4.newLine();
                    fout4.write("  mov ebx, [eax]");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C+520");
                    fout4.newLine();
                    fout4.write("  mov ecx, [eax]");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C+524");
                    fout4.newLine();
                    fout4.write("  mov edx, [eax]");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C+528");
                    fout4.newLine();
                    fout4.write("  mov esi, [eax]");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C+532");
                    fout4.newLine();
                    fout4.write("  mov edi, [eax]");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C+536");
                    fout4.newLine();
                    fout4.write("  mov ebp, [eax]");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C+512");
                    fout4.newLine();
                    fout4.write("  mov eax, [eax]");
                    fout4.newLine();
                    fout4.newLine();
                    fout4.write("  ret");
                    fout4.newLine();
                    fout4.write("_"+Ind[i][j]+"_restore ENDP");
                    fout4.newLine();
 
                    fout4.write(";-------------------------------------------");
                    fout4.newLine();
                    fout4.write("_"+Ind[i][j]+"_prep PROC NEAR");
                    fout4.newLine();
                    fout4.write(" pusha");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("  push eax");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C");
                    fout4.newLine();
                    fout4.write("  fsave [eax]");
                    fout4.newLine();
                    fout4.write("  pop eax");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("  push eax");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C+516");
                    fout4.newLine();
                    fout4.write("  mov [eax], ebx");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C+520");
                    fout4.newLine();
                    fout4.write("  mov [eax], ecx");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C+524");
                    fout4.newLine();
                    fout4.write("  mov [eax], edx");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C+528");
                    fout4.newLine();
                    fout4.write("  mov [eax], esi");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C+532");
                    fout4.newLine();
                    fout4.write("  mov [eax], edi");
                    fout4.newLine();
                    fout4.write("  lea eax, $$MPC"+String.valueOf(proc+1)+"C+536");
                    fout4.newLine();
                    fout4.write("  mov [eax], ebp");
                    fout4.newLine();
                    fout4.write("  pop eax");
                    fout4.newLine();
                    fout4.write("  push ebx");
                    fout4.newLine();
                    fout4.write("  lea ebx, $$MPC"+String.valueOf(proc+1)+"C+512");
                    fout4.newLine();
                    fout4.write("  mov [ebx], eax");
                    fout4.newLine();
                    fout4.write("  pop ebx");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    if (str_lp.equals("")) str_lp="1";
                    fout4.write(" mov edx, "+String.valueOf(str_lp));
                    fout4.newLine();
                    fout4.write("_"+Ind[i][j]+"_prep_1:");
                    fout4.newLine();
                    fout4.write(" push edx");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write(" mov edx, "+String.valueOf(inst));
                    fout4.newLine();
                    fout4.write(" lea ebx, $$MPC"+String.valueOf(proc+1)+"A");
                    fout4.newLine();
                    fout4.write(" lea ecx, $$MPC"+String.valueOf(proc+1)+"B");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write("_"+Ind[i][j]+"_prep_2:");
                    fout4.newLine();
                    fout4.write(" mov eax, [ebx]");
                    fout4.newLine();
                    fout4.write(" mov [ecx], eax");
                    fout4.newLine();
                    fout4.write(" mov eax, [ebx+4]");
                    fout4.newLine();
                    fout4.write(" mov [ecx+4], eax");
                    fout4.newLine();
                    fout4.write(" mov eax, [ebx+8]");
                    fout4.newLine();
                    fout4.write(" mov [ecx+8], eax");
                    fout4.newLine();
                    fout4.write(" mov eax, [ebx+12]");
                    fout4.newLine();
                    fout4.write(" mov [ecx+12], eax");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write(" add ebx, 16");
                    fout4.newLine();
                    fout4.write(" add ecx, 16");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write(" dec edx");
                    fout4.newLine();
                    fout4.write(" jnz _"+Ind[i][j]+"_prep_2");
                    fout4.newLine();
                    fout4.write("");
                    fout4.newLine();
                    fout4.write(" pop edx");
                    fout4.newLine();
                    fout4.write(" dec edx");
                    fout4.newLine();
                    fout4.write(" jnz _"+Ind[i][j]+"_prep_1");
                    fout4.newLine();
                    fout4.write(" popa");
                    fout4.newLine();
                  }
                }
                catch (Exception e1)
                {
                  try
                  {
                    fin.close();
                    fout2.close();
                    fout3.close();
                    fout4.close();
                  }
                  catch (Exception e2)
                  {}
                  fout.write("Error reading atom file");
                  fout.newLine();
                  fout.flush();
                  return;
                }
    
                try
                {
                  fin.close();
                  fout2.close();
                  fout3.close();
                  fout4.close();
                }
                catch (Exception e3)
                {}
  
                if (inst>inst_max) inst_max=inst;
  
                proc++;
              }
            }
  
            fout.write("Ok");
            fout.newLine();
  
            fout.write("Start preparing asm file        : ");
            fout.flush();
 
            proc=0;
  
            for (i=0; i<nf; i++)
            {
              //work with asm file
              String f2a=Fil[i];
              String f2b="";
    
              i1=Fil[i].lastIndexOf(".");
              if (i1!=-1)
              {
                f2a=Fil[i].substring(0, i1);
                f2b=Fil[i].substring(i1, Fil[i].length());
              }

              //preparing asm files
              str1=str_dir+f2a+f_mpc5b+g.cfg.GetParamStr(g.cfgAsmExt);
              str2=str_dir+f2a+f_mpc5c+g.cfg.GetParamStr(g.cfgAsmExt);
  
              try
              {
                 fin=new BufferedReader(new FileReader(str1));
                fout2=new BufferedWriter(new FileWriter(str2));
              }
              catch (IOException e2)
              {
                fout.write("Error opening original asm file!");
                fout.newLine();
                fout.flush();
                return;
              }

              boolean found1=false;
              boolean found2=false;
       
              String sb="";
              String sb1="";
  
              fout3=null;
              int gpd9=0;

              try
              {
                while ((str2=fin.readLine())!=null)
                {
                  if (asm_id==1)
                  {
                    if (str2.equals("END")) str2="; "+str2;

                    if (found2)
                    {
                      String str3=str2;
              
                      i1=str3.indexOf(" jmp ");
                      if (i1!=-1)
                      {
                        if (str3.substring(i1+5, str3.length()).equals(sb1+"_ret"))
                        {
                          fout3.write("  jmp "+sb1+"_tr_ret");
                          fout3.newLine();
                        }
                        else
                        {
                          fout2.write(str2);
                          fout2.newLine();
    
                          fout3.write(str3);
                          fout3.newLine();
                        }
                      }
                      else if ((i1=str2.indexOf(" ENDP"))!=-1)
                      {
                        sb=str2.substring(0, i1);
          
                        if (sb.equals(sb1))
                        {
                          fout3.write(sb1+"_tr ENDP");
                          fout3.newLine();
                      
                          try
                          {
                            fout3.close();
                          }
                          catch (Exception e3)
                          {}
        
                          found2=false;
                        }
                      }
                      else
                      {
                        fout3.write(str3);
                        fout3.newLine();
                      }
                    }
                    else if ((!found2) && (str2.indexOf(" PROC ")!=-1))
                    {
                      i1=str2.indexOf(" PROC ");
                    
                      sb=str2.substring(0, i1);
              
                      for (k=0; k<proc_max; k++)
                      {
                        if (sb.equals(Ind1[k]))
                        {
                          sb1=sb;
  
                          String Fil1=str_dir+f2a+f_mpc5c+"."+String.valueOf(k);

                          try
                          {
                            fout3=new BufferedWriter(new FileWriter(Fil1));
                          }
                          catch (IOException e2)
                          {
                            fout.write("Error opening asm subroutine file!");
                            fout.newLine();
                            fout.flush();
                            return;
                          }
            
                          fout3.write(sb1+"_tr PROC NEAR");
                          fout3.newLine();
      
                          fout2.write(str2);
                          fout2.newLine();
          
                          str2=fin.readLine();
                          fout2.write(str2);
                          fout2.newLine();
  
                          str2=fin.readLine();
            
                          found2=true;
                          gpd9=0;
        
                          break;
                        }
                      }
                    }

                    if (str2.indexOf(" jmp ")!=-1)
                    {
                      i1=str2.indexOf(" jmp ");
                      i2=str2.indexOf(" ", i1+5);
 
                      if (i2!=-1)
                      {
                        sb=str2.substring(i1+5, i2);

                        for (k=0; k<LM[i]; k++)
                        {
                          if (sb.equals("_"+Ind[i][k]))
                          {
                            fout2.write("  call _"+Ind[i][k]+"_prep");
                            fout2.newLine();
 
                            fout2.write("  jmp _"+Ind[i][k]+"_tr");
                            fout2.newLine();
 
                            fout2.write("_"+Ind[i][k]+"_tr_ret LABEL NEAR");
                            fout2.newLine();

                            fout2.write("  call _"+Ind[i][k]+"_restore");
                            fout2.newLine();

                            break;
                          }
                        }
                      }

                      fout2.write(str2);
                      fout2.newLine();
                    }
                    else
                    {
                      fout2.write(str2);
                      fout2.newLine();
                    }
                  }
                  else if (asm_id==0)
                  {
                    if (found2)
                    {
                      String str3=str2;
              
                      if (str2.indexOf("mpcdummy_")!=-1)
                      {
                        str2=" #MPC_CALL "+str2;
                      }
                      else if ((str2.indexOf("!gpdisp!")!=-1) && (gpd9<2))
                      {
                        str2=" #MPC_CALL "+str2;
                        gpd9++;
                      }

                      i1=str3.indexOf("!gpdisp!");
                      if (i1!=-1)
                      {
                        i2=str3.indexOf("\t", i1+1);
                        if (i2==-1) i2=str3.length();
                  
                        String gpd=str3.substring(i1+8,i2);

                        int gpd1=0;
                        try
                        {
                          gpd1=Integer.valueOf(gpd).intValue();
                        }
                        catch (NumberFormatException nfe1)
                        {}

                        str3=str3.substring(0, i1+8)+String.valueOf(gpd1+20000);
                      }

                      i1=str3.indexOf("!literal!");
                      if (i1!=-1)
                      {
                        i2=str3.indexOf("\t", i1+1);
                        if (i2==-1) i2=str3.length();

                        String gpd=str3.substring(i1+9,i2);
              
                        int gpd1=0;
                        try
                        {
                          gpd1=Integer.valueOf(gpd).intValue();
                        }
                        catch (NumberFormatException nfe1)
                        {}

                        str3=str3.substring(0, i1+9)+String.valueOf(gpd1+20000);
                      }

                      i1=str3.indexOf("!lituse_base!");
                      if (i1!=-1)
                      {
                        i2=str3.indexOf("\t", i1+1);
                        if (i2==-1) i2=str3.length();
            
                        String gpd=str3.substring(i1+13,i2);

                        int gpd1=0;
                        try
                        {
                          gpd1=Integer.valueOf(gpd).intValue();
                        }
                        catch (NumberFormatException nfe1)
                        {}

                        str3=str3.substring(0, i1+13)+String.valueOf(gpd1+20000);
                      }

                      if ((i1=str3.indexOf("L$"))!=-1)
                      {
                        String a1="";
                        String a2="";
  
                        if (i1>0) a1=str3.substring(0, i1);
                        a2=str3.substring(i1+2, str3.length());
                  
                        str3=a1+"MPCL$"+a2;
                      }
                      else if ((i1=str3.indexOf("lab$"))!=-1)
                      {
                        String a1="";
                        String a2="";
  
                        if (i1>0) a1=str3.substring(0, i1);
                        a2=str3.substring(i1+4, str3.length());
                  
                        str3=a1+"MPClab$"+a2;
                      }
                      else if ((i1=str3.indexOf("$$L"))!=-1)
                      {
                        String a1="";
                        String a2="";
  
                        if (i1>0) a1=str3.substring(0, i1);
                        a2=str3.substring(i1+2, str3.length());
                  
                        str3=a1+"MPC"+a2;
                      }
            
                      i1=str3.indexOf("br ");
                      if (i1!=-1)
                      {
                        if (str3.substring(i1+3, str3.length()).equals(sb1+"ret_"))
                        {
                          fout3.write("        br "+sb1+"tr_ret_");
                          fout3.newLine();
                        }
                        else
                        {
                          fout2.write(str2);
                          fout2.newLine();
    
                          fout3.write(str3);
                          fout3.newLine();
                        }
                      }
                      else if ((i1=str2.indexOf(".end "))!=-1)
                      {
                        sb=str2.substring(i1+5, str2.length());
          
                        if (sb.equals(sb1))
                        {
                          fout3.write("        .end "+sb1+"tr_");
                          fout3.newLine();
                      
                          try
                          {
                            fout3.close();
                          }
                          catch (Exception e3)
                          {}
        
                          found2=false;
                        }
                      }
                      else
                      {
                        fout3.write(str3);
                        fout3.newLine();
                      }
                    }
                    else if ((!found2) && (str2.indexOf(".globl ")!=-1))
                    {
                      i1=str2.indexOf(".globl ");
                    
                      sb=str2.substring(i1+7, str2.length());
              
                      for (k=0; k<proc_max; k++)
                      {
                        if (sb.equals(Ind1[k]))
                        {
                          sb1=sb;
  
                          String Fil1=str_dir+f2a+f_mpc5c+"."+String.valueOf(k);

                          try
                          {
                            fout3=new BufferedWriter(new FileWriter(Fil1));
                          }
                          catch (IOException e2)
                          {
                            fout.write("Error opening asm subroutine file!");
                            fout.newLine();
                            fout.flush();
                            return;
                          }
            
                          fout3.write("        .globl "+sb1+"tr_");
                          fout3.newLine();
      
                          fout3.write("        .ent   "+sb1+"tr_");
                          fout3.newLine();
                      
                          fout3.write(sb1+"tr_:");
                          fout3.newLine();
                
                          fout2.write(str2);
                          fout2.newLine();
          
                          str2=fin.readLine();
                          fout2.write(str2);
                          fout2.newLine();
  
                          str2=fin.readLine();
            
                          found2=true;
                          gpd9=0;
        
                          break;
                        }
                      }
                    }
              
                    if ((!found1) && ((str2.indexOf(".data")!=-1) || (str2.indexOf(".rdata")!=-1)))
                    {
                      //Adding data files
                      fout2.write("        .data");
                      fout2.newLine();
  
                      int proc1=proc;
              
                      String Fil1="";
                      BufferedReader fin2=null;
        
                      for (j=0; j<LM[i]; j++)
                      {
                        Fil1=str_dir+f_tmp_atom+"5."+String.valueOf(proc1)+"a";

                        try
                        {
                          fin2=new BufferedReader(new FileReader(Fil1));
                        }
                        catch (IOException e2)
                        {
                          fout.write("Error opening ATOM file!");
                          fout.newLine();
                          fout.flush();
                          return;
                        }
                  
                        try
                        {
                          while ((str2=fin2.readLine())!=null)
                          {
                            fout2.write(str2);
                            fout2.newLine();
                          }
                        }
                        catch (Exception e1)
                        {
                          try
                          {
                            fin2.close();
                          }
                          catch (Exception e2)
                          {}
                          fout.write("Error reading original asm file!");
                          fout.newLine();
                          fout.flush();
                          return;
                        }
  
                        try
                        {
                          fin2.close();
                        }
                        catch (Exception e3)
                        {}
             
                        Fil1=str_dir+f_tmp_atom+"5."+String.valueOf(proc1)+"b";
  
                        try
                        {
                          fin2=new BufferedReader(new FileReader(Fil1));
                        }
                        catch (IOException e2)
                        {
                          fout.write("Error opening ATOM file!");
                          fout.newLine();
                          fout.flush();
                          return;
                        }

                        try
                        {
                          while ((str2=fin2.readLine())!=null)
                          {
                            fout2.write(str2);
                            fout2.newLine();
                          }
                        }
                        catch (Exception e1)
                        {
                          try
                          {
                            fin2.close();
                          }
                          catch (Exception e2)
                          {}
                          fout.write("Error reading original asm file!");
                          fout.newLine();
                          fout.flush();
                          return;
                        }

                        try
                        {
                          fin2.close();
                        }
                        catch (Exception e3)
                        {}

                        proc1++;
                      }

                      fout2.write("        .data");
                      fout2.newLine();

                      found1=true;
                    }
                    else if (str2.indexOf("br ")!=-1)
                    {
                      i1=str2.indexOf("br ");
                      i2=str2.indexOf(" ", i1+3);
 
                      if (i2!=-1)
                      {
                        sb=str2.substring(i1+3, i2);

                        for (k=0; k<LM[i]; k++)
                        {
                          if (sb.equals(Ind[i][k]+"_"))
                          {
                            fout2.write("        br "+Ind[i][k]+"_prep_    #MPC");
                            fout2.newLine();
 
                            fout2.write(Ind[i][k]+"_prep_ret_:");
                            fout2.newLine();

                            fout2.write("        br "+Ind[i][k]+"_tr_    #MPC");
                            fout2.newLine();
 
                            fout2.write(Ind[i][k]+"_tr_ret_:");
                            fout2.newLine();

                            fout2.write("        br "+Ind[i][k]+"_restore_    #MPC");
                            fout2.newLine();
 
                            fout2.write(Ind[i][k]+"_restore_ret_:");
                            fout2.newLine();
  
                            break;
                          }
                        }
                      }

                      fout2.write(str2);
                      fout2.newLine();
                    }
                    else
                    {
                      fout2.write(str2);
                      fout2.newLine();
                    }
                  }
                }
              }
              catch (Exception e1)
              {
                try
                {
                  fin.close();
                  fout2.close();
                }
                catch (Exception e2)
                {}
                fout.write("Error reading original asm file!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                fin.close();
                fout2.close();
              }
              catch (Exception e3)
              {}
 
              proc+=LM[i];
            }

            fout.write("Ok");
            fout.newLine();

            //****** Finishing ******
            fout.write("Transforming subroutines        : ");
            fout.flush();

            proc=0;

            for (i=0; i<nf; i++)
            {
              String f2a=Fil[i];
              String f2b="";
  
              i1=Fil[i].lastIndexOf(".");
              if (i1!=-1)
              {
                f2a=Fil[i].substring(0, i1);
                f2b=Fil[i].substring(i1, Fil[i].length());
              }

              for (j=0; j<LM[i]; j++)
              {
                String[] ProcTmp=new String[32000];

                //Transformations
                str1=str_dir+f2a+f_mpc5c+"."+String.valueOf(proc);

                try
                {
                  fin=new BufferedReader(new FileReader(str1));
                }
                catch (IOException e2)
                {
                  fout.write("Error opening asm file with procedure!");
                  fout.newLine();
                  fout.flush();
                  return;
                }

                int z=0;

                try
                {
                  while ((str2=fin.readLine())!=null)
                  {
                    ProcTmp[z]=str2;
                    z++;
                  }
                }
                catch (Exception e1)
                {
                  try
                  {
                    fin.close();
                  }
                  catch (Exception e2)
                  {}
                  fout.write("Error reading asm file with procedure!");
                  fout.newLine();
                  fout.flush();
                  return;
                }

                try
                {
                  fin.close();
                }
                catch (Exception e3)
                {}

                if (asm_id==0)
                {
                  str1=str_dir+f_tmp_atom+"4."+String.valueOf(proc);
                  try
                  {
                    fin=new BufferedReader(new FileReader(str1));
                  }
                  catch (IOException e2)
                  {
                    fout.write("Error opening ATOM file4!");
                    fout.newLine();
                    fout.flush();
                    return;
                  }

                  int z1=0;
                  int z2=0;
                  int z3=0;

                  inst=-1;
                  try
                  {
                    while ((str2=fin.readLine())!=null)
                    {
                      inst++;
  
                      i1=str2.indexOf("type={");
                      i2=str2.indexOf("}", i1+1);
  
                      String type=str2.substring(i1+6,i2).trim(); 

                      i1=str2.indexOf("reg={");
                      i2=str2.indexOf("}", i1+1);

                      String reg=str2.substring(i1+5,i2).trim(); 
                      if (reg.equals("29")) reg="gp";

                      i1=str2.indexOf("offset={");
                      i2=str2.indexOf("}", i1+1);
      
                      String offset=str2.substring(i1+8,i2).trim(); 

                      //Searching instruction
                      boolean found=false;
                      boolean found28=false;

                      while (z2<z)
                      {
                        str2=ProcTmp[z2];
       
                        found28=false;

//                        if ((str2.indexOf("$28")!=-1) && (str2.indexOf("($28)")==-1) &&
//                            (str2.indexOf("L$28")==-1) && (str2.indexOf("($$28)")==-1) && (str2.indexOf("(lab$28)")==-1) &&
//                             (reg.equals("28")))
                        if ((str2.indexOf("$28")!=-1) && (str2.indexOf("($28)")==-1) &&
                            (str2.indexOf("L$28")==-1) && (str2.indexOf("($$28)")==-1) && (str2.indexOf("(lab$28)")==-1))
                        {
                          found28=true;
                        }

                        i1=str2.indexOf(type+"\t");
                        if (i1!=-1)
                        {
                          i1=str2.indexOf("($"+reg+")");
                          if (i1!=-1)
                          {
                            i1=str2.indexOf("!");
                            if (i1!=-1) 
                            {
                              found=true; //if !literal, ... don't check offset
                              break;
                            }
                            else
                            {
                              if (offset.equals("0")) offset="(";
                              i1=str2.indexOf(" "+offset);
                              if (i1!=-1) 
                              {
                                found=true;
                                break;
                              }
                            }
                          }
                        }

                        if (found28) ProcTmp[z2]=" #MPC_28 "+ProcTmp[z2];

                        z2++;
                      }

                      if (!found)
                      {
                        fout.write("Failed matching instructions in asm and ATOM files!");
                        fout.newLine();
                        fout.flush();
                        return;
                      }

                      if ((!str_sr.equals("")) && 
                          (str2.indexOf(str_sr)!=-1) || 
                          (str2.indexOf("("+str_sr+")")!=-1) &&
                          (str2.indexOf("L"+str_sr)==-1) && 
                          (str2.indexOf("($"+str_sr+")")==-1) && 
                          (str2.indexOf("(lab"+str_sr+")")==-1))
                      {
                      }
                      else
                      {
                        for (z3=z; z3>z2; z3--) {ProcTmp[z3+1]=ProcTmp[z3];}z++;
                        i1=ProcTmp[z2].indexOf(",");
                        ProcTmp[z2+1]=ProcTmp[z2].substring(0, i1+1)+
                                      " "+String.valueOf(Offset[proc][inst])+
                                      "($28)     #MPC inst="+String.valueOf(inst);
                        ProcTmp[z2]=" #MPC "+ProcTmp[z2];
  
                        if (found28) ProcTmp[z2+1]=" #MPC_28 "+ProcTmp[z2+1];
  
                        z2++;
                      }

                      z1++;
                    }
                  }
                  catch (Exception e1)
                  {
                    try
                    {
                      fin.close();
                    }
                    catch (Exception e2)
                    {}
                    fout.write("Error reading ATOM file4!");
                    fout.newLine();
                    fout.flush();
                    return;
                  }

                  try
                  {
                    fin.close();
                  }
                  catch (Exception e3)
                  {}
                }
                else if (asm_id==1)
                {
                  int inst_max9=0;

                  for (int zq=0; zq<z; zq++)
                  {
                    str2=ProcTmp[zq];

                    int found1=0;
                    int rw=0;

                    if (GetInstClassIntelWin(str2)==0)
                    {
                      found1=1;
                      rw=0;
                    }
                    else if (GetInstClassIntelWin(str2)==1)
                    {
                      found1=1;
                      rw=0;
                    }
                    else if (GetInstClassIntelWin(str2)==2)
                    {
                      found1=1;
                      rw=1;
                    }
                    else if (GetInstClassIntelWin(str2)==3)
                    {
                      found1=1;
                      rw=1;
                    }

//                    if (str2.indexOf("esp")!=-1 || str2.indexOf("ebp")!=-1)
                    if (str2.indexOf("esp")!=-1)
                      found1=0;

                    if (found1!=0)
                    {
                      i1=str2.indexOf(" PTR ");
                      i2=i1+5;

                      int i3=str2.indexOf("]", i2);
                      if (i3==-1)
                      {
                        i3=str2.indexOf(",",i2);
                        if (i3==-1)
                        {
                          i3=str2.indexOf(" ",i2);
                          if (i3==-1)
                          {
                            i3=str2.indexOf(";",i2);
                          }
                        }
                      }
                      else i3++;

                      ProcTmp[zq]=";MPC "+ProcTmp[zq]+"\r\n"+
                                  str2.substring(0, i2)+"$$MPC"+
                                  String.valueOf(proc+1)+"B+"+
                                  String.valueOf(Offset[proc][inst_max9])+
                                  str2.substring(i3, str2.length())+
                                  " ;MPC inst="+String.valueOf(inst_max9);

                      inst_max9++;
                    }
                  }
                }

                str1=str_dir+f2a+f_mpc5d+"."+String.valueOf(proc);
                try
                {
                  fout2=new BufferedWriter(new FileWriter(str1));
                }
                catch (IOException e2)
                {
                  fout.write("Error opening asm file for transformed subroutine!");
                  fout.newLine();
                  fout.flush();
                  return;
                }

                try
                {
                  for (int z9=0; z9<z; z9++)
                  {
                    fout2.write(ProcTmp[z9]);
                    fout2.newLine();
                  }
                }
                catch (Exception e1)
                {
                  try
                  {
                    fout2.close();
                  }
                  catch (Exception e2)
                  {}
                  fout.write("Error writing asm file for transformed subroutine!");
                  fout.newLine();
                  fout.flush();
                  return;
                }

                try
                {
                  fout2.close();
                }
                catch (Exception e3)
                {}

                //Finishing preparation files
                String Fil4=str_dir+f_tmp_atom+"5."+String.valueOf(proc)+"c";
  
                try
                {
                  fout4=new BufferedWriter(new FileWriter(Fil4, true));
                }
                catch (IOException e2)
                {
                  fout.write("Error appeding prep subroutine");
                  fout.newLine();
                  fout.flush();
                  return;
                }
  
                try
                {
                  if (asm_id==0)
                  {
                    fout4.write("        br "+Ind[i][j]+"_prep_ret_");
                    fout4.newLine();
                    fout4.write("        .end "+Ind[i][j]+"_prep_");
                    fout4.newLine();
                  }
                  else if (asm_id==1)
                  {
                    fout4.write("  ret");
                    fout4.newLine();
                    fout4.write("_"+Ind[i][j]+"_prep ENDP");
                    fout4.newLine();
                  }

                }
                catch (Exception e1)
                {
                  try
                  {
                    fout4.close();
                  }
                  catch (Exception e2)
                  {}
                  fout.write("Error appending prep subroutine");
                  fout.newLine();
                  fout.flush();
                  return;
                }
    
                try
                {
                  fout4.close();
                }
                catch (Exception e3)
                {}

                //Add to final asm file
                String Fil1=str_dir+f2a+f_mpc5d+"."+String.valueOf(proc);
                String Fil2=str_dir+f_tmp_atom+"5."+String.valueOf(proc)+"c";
                str2=str_dir+f2a+f_mpc5c+g.cfg.GetParamStr(g.cfgAsmExt);
  
                try
                {
                    fin=new BufferedReader(new FileReader(Fil1));
                   fin1=new BufferedReader(new FileReader(Fil2));
                  fout2=new BufferedWriter(new FileWriter(str2, true));
                }
                catch (IOException e2)
                {
                  fout.write("Error opening asm files to finalize!");
                  fout.newLine();
                  fout.flush();
                  return;
                }

                try
                {
  
                  if (asm_id==1)
                  {
                    fout2.write("_TEXT  SEGMENT DWORD PUBLIC USE32 'CODE'");
                    fout2.newLine();
                  }

                  while ((str2=fin.readLine())!=null)
                  {
                    fout2.write(str2);
                    fout2.newLine();
                  }

                  if (asm_id==0)
                  {
                    fout2.write(" #*******************************************************************");
                    fout2.newLine();
                  }
                  else if (asm_id==1)
                  {
                    fout2.write(";*******************************************************************");
                    fout2.newLine();
                  }

                  while ((str2=fin1.readLine())!=null)
                  {
                    fout2.write(str2);
                    fout2.newLine();
                  }
  
                  if (asm_id==0)
                  {
                    fout2.write(" #*******************************************************************");
                    fout2.newLine();
                  }
                  else if (asm_id==1)
                  {
                    fout2.write(";*******************************************************************");
                    fout2.newLine();
                  }

                  if (asm_id==1)
                  {
                    fout2.write("_TEXT  ENDS");
                    fout2.newLine();
                  }
                }
                catch (Exception e1)
                {
                  try
                  {
                    fin.close();
                    fin1.close();
                    fout2.close();
                  }
                  catch (Exception e2)
                  {}
                  fout.write("Error opening asm files to finalize!");
                  fout.newLine();
                  fout.flush();
                  return;
                }

                try
                {
                    fin.close();
                    fin1.close();
                    fout2.close();
                }
                catch (Exception e3)
                {}

                //Add data files to final asm file is IntelWin
                if (asm_id==1)
                {
                  Fil1=str_dir+f_tmp_atom+"5."+String.valueOf(proc)+"a";
                  Fil2=str_dir+f_tmp_atom+"5."+String.valueOf(proc)+"b";
                  str2=str_dir+f2a+f_mpc5c+g.cfg.GetParamStr(g.cfgAsmExt);
  
                  try
                  {
                      fin=new BufferedReader(new FileReader(Fil1));
                     fin1=new BufferedReader(new FileReader(Fil2));
                    fout2=new BufferedWriter(new FileWriter(str2, true));
                  }
                  catch (IOException e2)
                  {
                    fout.write("Error opening asm files to finalize!");
                    fout.newLine();
                    fout.flush();
                    return;
                  }

                  try
                  {
                    fout2.write("_DATA	SEGMENT DWORD PUBLIC USE32 'DATA'");
                    fout2.newLine();

                    while ((str2=fin.readLine())!=null)
                    {
                      fout2.write(str2);
                      fout2.newLine();
                    }

                    fout2.write(";*******************************************************************");
                    fout2.newLine();

                    while ((str2=fin1.readLine())!=null)
                    {
                      fout2.write(str2);
                      fout2.newLine();
                    }
  
                    fout2.write(";*******************************************************************");
                    fout2.newLine();

                    fout2.write("_DATA  ENDS");
                    fout2.newLine();
                  }
                  catch (Exception e1)
                  {
                    try
                    {
                      fin.close();
                      fin1.close();
                      fout2.close();
                    }
                    catch (Exception e2)
                    {}
                    fout.write("Error opening asm files to finalize!");
                    fout.newLine();
                    fout.flush();
                    return;
                  }

                  try
                  {
                      fin.close();
                      fin1.close();
                      fout2.close();
                  }
                  catch (Exception e3)
                  {}
                } 

                proc++;
              }

              if (asm_id==1)
              {
                str2=str_dir+f2a+f_mpc5c+g.cfg.GetParamStr(g.cfgAsmExt);
  
                try
                {
                  fout2=new BufferedWriter(new FileWriter(str2, true));
                }
                catch (IOException e2)
                {
                  fout.write("Error opening asm files to finalize!");
                  fout.newLine();
                  fout.flush();
                  return;
                }

                try
                {
                  fout2.write("END");
                  fout2.newLine();
                }
                catch (Exception e1)
                {
                  try
                  {
                    fout2.close();
                  }
                  catch (Exception e2)
                  {}
                  fout.write("Error opening asm files to finalize!");
                  fout.newLine();
                  fout.flush();
                  return;
                }

                try
                {
                    fout2.close();
                }
                catch (Exception e3)
                {}
              }
            }

            fout.write("Ok");
            fout.newLine();
          }
          else 
          {
            if (mode==11)
            {
              //Compiling transformed files
              fout.write("Compiling transformed files2    : ");
              fout.flush();
  
              String fprj1=str_dir+f_tmp_prj;
            
              CProject cp_new = new CProject();
              cp_new.Init(cp, fprj1);
   
              for (i=1; i<cp.GetNumFiles(); i++)
              {
                boolean found=false;
      
                j=0;
       
                for (j=0; j<nf; j++)
                {
                  if (Fil[j].equals(cp.GetFileName(i)+cp.GetFileExt(i)))
                  {
                    found=true;
                    break;
                  }
                }

                if (!found)
                {
                  if (cp_new.AddLine1(cp.GetLngNameOrig(i), cp.GetFileName(i), cp.GetFileExt(i), cp.GetExtraOpt(i))<0)
                  {
                    out.println("Error: Too many files in new project file!");
                    out.flush();
                    return;
                  }
                }
                else
                {
                  String f2a=Fil[j];
                  String f2b="";
  
                  i1=Fil[j].lastIndexOf(".");
                  if (i1!=-1)
                  {
                    f2a=Fil[j].substring(0, i1);
                    f2b=Fil[j].substring(i1, Fil[j].length());
                  }

                  cp_new.AddLine1(g.cfg.GetParamStr(g.cfgAsmPrjN), 
                                  f2a+f_mpc5c, 
                                  g.cfg.GetParamStr(g.cfgAsmExt),
                                  "");
                }
              }

              if (asm_id==0)
              {
                cp_new.AddLine1(g.cfg.GetParamStr(g.cfgAsmPrjN),
                                f_mpc_d, 
                                g.cfg.GetParamStr(g.cfgAsmExt),
                                "");
              }

              if (cp_new.Save()<0)
              {
                fout.write("Error: Can't save new project!");
                fout.newLine();
                fout.flush();
                return;
              }
  
              cl.Add("initiate compilation",    true, true);
              cl.Add("directory="+str_dir,      true, true);
              cl.Add("file_project="+f_tmp_prj, true, true);
              cl.Add("file_diag_out="+f_tmp,    true, true);
              cl.Add("compilation_mode=0",      true, true);
              cl.Add("no_checking=0",           true, true);
              cl.Add("profile=0",               true, true);
              cl.Add("optimizations="+str_opt,  true, true);
              cl.Add("compile",                 true, true);
              cl.Add("terminate",              false, true);

              ret=cl.Send();
              if (ret==-1)
              {
                out.println("Error: Can't find host with Compilation Server!");
                out.flush();
                return;
              }
              else if (ret==-2)
              {
                out.println("Error: Can't connect to Compilation Server!");
                out.flush();
                return;
              }  
              else if (ret==-4)
              {
                out.println("Error: Can't close connection with Compilation Server!");
                out.flush();
                return;
              }
              else if (ret<0)
              {
                out.println(cl.GetLastError());
                out.flush();
                return;
              }
    
              fout.write("Ok");
              fout.newLine();
            }

            fout.write("Running (cycle profiling) 4a    : ");
            fout.flush();

            if (asm_id==0)
            {
              cl.Add("initiate compilation",          true, true);
              cl.Add("directory="+str_dir,            true, true);
              cl.Add("file_project="+f_tmp_prj,       true, true);
              cl.Add("file_out_time="+f_tmp1,         true, true);
              cl.Add("file_out_comp="+f_tmp2,         true, true);
              cl.Add("file_out_prof="+str_fpf+"4a",   true, true);
              cl.Add("prof_type=1",                   true, true);
              cl.Add("runmode=1",                     true, true);
              cl.Add("run",                           true, true);
              cl.Add("terminate",                    false, true);
            }
            else if (asm_id==1)
            {
              cl.Add("initiate compilation",          true, true);
              cl.Add("directory="+str_dir,            true, true);
              cl.Add("file_project="+f_tmp_prj,       true, true);
              cl.Add("file_out_time="+f_tmp1,         true, true);
              cl.Add("file_out_comp="+f_tmp2,         true, true);
              cl.Add("runmode=1",                     true, true);
              cl.Add("run",                           true, true);
              cl.Add("terminate",                    false, true);
            } 

            ret=cl.Send();
            if (ret==-1)
            {
              out.println("Error: Can't find host with Compilation Server!");
              out.flush();
              return;
            }
            else if (ret==-2)
            {
              out.println("Error: Can't connect to Compilation Server!");
              out.flush();
              return;
            }
            else if (ret==-4)
            {
              out.println("Error: Can't close connection with Compilation Server!");
              out.flush();
              return;
            }
            else if (ret<0)
            {
              out.println(cl.GetLastError());
              out.flush();
              return;
            }
    
            fout.write("Ok");
            fout.newLine();

            if (asm_id==0)
            {
              fout.write("Running (cycle profiling) 4b    : ");
              fout.flush();
 
              str1=str_dir+f_mpc_fb2a;
              BufferedWriter fout1=null;

              try
              {
                fout1=new BufferedWriter(new FileWriter(str1));
              }
              catch (IOException e2)
              {
                fout.write("Can't open _eos_mpc_fb2.dat for writing!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                for (i=0; i<nf; i++)
                {
                  for (j=0; j<LM[i]; j++)
                  {
                    fout1.write(Ind[i][j]+"_");
                    fout1.newLine();
                    fout1.write(Ind[i][j]+"_tr_");
                    fout1.newLine();
                  }
                }
              }
              catch (Exception e1)
              {
                try
                {
                  fout1.close();
                }
                catch (Exception e2)
                {}
                fout.write("Error reading original file!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                fout1.close();
              }
              catch (Exception e3)
              {}

              cl.Add("initiate compilation",          true, true);
              cl.Add("directory="+str_dir,            true, true);
              cl.Add("file_project="+str_fp,          true, true);
              cl.Add("tool=2",                        true, true);
              cl.Add("run_tool",                      true, true);
              cl.Add("terminate",                    false, true);

              ret=cl.Send();
              if (ret==-1)
              {
                out.println("Error: Can't find host with Compilation Server!");
                out.flush();
                return;
              }
              else if (ret==-2)
              {
                out.println("Error: Can't connect to Compilation Server!");
                out.flush();
                return;
              }
              else if (ret==-4)
              {
                out.println("Error: Can't close connection with Compilation Server!");
                out.flush();
                return;
              }
              else if (ret<0)
              {
                out.println(cl.GetLastError());
                out.flush();
                return;
              }

              cl.Add("initiate compilation",          true, true);
              cl.Add("directory="+str_dir,            true, true);
              cl.Add("file_project="+str_fp,          true, true);
              cl.Add("file_out_time="+f_tmp1,         true, true);
              cl.Add("runmode=3",                     true, true);
              cl.Add("param1="+str_fi,                true, true);
              cl.Add("tool=1",                        true, true);
              cl.Add("run",                           true, true);
              cl.Add("terminate",                    false, true);
  
              ret=cl.Send();
              if (ret==-1)
              {
                out.println("Error: Can't find host with Compilation Server!");
                out.flush();
                return;
              }
              else if (ret==-2)
              {
                out.println("Error: Can't connect to Compilation Server!");
                out.flush();
                return;
              }
              else if (ret==-4)
              {
                out.println("Error: Can't close connection with Compilation Server!");
                out.flush();
                return;
              }
              else if (ret<0)
              {
                out.println(cl.GetLastError());
                out.flush();
                return;
              }

              if (CMisc.CopyTextFile(str_dir+f_mpc_fb2b, str_dir+str_fpf+"4b")!=0)
              {
                fout.write("Error: Can't copy file with instr. counts!");
                fout.newLine();
                fout.flush();
                return;
              }

              fout.write("Ok");
              fout.newLine();

              fout.write("Parsing profiling info    4d    : ");
              fout.flush();

              str1=str_dir+str_fpf+"4b";
              str2=str_dir+str_fpf+"4d";
 
              BufferedWriter fout2=null;

              try
              {
                  fin=new BufferedReader(new FileReader(str1));
                fout2=new BufferedWriter(new FileWriter(str2));
              }
              catch (IOException e2)
              {
                fout.write("Error opening original prof file!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                while ((str2=fin.readLine())!=null)
                {
                  String prc="";
         
                  i1=str2.indexOf("proc={");
                  if (i1!=-1)
                  {
                    i2=str2.indexOf("}", i1+1);
                    if (i2!=-1)
                    {
                      prc=str2.substring(i1+6, i2).trim();
                    }
                  }
  
                  String instr="";
  
                  i1=str2.indexOf("instr={");
                  if (i1!=-1)
                  {
                    i2=str2.indexOf("}", i1+1);
                    if (i2!=-1)
                    {
                      instr=str2.substring(i1+7, i2).trim();
                    }
                  }
  
                  long instr1=0;
                  try
                  {
                    instr1=Long.valueOf(instr).longValue();
                  }
                  catch (NumberFormatException nfe1)
                  {}
 
                  boolean found=false;
                  String tm="";
                  String cyc="";
                  str1=str_dir+str_fpf+"4a";

                  try
                  {
                    fin1=new BufferedReader(new FileReader(str1));
                  }
                  catch (IOException e2)
                  {
                    fout.write("Error opening original prof file!");
                    fout.newLine();
                    fout.flush();
                    return;
                  }

                  try
                  {
                    while (((str2=fin1.readLine())!=null) && !found)
                    {
                      String prc1="";
         
                      i1=str2.indexOf("'");
                      if (i1!=-1)
                      {
                        i2=str2.indexOf("'", i1+1);
                        if (i2!=-1)
                        {
                          prc1=str2.substring(i1+1, i2).trim()+"_";
                        }
                      }

                      if (prc.equals(prc1))
                      {
                        found=true;
  
                        i1=str2.indexOf("'", i2+1);
                        if (i1!=-1)
                        {
                          i2=str2.indexOf("'", i1+1);
                          if (i2!=-1)
                          {
                            tm=str2.substring(i1+1, i2).trim();;
                          }
                        }

                        i1=str2.indexOf("'", i2+1);
                        if (i1!=-1)
                        {
                          i2=str2.indexOf("'", i1+1);
                          if (i2!=-1)
                          {
                            cyc=str2.substring(i1+1, i2).trim();
                          }
                        }
                      }
                    }
                  }
                  catch (Exception e1)
                  {
                    try
                    {
                      fin1.close();
                    }
                    catch (Exception e2)
                    {}
                    fout.write("Error reading prof file!");
                    fout.newLine();
                    fout.flush();
                    return;
                  }

                  try
                  {
                    fin1.close();
                  }
                  catch (Exception e3)
                  {}

                  if (!found)
                  {
                    fout.write("Failed matching instructions in prof files!");
                    fout.newLine();
                    fout.flush();
                    return;
                  }

                  long cyc1=0;
                  try
                  {
                    cyc1=Long.valueOf(cyc).longValue();
                  }
                  catch (NumberFormatException nfe1)
                  {}

                  long ipc1=instr1*100;
                  long ipc2=ipc1/cyc1;
                  double ipc3=ipc2;
                  double ipc=ipc3/100;
                
                  fout2.write("proc={"+prc+"}, time={"+tm+"}, instr={"+
                              instr+"}, cycles={"+cyc+"}, IPC={"+
                              String.valueOf(ipc)+"}");
                  fout2.newLine();
                }
              }
              catch (Exception e1)
              {
                try
                {
                  fin.close();
                  fout2.close();
                }
                catch (Exception e2)
                {}
                fout.write("Error reading prof file!");
                fout.newLine();
                fout.flush();
                return;
              }

              try
              {
                fin.close();
                fout2.close();
              }
              catch (Exception e3)
              {}

              fout.write("Ok");
              fout.newLine();
            }
          }
        }
      }
 
      fout.write("*****************************************************************************");
      fout.newLine();
    }
    catch (Exception e2)
    {
      out.println("Error: Problems while writing diagnostic files!");
      out.flush();
      return;
    }
 
    out.println("Ok");
    out.flush();
  }

  //*************************************************************************
  //-1 - unknown
  // 0 - Load
  // 1 - Fload
  // 2 - Store
  // 3 - Fstore

  private static int GetInstClassIntelWin(String str)
  {
    int nfop1=18;
    String[] fop1=new String [nfop1];
    fop1[ 0]="fld";
    fop1[ 1]="fild";
    fop1[ 2]="fbld";
    fop1[ 3]="fcom";
    fop1[ 4]="fcomp";
    fop1[ 5]="ficom";
    fop1[ 6]="ficomp";
    fop1[ 7]="fadd";
    fop1[ 8]="fiadd";
    fop1[ 9]="fsub";
    fop1[10]="fsubr";
    fop1[11]="fisub";
    fop1[12]="fisubr";
    fop1[13]="fmul";
    fop1[14]="fimul";
    fop1[15]="fdiv";
    fop1[16]="fidiv";
    fop1[17]="fidivr";
           
    int nfop2=6;
    String[] fop2=new String [nfop2];
    fop2[ 0]="fst";
    fop2[ 1]="fstp";
    fop2[ 2]="fist";
    fop2[ 3]="fistp";
    fop2[ 4]="fbst";
    fop2[ 5]="fbstp";

    int nop1=10;
    String[] op1=new String [nop1];
    op1[ 0]="inc";
    op1[ 1]="dec";
    op1[ 2]="neg";
    op1[ 3]="not";
    op1[ 4]="rcl";
    op1[ 5]="rcr";
    op1[ 6]="rol";
    op1[ 7]="ror";
    op1[ 8]="sal";
    op1[ 9]="sar";

    int nop2=5;
    String[] op2=new String [nop2];
    op2[ 0]="cmp";
    op2[ 1]="mul";
    op2[ 2]="imul";
    op2[ 3]="div";
    op2[ 4]="idiv";

    int nop3=9;
    String[] op3=new String [nop3];
    op3[ 0]="mov";
    op3[ 1]="add";
    op3[ 2]="adc";
    op3[ 3]="sub";
    op3[ 4]="sbb";
    op3[ 5]="and";
    op3[ 6]="test";
    op3[ 7]="or";
    op3[ 8]="xor";
    
    int i1=str.indexOf(" PTR ");
    int i2=0;
    String op="";

    if (i1!=-1)
    {
      str=str.trim();

      i2=str.indexOf(" ");
      op=str.substring(0,i2);

      if (str.charAt(0)=='f')
      {
        for (int i=0; i<nfop1; i++)
        {
          if (fop1[i].equals(op)) return 1;
        }

        for (int i=0; i<nfop2; i++)
        {
          if (fop2[i].equals(op)) return 3;
        }
      }
      else
      {
        for (int i=0; i<nop1; i++)
        {
          if (op1[i].equals(op)) return 2;
        }

        for (int i=0; i<nop2; i++)
        {
          if (op2[i].equals(op)) return 0;
        }

        for (int i=0; i<nop3; i++)
        {
          if (op3[i].equals(op))
          {
            if (str.indexOf(",", i1+1)!=-1) return 2;
            return 0;
          }
        }
      }
    }

    return -1;
  }
}
   